@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * main.c
- * Copyright (C) John Stebbins 2008-2016 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
  *
  * main.c is free software.
  *
@@ -40,7 +40,6 @@
 
 #if !defined(_WIN32)
 #include <libnotify/notify.h>
-#include <dbus/dbus-glib.h>
 #else
 #include <windows.h>
 #include <io.h>
@@ -66,7 +65,7 @@
 #include "resources.h"
 #include "presets.h"
 #include "preview.h"
-#include "ghbcompositor.h"
+#include "ui_res.h"
 
 
 /*
@@ -89,9 +88,15 @@ GtkBuilder*
 create_builder_or_die(const gchar * name)
 {
     guint res = 0;
-    GValue *gval;
     GError *error = NULL;
     const gchar *ghb_ui;
+    gsize data_size;
+
+    GResource *ui_res = ghb_ui_get_resource();
+    GBytes *gbytes = g_resource_lookup_data(ui_res,
+                                            "/org/handbrake/ghb/ui/ghb.ui",
+                                            0, NULL);
+    ghb_ui = g_bytes_get_data(gbytes, &data_size);
 
     const gchar *markup =
         N_("<b><big>Unable to create %s.</big></b>\n"
@@ -100,8 +105,6 @@ create_builder_or_die(const gchar * name)
         "%s");
     g_debug("create_builder_or_die()\n");
     GtkBuilder *xml = gtk_builder_new();
-    gval = ghb_resource_get("ghb-ui");
-    ghb_ui = g_value_get_string(gval);
     if (xml != NULL)
         res = gtk_builder_add_from_string(xml, ghb_ui, -1, &error);
     if (!xml || !res)
@@ -327,7 +330,7 @@ bind_audio_tree_model(signal_user_data_t *ud)
     GtkTreeSelection *selection;
 
     g_debug("bind_audio_tree_model()\n");
-    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "audio_list"));
+    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "audio_list_view"));
     selection = gtk_tree_view_get_selection(treeview);
     treestore = gtk_tree_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING,
@@ -392,7 +395,7 @@ bind_subtitle_tree_model(signal_user_data_t *ud)
     GtkTreeSelection *selection;
 
     g_debug("bind_subtitle_tree_model()\n");
-    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "subtitle_list"));
+    treeview = GTK_TREE_VIEW(GHB_WIDGET(ud->builder, "subtitle_list_view"));
     selection = gtk_tree_view_get_selection(treeview);
     treestore = gtk_tree_store_new(6, G_TYPE_STRING, G_TYPE_STRING,
                                       G_TYPE_STRING, G_TYPE_STRING,
@@ -627,19 +630,26 @@ IoRedirect(signal_user_data_t *ud)
     // Set encoding to raw.
     g_io_channel_set_encoding(ud->activity_log, NULL, NULL);
     // redirect stderr to the writer end of the pipe
+
 #if defined(_WIN32)
-    // dup2 doesn't work on windows for some stupid reason
-    stderr->_file = pfd[1];
+    _dup2(pfd[1], _fileno(stderr));
 #else
-    dup2(pfd[1], /*stderr*/2);
+    dup2(pfd[1], STDERR_FILENO);
 #endif
     setvbuf(stderr, NULL, _IONBF, 0);
+
+#if defined(_WIN32)
+    channel = g_io_channel_win32_new_fd(pfd[0]);
+#else
     channel = g_io_channel_unix_new(pfd[0]);
+#endif
+
     // I was getting an this error:
     // "Invalid byte sequence in conversion input"
     // Set disable encoding on the channel.
     g_io_channel_set_encoding(channel, NULL, NULL);
-    g_io_add_watch(channel, G_IO_IN, ghb_log_cb, (gpointer)ud );
+    ud->stderr_src_id =
+        g_io_add_watch(channel, G_IO_IN, ghb_log_cb, (gpointer)ud );
 }
 
 typedef struct
@@ -651,12 +661,20 @@ typedef struct
 static gchar *dvd_device = NULL;
 static gchar *arg_preset = NULL;
 static gboolean ghb_debug = FALSE;
+static gchar *arg_config_dir = NULL;
+#if defined(_WIN32)
+static gboolean win32_console = FALSE;
+#endif
 
 static GOptionEntry entries[] =
 {
     { "device", 'd', 0, G_OPTION_ARG_FILENAME, &dvd_device, N_("The device or file to encode"), NULL },
     { "preset", 'p', 0, G_OPTION_ARG_STRING, &arg_preset, N_("The preset values to use for encoding"), NULL },
     { "debug",  'x', 0, G_OPTION_ARG_NONE, &ghb_debug, N_("Spam a lot"), NULL },
+    { "config", 'o', 0, G_OPTION_ARG_STRING, &arg_config_dir, N_("The path to override user config dir"), NULL },
+#if defined(_WIN32)
+    { "console",'c', 0, G_OPTION_ARG_NONE, &win32_console, N_("Open a console for debug output"), NULL },
+#endif
     { NULL }
 };
 
@@ -697,26 +715,14 @@ watch_volumes(signal_user_data_t *ud)
 
 G_MODULE_EXPORT void x264_entry_changed_cb(GtkWidget *widget, signal_user_data_t *ud);
 G_MODULE_EXPORT void video_option_changed_cb(GtkWidget *widget, signal_user_data_t *ud);
+G_MODULE_EXPORT void plot_changed_cb(GtkWidget *widget, signal_user_data_t *ud);
 G_MODULE_EXPORT void position_overlay_cb(GtkWidget *widget, signal_user_data_t *ud);
 G_MODULE_EXPORT void preview_hud_size_alloc_cb(GtkWidget *widget, signal_user_data_t *ud);
 
+// Important: any widgets named in CSS must have their widget names set
+// below before setting CSS properties on them.
 const gchar *MyCSS =
 "                                   \n\
-GtkRadioButton .button {            \n\
-    border-width: 0px;              \n\
-    padding: 0px;                   \n\
-}                                   \n\
-GtkComboBox {                       \n\
-    padding: 0px;                   \n\
-}                                   \n\
-GtkComboBox .button {               \n\
-    padding: 2px;                   \n\
-    border-width: 0px;              \n\
-}                                   \n\
-GtkEntry {                          \n\
-    padding: 4px 4px;               \n\
-}                                   \n\
-                                    \n\
 @define-color black  #000000;       \n\
 @define-color gray18 #2e2e2e;       \n\
 @define-color gray22 #383838;       \n\
@@ -726,10 +732,16 @@ GtkEntry {                          \n\
 @define-color gray46 #757575;       \n\
 @define-color white  #ffffff;       \n\
                                     \n\
-#preview_hud,                       \n\
+#preview_hud                        \n\
+{                                   \n\
+    border-radius: 20px;            \n\
+    background-color: alpha(@gray18, 0.8); \n\
+    color: @white;                  \n\
+}                                   \n\
+                                    \n\
 #live_preview_play,                 \n\
 #live_duration,                     \n\
-#preview_fullscreen                 \n\
+#preview_reset                      \n\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray18;      \n\
@@ -748,72 +760,216 @@ GtkEntry {                          \n\
 #preview_frame                      \n\
 {                                   \n\
     background: @black;             \n\
-    background-color: @gray46;      \n\
+    background-color: alpha(@gray18, 0.0); \n\
     color: @white;                  \n\
 }                                   \n\
                                     \n\
-#preview_fullscreen:prelight        \n\
+"
+#if GTK_CHECK_VERSION(3, 20, 0)
+"\
+#preview_reset:hover                \n\
+"
+#else
+"\
+#preview_reset:prelight             \n\
+"
+#endif
+"\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray32;      \n\
     color: @white;                  \n\
 }                                   \n\
                                     \n\
-#preview_fullscreen:active          \n\
+#preview_reset:active               \n\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray32;      \n\
     color: @white;                  \n\
 }                                   \n\
                                     \n\
-#preview_fullscreen:active          \n\
+#preview_reset:active               \n\
 {                                   \n\
     background: @black;             \n\
     background-color: @gray32;      \n\
     color: @white;                  \n\
 }                                   \n\
-";
+                                    \n\
+entry                               \n\
+{                                   \n\
+    margin: 0px 0px 0px 0px;        \n\
+    padding: 0px 0px 0px 0px;       \n\
+}                                   \n\
+"
+#if GTK_CHECK_VERSION(3, 20, 0)
+"                                   \n\
+stackswitcher button.text-button    \n\
+{                                   \n\
+    min-width: 50px;                \n\
+}                                   \n\
+"
+#endif
+#if GTK_CHECK_VERSION(3, 16, 0)
+"                                   \n\
+                                    \n\
+#activity_view                      \n\
+{                                   \n\
+    font-family: monospace;         \n\
+    font-size: 7pt;                 \n\
+    font-weight: lighter;           \n\
+}                                   \n\
+"
+#endif
+;
 
 extern G_MODULE_EXPORT void status_icon_query_tooltip_cb(void);
 
-int
-main(int argc, char *argv[])
+extern G_MODULE_EXPORT void
+ghb_shutdown_cb(GApplication * app, signal_user_data_t *ud)
 {
-    signal_user_data_t *ud;
-    GValue *preset;
-    GError *error = NULL;
-    GOptionContext *context;
+}
 
-    hb_global_init();
+G_MODULE_EXPORT void
+source_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+single_title_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+destination_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preferences_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+quit_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+queue_add_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+queue_add_all_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+queue_start_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+queue_pause_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+queue_save_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+queue_open_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+show_presets_action_cb(GSimpleAction *action, GVariant *value, gpointer ud);
+G_MODULE_EXPORT void
+hbfd_action_cb(GSimpleAction *action, GVariant *value, gpointer ud);
+G_MODULE_EXPORT void
+show_queue_action_cb(GSimpleAction *action, GVariant *value, gpointer ud);
+G_MODULE_EXPORT void
+show_preview_action_cb(GSimpleAction *action, GVariant *value, gpointer ud);
+G_MODULE_EXPORT void
+show_activity_action_cb(GSimpleAction *action, GVariant *value, gpointer ud);
+G_MODULE_EXPORT void
+preset_save_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_save_as_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_rename_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_remove_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_default_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_export_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_import_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+presets_reload_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+about_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+guide_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_select_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
+G_MODULE_EXPORT void
+preset_reload_action_cb(GSimpleAction *action, GVariant *param, gpointer ud);
 
-#ifdef ENABLE_NLS
-    bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
-    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-    textdomain(GETTEXT_PACKAGE);
-#endif
-
-    context = g_option_context_new(_("- Transcode media formats"));
-    g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
-    g_option_context_add_group(context, gtk_get_option_group(TRUE));
-#if defined(_ENABLE_GST)
-    g_option_context_add_group(context, gst_init_get_option_group());
-#endif
-    g_option_context_parse(context, &argc, &argv, &error);
-    if (error != NULL)
+static void map_actions(GApplication * app, signal_user_data_t * ud)
+{
+    const GActionEntry entries[] =
     {
-        g_warning("%s: %s", G_STRFUNC, error->message);
-        g_clear_error(&error);
+        { "source",         source_action_cb                },
+        { "single",         single_title_action_cb          },
+        { "destination",    destination_action_cb           },
+        { "preferences",    preferences_action_cb           },
+        { "quit",           quit_action_cb                  },
+        { "queue-add",      queue_add_action_cb             },
+        { "queue-add-all",  queue_add_all_action_cb         },
+        { "queue-start",    queue_start_action_cb           },
+        { "queue-pause",    queue_pause_action_cb           },
+        { "queue-save",     queue_save_action_cb            },
+        { "queue-open",     queue_open_action_cb            },
+        { "hbfd",           NULL,
+          NULL, "false",    hbfd_action_cb                  },
+        { "show-presets",   NULL,
+          NULL, "false",    show_presets_action_cb          },
+        { "show-queue",     NULL,
+          NULL, "false",    show_queue_action_cb            },
+        { "show-preview",   NULL,
+          NULL, "false",    show_preview_action_cb          },
+        { "show-activity",  NULL,
+          NULL, "false",    show_activity_action_cb         },
+        { "preset-save",    preset_save_action_cb           },
+        { "preset-save-as", preset_save_as_action_cb        },
+        { "preset-rename",  preset_rename_action_cb         },
+        { "preset-remove",  preset_remove_action_cb         },
+        { "preset-default", preset_default_action_cb        },
+        { "preset-export",  preset_export_action_cb         },
+        { "preset-import",  preset_import_action_cb         },
+        { "presets-reload", presets_reload_action_cb        },
+        { "about",          about_action_cb                 },
+        { "guide",          guide_action_cb                 },
+        { "preset-select",  preset_select_action_cb, "s"    },
+        { "preset-reload",  preset_reload_action_cb,        },
+    };
+    g_action_map_add_action_entries(G_ACTION_MAP(app), entries,
+                                    G_N_ELEMENTS(entries), ud);
+}
+
+gboolean
+ghb_idle_ui_init(signal_user_data_t *ud)
+{
+    ghb_settings_to_ui(ud, ud->globals);
+    ghb_settings_to_ui(ud, ud->prefs);
+    // Note that ghb_settings_to_ui(ud->settings) happens when initial
+    // empty title is initialized.
+
+    // Init settings that are dependent on command line args
+    if (dvd_device != NULL)
+    {
+        // Source overridden from command line option
+        ghb_dict_set_string(ud->globals, "scan_source", dvd_device);
+        g_idle_add((GSourceFunc)ghb_idle_scan, ud);
     }
-    g_option_context_free(context);
-
-    if (argc > 1 && dvd_device == NULL && argv[1][0] != '-')
+    else
     {
-        dvd_device = argv[1];
+        GhbValue *gval = ghb_dict_get_value(ud->prefs, "default_source");
+        ghb_dict_set(ud->globals, "scan_source", ghb_value_dup(gval));
     }
 
-    gtk_init(&argc, &argv);
+    if (arg_preset != NULL)
+    {
+        ghb_select_preset(ud, arg_preset, HB_PRESET_TYPE_ALL);
+    }
+    else
+    {
+        ghb_select_default_preset(ud);
+    }
+
+    // Grey out widgets that are dependent on a disabled feature
+    ghb_check_all_depencencies(ud);
+
+    return FALSE;
+}
+
+extern G_MODULE_EXPORT void
+ghb_activate_cb(GApplication * app, signal_user_data_t * ud)
+{
+    GError             * error = NULL;
+    GtkCssProvider     * css = gtk_css_provider_new();
 
-    GtkCssProvider *css = gtk_css_provider_new();
     error = NULL;
     gtk_css_provider_load_from_data(css, MyCSS, -1, &error);
     if (error == NULL)
@@ -831,39 +987,42 @@ main(int argc, char *argv[])
 #if !defined(_WIN32)
     notify_init("HandBrake");
 #endif
-    ghb_register_transforms();
     ghb_resource_init();
     ghb_load_icons();
 
-#if !defined(_WIN32)
-    dbus_g_thread_init();
-#endif
     ghb_udev_init();
 
+    // Override user config dir
+    if (arg_config_dir != NULL)
+    {
+        ghb_override_user_config_dir(arg_config_dir);
+    }
+
     ghb_write_pid_file();
-    ud = g_malloc0(sizeof(signal_user_data_t));
+
+    // map application actions (menu callbacks)
+    map_actions(app, ud);
+
+    // connect shutdown signal for cleanup
+    g_signal_connect(app, "shutdown", (GCallback)ghb_shutdown_cb, ud);
+
     ud->debug = ghb_debug;
     g_log_set_handler(NULL, G_LOG_LEVEL_DEBUG, debug_log_handler, ud);
     g_log_set_handler("Gtk", G_LOG_LEVEL_WARNING, warn_log_handler, ud);
-    //g_log_set_handler("Gtk", G_LOG_LEVEL_CRITICAL, warn_log_handler, ud);
 
-    ud->globals = ghb_settings_new();
-    ud->prefs = ghb_settings_new();
-    ud->settings_array = ghb_array_value_new(1);
-    ud->settings = ghb_settings_new();
+    ud->globals = ghb_dict_new();
+    ud->prefs = ghb_dict_new();
+    ud->settings_array = ghb_array_new();
+    ud->settings = ghb_dict_new();
     ghb_array_append(ud->settings_array, ud->settings);
 
     ud->builder = create_builder_or_die(BUILDER_NAME);
+
     // Enable events that alert us to media change events
     watch_volumes(ud);
 
-    //GtkWidget *widget = GHB_WIDGET(ud->builder, "PictureDetelecineCustom");
-    //gtk_entry_set_inner_border(widget, 2);
-
-    // Since GtkBuilder no longer assigns object ids to widget names
-    // Assign a few that are necessary for style overrides to work
-    GtkWidget *widget;
 #if defined(_NO_UPDATE_CHECK)
+    GtkWidget *widget;
     widget = GHB_WIDGET(ud->builder, "check_updates_box");
     gtk_widget_hide(widget);
 #endif
@@ -877,52 +1036,8 @@ main(int argc, char *argv[])
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_encode_progress"), "live_encode_progress");
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "live_duration"), "live_duration");
     gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_show_crop"), "preview_show_crop");
-    gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_fullscreen"), "preview_fullscreen");
-    widget = GHB_WIDGET(ud->builder, "preview_hud");
-    gtk_widget_set_name(widget, "preview_hud");
-    widget = GHB_WIDGET(ud->builder, "preview_window");
-    gtk_widget_set_name(widget, "preview_window");
-
-    // Set up the "hud" control overlay for the preview window
-    GtkWidget *preview_box, *draw, *hud, *blender;
-
-    preview_box = GHB_WIDGET(ud->builder, "preview_window_box");
-    draw = GHB_WIDGET(ud->builder, "preview_image");
-    hud = GHB_WIDGET(ud->builder, "preview_hud");
-
-#if 0 // GTK_CHECK_VERSION(3, 0, 0)
-    // This uses the new GtkOverlay widget.
-    //
-    // Unfortunately, GtkOverlay is broken in a couple of ways.
-    //
-    // First, it doesn't respect gtk_widget_shape_combine_region()
-    // on it's child overlays.  It appears to just ignore the clip
-    // mask of the child.
-    //
-    // Second, it doesn't respect window opacity.
-    //
-    // So for now, I'll just continue using my home-grown overlay
-    // widget (GhbCompositor).
-    blender = gtk_overlay_new();
-    gtk_container_add(GTK_CONTAINER(preview_box), blender);
-    gtk_container_add(GTK_CONTAINER(blender), draw);
-    gtk_widget_set_valign (hud, GTK_ALIGN_END);
-    gtk_widget_set_halign (hud, GTK_ALIGN_CENTER);
-    gtk_overlay_add_overlay(GTK_OVERLAY(blender), hud);
-
-    g_signal_connect(G_OBJECT(blender), "get-child-position",
-                    G_CALLBACK(position_overlay_cb), ud);
-
-    gtk_widget_show(blender);
-#else
-    // Set up compositing for hud
-    blender = ghb_compositor_new();
-
-    gtk_container_add(GTK_CONTAINER(preview_box), blender);
-    ghb_compositor_zlist_insert(GHB_COMPOSITOR(blender), draw, 1, 1);
-    ghb_compositor_zlist_insert(GHB_COMPOSITOR(blender), hud, 2, .85);
-    gtk_widget_show(blender);
-#endif
+    gtk_widget_set_name(GHB_WIDGET(ud->builder, "preview_reset"), "preview_reset");
+    gtk_widget_set_name(GHB_WIDGET(ud->builder, "activity_view"), "activity_view");
 
     // Redirect stderr to the activity window
     ghb_preview_init(ud);
@@ -943,6 +1058,13 @@ main(int argc, char *argv[])
     buffer = gtk_text_view_get_buffer(textview);
     g_signal_connect(buffer, "changed", (GCallback)video_option_changed_cb, ud);
 
+    textview = GTK_TEXT_VIEW(GHB_WIDGET(ud->builder, "MetaLongDescription"));
+    buffer = gtk_text_view_get_buffer(textview);
+    g_signal_connect(buffer, "changed", (GCallback)plot_changed_cb, ud);
+
+    // Initialize HB global settings and tables.
+    ghb_backend_init(0);
+    // Set up UI combo boxes.  Some of these rely on HB global settings.
     ghb_combo_init(ud);
 
     g_debug("ud %p\n", ud);
@@ -970,66 +1092,30 @@ main(int argc, char *argv[])
     ghb_settings_init(ud->prefs, "Preferences");
     ghb_settings_init(ud->globals, "Globals");
     ghb_settings_init(ud->settings, "Initialization");
+    ghb_settings_init(ud->settings, "OneTimeInitialization");
     // Load user preferences file
     ghb_prefs_load(ud);
     // Store user preferences into ud->prefs
     ghb_prefs_to_settings(ud->prefs);
 
-    // Load all settings with default preset values
-    ghb_settings_init(ud->settings, "Presets");
+    int logLevel = ghb_dict_get_int(ud->prefs, "LoggingLevel");
+    ghb_log_level_set(logLevel);
+
     // Load the presets files
     ghb_presets_load(ud);
-    // Note that ghb_preset_to_settings(ud->settings) is called when
-    // the default preset is selected.
 
-    ghb_settings_to_ui(ud, ud->globals);
-    ghb_settings_to_ui(ud, ud->prefs);
-    // Note that ghb_settings_to_ui(ud->settings) happens when initial
-    // empty title is initialized.
+    // GActions associated with widgets do not fire when the widget
+    // is changed from this GtkApplication "activate" signal.
+    // So initialize UI when idle.
+    g_idle_add((GSourceFunc)ghb_idle_ui_init, ud);
 
-    gint logLevel;
-    logLevel = ghb_settings_get_int(ud->prefs, "LoggingLevel");
-    ghb_backend_init(logLevel);
-
-    if (ghb_settings_get_boolean(ud->prefs, "hbfd"))
-    {
-        ghb_hbfd(ud, TRUE);
-    }
-    gchar *source = ghb_settings_get_string(ud->prefs, "default_source");
+    const gchar *source = ghb_dict_get_string(ud->prefs, "default_source");
     ghb_dvd_set_current(source, ud);
-    g_free(source);
 
     // Populate the presets tree view
-    ghb_presets_list_init(ud, NULL, 0);
-    // Get the first preset name
-    if (arg_preset != NULL)
-    {
-        preset = ghb_parse_preset_path(arg_preset);
-        if (preset)
-        {
-            ghb_select_preset(ud->builder, preset);
-            ghb_value_free(preset);
-        }
-    }
-    else
-    {
-        ghb_select_default_preset(ud->builder);
-    }
+    ghb_presets_list_init(ud, NULL);
+    ghb_presets_menu_init(ud);
 
-    // Grey out widgets that are dependent on a disabled feature
-    ghb_check_all_depencencies(ud);
-
-    if (dvd_device != NULL)
-    {
-        // Source overridden from command line option
-        ghb_settings_set_string(ud->globals, "scan_source", dvd_device);
-        g_idle_add((GSourceFunc)ghb_idle_scan, ud);
-    }
-    else
-    {
-        GValue *gval = ghb_settings_get_value(ud->prefs, "default_source");
-        ghb_settings_set_value(ud->globals, "scan_source", gval);
-    }
     // Reload and check status of the last saved queue
     g_idle_add((GSourceFunc)ghb_reload_queue, ud);
 
@@ -1044,14 +1130,14 @@ main(int argc, char *argv[])
 
     gint window_width, window_height;
     GdkGeometry geo = {
-        -1, -1, 1920, 768, -1, -1, 10, 10, 0, 0, GDK_GRAVITY_NORTH_WEST
+        -1, -1, 1920, 1080, -1, -1, 10, 10, 0, 0, GDK_GRAVITY_NORTH_WEST
     };
     GdkWindowHints geo_mask;
     geo_mask = GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE | GDK_HINT_BASE_SIZE;
     gtk_window_set_geometry_hints(GTK_WINDOW(ghb_window), ghb_window,
                                   &geo, geo_mask);
-    window_width = ghb_settings_get_int(ud->prefs, "window_width");
-    window_height = ghb_settings_get_int(ud->prefs, "window_height");
+    window_width = ghb_dict_get_int(ud->prefs, "window_width");
+    window_height = ghb_dict_get_int(ud->prefs, "window_height");
 
     /*
      * Filter objects in GtkBuilder xml
@@ -1156,11 +1242,13 @@ main(int argc, char *argv[])
     filter = GTK_FILE_FILTER(GHB_OBJECT(ud->builder, "SourceFilterAll"));
     gtk_file_chooser_set_filter(chooser, filter);
 
+#if !GTK_CHECK_VERSION(3, 16, 0)
     PangoFontDescription *font_desc;
     font_desc = pango_font_description_from_string("monospace 10");
     textview = GTK_TEXT_VIEW(GHB_WIDGET(ud->builder, "activity_view"));
     gtk_widget_override_font(GTK_WIDGET(textview), font_desc);
     pango_font_description_free(font_desc);
+#endif
 
     // Grrrr!  Gtk developers !!!hard coded!!! the width of the
     // radio buttons in GtkStackSwitcher to 100!!!
@@ -1182,30 +1270,113 @@ main(int argc, char *argv[])
     g_list_free(stack_switcher_children);
 
     gtk_window_resize(GTK_WINDOW(ghb_window), window_width, window_height);
+
+    ghb_set_custom_filter_tooltip(ud, "PictureDetelecineCustom",
+                                  "detelecine", HB_FILTER_DETELECINE);
+    ghb_set_custom_filter_tooltip(ud, "PictureCombDetectCustom",
+                                  "interlace detection", HB_FILTER_COMB_DETECT);
+
+    gtk_application_add_window(GTK_APPLICATION(app), GTK_WINDOW(ghb_window));
+    GtkWidget * window = GHB_WIDGET(ud->builder, "presets_window");
+    gtk_application_add_window(GTK_APPLICATION(app), GTK_WINDOW(window));
+
     gtk_widget_show(ghb_window);
+}
 
-    // Everything should be go-to-go.  Lets rock!
+extern G_MODULE_EXPORT void
+ghb_open_file_cb(GApplication *app, gpointer pfiles, gint nfiles,
+                 gchar *hint, signal_user_data_t * ud)
+{
+    GFile ** files = pfiles;
+
+    if (nfiles < 1)
+        return;
+
+    if (dvd_device == NULL)
+    {
+        dvd_device = g_file_get_path(files[0]);
+    }
+    ghb_activate_cb(app, ud);
+}
+
+int
+main(int argc, char *argv[])
+{
+#if defined(_WIN32)
+    // Tell gdk pixbuf where it's loader config file is.
+    _putenv_s("GDK_PIXBUF_MODULE_FILE", "ghb.exe.local/loaders.cache");
+    _putenv_s("GST_PLUGIN_PATH", "lib/gstreamer-1.0");
+#endif
+
+
+#ifdef ENABLE_NLS
+    bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+    textdomain(GETTEXT_PACKAGE);
+#endif
+
+#if defined(_WIN32)
+    if (win32_console)
+    {
+        // Enable console logging
+        if(AttachConsole(ATTACH_PARENT_PROCESS) || AllocConsole()){
+            close(STDOUT_FILENO);
+            freopen("CONOUT$", "w", stdout);
+            close(STDERR_FILENO);
+            freopen("CONOUT$", "w", stderr);
+        }
+    }
+    else
+    {
+        // Non-console windows apps do not have a stderr->_file
+        // assigned properly
+        stderr->_file = STDERR_FILENO;
+        stdout->_file = STDOUT_FILENO;
+    }
+#endif
+
+    int                  status;
+    signal_user_data_t * ud;
+
+    ghb_ui_register_resource();
+    ud = g_malloc0(sizeof(signal_user_data_t));
+    ud->app = gtk_application_new("org.handbrake.ghb",
+                                  G_APPLICATION_HANDLES_OPEN);
+    // Connect application signals
+    g_signal_connect(ud->app, "activate", (GCallback)ghb_activate_cb, ud);
+    g_signal_connect(ud->app, "open", (GCallback)ghb_open_file_cb, ud);
+    // Set application options
+    g_application_add_main_option_entries(G_APPLICATION(ud->app), entries);
+#if defined(_ENABLE_GST)
+    g_application_add_option_group(G_APPLICATION(ud->app),
+                                   gst_init_get_option_group());
+#endif
+    status = g_application_run(G_APPLICATION(ud->app), argc, argv);
 
-    gtk_main();
     ghb_backend_close();
 
-    ghb_value_free(ud->queue);
-    ghb_value_free(ud->settings_array);
-    ghb_value_free(ud->prefs);
-    ghb_value_free(ud->globals);
-    ghb_value_free(ud->x264_priv);
+    // Remove stderr redirection
+    if (ud->stderr_src_id > 0)
+        g_source_remove(ud->stderr_src_id);
+    ghb_value_free(&ud->queue);
+    ghb_value_free(&ud->settings_array);
+    ghb_value_free(&ud->prefs);
+    ghb_value_free(&ud->globals);
+    ghb_value_free(&ud->x264_priv);
 
-    g_io_channel_unref(ud->activity_log);
+    if (ud->activity_log != NULL)
+        g_io_channel_unref(ud->activity_log);
     ghb_settings_close();
     ghb_resource_free();
 #if !defined(_WIN32)
     notify_uninit();
 #endif
 
-    g_object_unref(ud->builder);
+    if (ud->builder != NULL)
+        g_object_unref(ud->builder);
 
     g_free(ud->current_dvd_device);
     g_free(ud);
 
-    return 0;
+    return status;
 }
