@@ -1,6 +1,6 @@
 /*
  * plist.h
- * Copyright (C) John Stebbins 2008-2016 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
  *
  * plist.h is free software.
  *
@@ -27,11 +27,12 @@
 #include <stdio.h>
 #include <glib.h>
 #include <glib-object.h>
+#include "values.h"
 
-GValue* ghb_plist_parse(const gchar *buf, gssize len);
-GValue* ghb_plist_parse_file(const gchar *filename);
-void ghb_plist_write(FILE *file, GValue *gval);
-void ghb_plist_write_file(const gchar *filename, GValue *gval);
+GhbValue* ghb_plist_parse(const gchar *buf, gssize len);
+GhbValue* ghb_plist_parse_file(const gchar *filename);
+void ghb_plist_write(FILE *file, GhbValue *gval);
+void ghb_plist_write_file(const gchar *filename, GhbValue *gval);
 
 #endif // _PLIST_H_
 
