@@ -1,6 +1,6 @@
 /*
  * preview.h
- * Copyright (C) John Stebbins 2008-2016 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
  *
  * preview.h is free software.
  *
@@ -27,12 +27,14 @@
 #define GHB_PREVIEW_MAX 60
 
 void ghb_preview_init(signal_user_data_t *ud);
+void ghb_rescale_preview_image(signal_user_data_t *ud);
+void ghb_reset_preview_image(signal_user_data_t *ud);
 void ghb_set_preview_image(signal_user_data_t *ud);
 void ghb_live_preview_progress(signal_user_data_t *ud);
 void ghb_live_encode_done(signal_user_data_t *ud, gboolean success);
 void ghb_preview_cleanup(signal_user_data_t *ud);
 void ghb_live_reset(signal_user_data_t *ud);
 void ghb_par_scale(signal_user_data_t *ud, gint *width, gint *height, gint par_n, gint par_d);
-void ghb_preview_set_visible(signal_user_data_t *ud);
+void ghb_preview_set_visible(signal_user_data_t *ud, gboolean visible);
 
 #endif // _GHB_PREVIEW_H_
