@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * queuehandler.h
- * Copyright (C) John Stebbins 2008-2016 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
  *
  * queuehandler.h is free software.
  *
@@ -31,5 +31,8 @@
 void ghb_queue_buttons_grey(signal_user_data_t *ud);
 gboolean ghb_reload_queue(signal_user_data_t *ud);
 void ghb_queue_remove_row(signal_user_data_t *ud, int row);
+void ghb_finalize_job(GhbValue *settings);
+gint ghb_find_queue_job(GhbValue *queue, gint unique_id, GhbValue **job);
+void ghb_low_disk_check(signal_user_data_t *ud);
 
 #endif // _QUEUEHANDLER_H_
