@@ -1,6 +1,6 @@
 /*
  * resources.h
- * Copyright (C) John Stebbins 2008-2016 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
  *
  * resources.h is free software.
  *
@@ -24,8 +24,10 @@
 #if !defined(_RESOURCES_H_)
 #define _RESOURCES_H_
 
+#include "values.h"
+
 void ghb_resource_init(void);
 void ghb_resource_free();
-GValue* ghb_resource_get(const gchar *name);
+GhbValue* ghb_resource_get(const gchar *name);
 
 #endif // _RESOURCES_H_
