@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
  * videohandler.h
- * Copyright (C) John Stebbins 2008-2016 <stebbins@stebbins>
+ * Copyright (C) John Stebbins 2008-2018 <stebbins@stebbins>
  *
  * videohandler.h is free software.
  *
@@ -25,9 +25,11 @@
 #if !defined(_VIDEOHANDLER_H_)
 #define _VIDEOHANDLER_H_
 
+#include "values.h"
 #include "settings.h"
 
-int ghb_get_video_encoder(GValue *settings);
+int ghb_get_video_encoder(GhbValue *settings);
 void ghb_video_setting_changed(GtkWidget *widget, signal_user_data_t *ud);
+void ghb_set_video_preset(GhbValue *settings, int encoder, const char * preset);
 
 #endif // _VIDEOHANDLER_H_
