@@ -1,6 +1,6 @@
 /* dvd.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -23,6 +23,7 @@ struct hb_bd_s
     hb_stream_t  * stream;
     int            chapter;
     int            next_chap;
+    hb_handle_t  * h;
 };
 
 /***********************************************************************
@@ -36,12 +37,13 @@ static int title_info_compare_mpls(const void *, const void *);
  ***********************************************************************
  *
  **********************************************************************/
-hb_bd_t * hb_bd_init( char * path )
+hb_bd_t * hb_bd_init( hb_handle_t *h, char * path )
 {
     hb_bd_t * d;
     int ii;
 
     d = calloc( sizeof( hb_bd_t ), 1 );
+    d->h = h;
 
     /* Open device */
     d->bd = bd_open( path, NULL );
@@ -93,11 +95,6 @@ static void add_subtitle(int track, hb_list_t *list_subtitle, BLURAY_STREAM_INFO
 
     subtitle->track = track;
     subtitle->id = bdsub->pid;
-    lang = lang_for_code2( (char*)bdsub->lang );
-    snprintf( subtitle->lang, sizeof( subtitle->lang ), "%s",
-              strlen(lang->native_name) ? lang->native_name : lang->eng_name);
-    snprintf( subtitle->iso639_2, sizeof( subtitle->iso639_2 ), "%s",
-              lang->iso639_2);
 
     switch ( bdsub->coding_type )
     {
@@ -111,6 +108,13 @@ static void add_subtitle(int track, hb_list_t *list_subtitle, BLURAY_STREAM_INFO
             free( subtitle );
             return;
     }
+    lang = lang_for_code2( (char*)bdsub->lang );
+    snprintf(subtitle->lang, sizeof( subtitle->lang ), "%s [%s]",
+             strlen(lang->native_name) ? lang->native_name : lang->eng_name,
+             hb_subsource_name(subtitle->source));
+    snprintf(subtitle->iso639_2, sizeof( subtitle->iso639_2 ), "%s",
+             lang->iso639_2);
+
     subtitle->reg_desc = STR4_TO_UINT32("HDMV");
     subtitle->stream_type = bdsub->coding_type;
     subtitle->codec = codec;
@@ -187,7 +191,7 @@ static void add_audio(int track, hb_list_t *list_audio, BLURAY_STREAM_INFO *bdau
 
     lang = lang_for_code2( (char*)bdaudio->lang );
 
-    audio->config.lang.type = 0;
+    audio->config.lang.attributes = HB_AUDIO_ATTR_NONE;
 
     snprintf( audio->config.lang.simple, 
               sizeof( audio->config.lang.simple ), "%s",
@@ -232,6 +236,24 @@ static int bd_audio_equal( BLURAY_CLIP_INFO *a, BLURAY_CLIP_INFO *b )
     return 1;
 }
 
+static void show_clip_list( BLURAY_TITLE_INFO * ti )
+{
+    int ii;
+
+    for (ii = 0; ii < ti->clip_count; ii++)
+    {
+        BLURAY_CLIP_INFO * ci = &ti->clips[ii];
+        int64_t            duration = ci->out_time - ci->in_time;
+        int                hh, mm, ss;
+
+        hh = duration / (90000 * 60 * 60);
+        mm = (duration / (90000 * 60)) % 60;
+        ss = (duration / 90000) % 60;
+        hb_log("bd:\t\t%s.M2TS -- Duration: %02d:%02d:%02d",
+               ti->clips[ii].clip_id, hh, mm, ss);
+    }
+}
+
 /***********************************************************************
  * hb_bd_title_scan
  **********************************************************************/
@@ -266,7 +288,10 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
     title->vts = 0;
     title->ttn = 0;
 
-    ti = d->title_info[tt - 1];
+    if (tt <= d->title_count)
+    {
+        ti = d->title_info[tt - 1];
+    }
     if ( ti == NULL )
     {
         hb_log( "bd: invalid title" );
@@ -302,7 +327,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
     title->hours    = title->duration / 90000 / 3600;
     title->minutes  = ( ( title->duration / 90000 ) % 3600 ) / 60;
     title->seconds  = ( title->duration / 90000 ) % 60;
-    hb_log( "bd: duration is %02d:%02d:%02d (%"PRId64" ms)",
+    hb_log( "bd: duration is %02d:%02d:%02d (%"PRIu64" ms)",
             title->hours, title->minutes, title->seconds,
             title->duration / 90 );
 
@@ -312,6 +337,10 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
         hb_log( "bd: ignoring title (too short)" );
         goto fail;
     }
+    if (global_verbosity_level >= 2)
+    {
+        show_clip_list(ti);
+    }
 
     BLURAY_STREAM_INFO * bdvideo = &ti->clips[0].video_streams[0];
 
@@ -361,16 +390,19 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
     switch ( bdvideo->aspect )
     {
         case BLURAY_ASPECT_RATIO_4_3:
-            title->container_aspect = 4. / 3.;
+            title->container_dar.num = 4;
+            title->container_dar.den = 3;
             break;
         case BLURAY_ASPECT_RATIO_16_9:
-            title->container_aspect = 16. / 9.;
+            title->container_dar.num = 16;
+            title->container_dar.den = 9;
             break;
         default:
             hb_log( "bd: unknown aspect" );
             goto fail;
     }
-    hb_log( "bd: aspect = %g", title->container_aspect );
+    hb_log("bd: aspect = %d:%d",
+           title->container_dar.num, title->container_dar.den);
 
     /* Detect audio */
     // Max primary BD audios is 32
@@ -422,7 +454,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
                 add_audio(ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_AC3,
                           HB_ACODEC_AC3, AV_CODEC_ID_AC3);
                 add_audio(ii, title->list_audio, bdaudio, HB_SUBSTREAM_BD_TRUEHD,
-                          HB_ACODEC_FFMPEG, AV_CODEC_ID_TRUEHD);
+                          HB_ACODEC_FFTRUEHD, AV_CODEC_ID_TRUEHD);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_DTS:
@@ -438,7 +470,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
 
             case BLURAY_STREAM_TYPE_AUDIO_AC3PLUS:
                 add_audio(ii, title->list_audio, bdaudio, 0,
-                          HB_ACODEC_FFMPEG, AV_CODEC_ID_EAC3);
+                          HB_ACODEC_FFEAC3, AV_CODEC_ID_EAC3);
                 break;
 
             case BLURAY_STREAM_TYPE_AUDIO_LPCM:
@@ -498,7 +530,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
         // If it is beyond the end of the title, drop it.
         if (ti->chapters[ii].start > ti->duration)
         {
-            hb_log("bd: chapter %d invalid start %ld, dropping", ii+1,
+            hb_log("bd: chapter %d invalid start %"PRIu64", dropping", ii+1,
                    ti->chapters[ii].start);
             continue;
         }
@@ -519,7 +551,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
             // Validate start time
             if (ti->chapters[ii+1].start < ti->chapters[ii].start)
             {
-                hb_log("bd: chapter %d invalid start %ld", ii+1,
+                hb_log("bd: chapter %d invalid start %"PRIu64"", ii+1,
                        ti->chapters[ii+1].start);
                 ti->chapters[ii+1].start = ti->chapters[ii].start +
                                            chapter->duration;
@@ -527,7 +559,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
             if (ti->chapters[ii+1].start - ti->chapters[ii].start !=
                 chapter->duration)
             {
-                hb_log("bd: chapter %d invalid duration %ld", ii+1,
+                hb_log("bd: chapter %d invalid duration %"PRIu64"", ii+1,
                        chapter->duration);
                 chapter->duration = ti->chapters[ii+1].start -
                                     ti->chapters[ii].start;
@@ -537,7 +569,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
         {
             if (ti->duration - ti->chapters[ii].start != chapter->duration)
             {
-                hb_log("bd: chapter %d invalid duration %ld", ii+1,
+                hb_log("bd: chapter %d invalid duration %"PRIu64"", ii+1,
                        chapter->duration);
                 chapter->duration = ti->duration - ti->chapters[ii].start;
             }
@@ -548,7 +580,7 @@ hb_title_t * hb_bd_title_scan( hb_bd_t * d, int tt, uint64_t min_duration )
         chapter->minutes = ( seconds % 3600 ) / 60;
         chapter->seconds = ( seconds % 60 );
 
-        hb_log( "bd: chap %d packet=%"PRIu64", %"PRId64" ms",
+        hb_log( "bd: chap %d packet=%"PRIu64", %"PRIu64" ms",
                 chapter->index,
                 chapter->block_start,
                 chapter->duration / 90 );
@@ -631,8 +663,9 @@ int hb_bd_start( hb_bd_t * d, hb_title_t *title )
     // Calling bd_get_event initializes libbluray event queue.
     bd_select_title( d->bd, d->title_info[title->index - 1]->idx );
     bd_get_event( d->bd, &event );
-    d->chapter = 1;
-    d->stream = hb_bd_stream_open( title );
+    d->chapter = 0;
+    d->next_chap = 1;
+    d->stream = hb_bd_stream_open( d->h, title );
     if ( d->stream == NULL )
     {
         return 0;
@@ -693,17 +726,12 @@ hb_buffer_t * hb_bd_read( hb_bd_t * d )
     uint8_t buf[192];
     BD_EVENT event;
     uint64_t pos;
-    hb_buffer_t * b;
+    hb_buffer_t * out = NULL;
     uint8_t discontinuity;
-    int new_chap = 0;
 
-    discontinuity = 0;
     while ( 1 )
     {
-        if ( d->next_chap != d->chapter )
-        {
-            new_chap = d->chapter = d->next_chap;
-        }
+        discontinuity = 0;
         result = next_packet( d->bd, buf );
         if ( result < 0 )
         {
@@ -714,13 +742,14 @@ hb_buffer_t * hb_bd_read( hb_bd_t * d )
             if (error_count > 10)
             {
                 hb_error("bd: Error, too many consecutive read errors");
-                return 0;
+                hb_set_work_error(d->h, HB_ERROR_READ);
+                return NULL;
             }
             continue;
         }
         else if ( result == 0 )
         {
-            return 0;
+            return NULL;
         }
 
         error_count = 0;
@@ -731,7 +760,10 @@ hb_buffer_t * hb_bd_read( hb_bd_t * d )
                 case BD_EVENT_CHAPTER:
                     // The muxers expect to only get chapter 2 and above
                     // They write chapter 1 when chapter 2 is detected.
-                    d->next_chap = event.param;
+                    if (event.param > d->chapter)
+                    {
+                        d->next_chap = event.param;
+                    }
                     break;
 
                 case BD_EVENT_PLAYITEM:
@@ -748,12 +780,18 @@ hb_buffer_t * hb_bd_read( hb_bd_t * d )
             }
         }
         // buf+4 to skip the BD timestamp at start of packet
-        b = hb_ts_decode_pkt( d->stream, buf+4 );
-        if ( b )
+        if (d->chapter != d->next_chap)
         {
-            b->s.discontinuity = discontinuity;
-            b->s.new_chap = new_chap;
-            return b;
+            d->chapter = d->next_chap;
+            out = hb_ts_decode_pkt(d->stream, buf+4, d->chapter, discontinuity);
+        }
+        else
+        {
+            out = hb_ts_decode_pkt(d->stream, buf+4, 0, discontinuity);
+        }
+        if (out != NULL)
+        {
+            return out;
         }
     }
     return NULL;
@@ -767,7 +805,7 @@ hb_buffer_t * hb_bd_read( hb_bd_t * d )
  **********************************************************************/
 int hb_bd_chapter( hb_bd_t * d )
 {
-    return d->next_chap;
+    return d->chapter;
 }
 
 /***********************************************************************
@@ -826,7 +864,8 @@ static int have_ts_sync(const uint8_t *buf, int psize)
 
 static uint64_t align_to_next_packet(BLURAY *bd, uint8_t *pkt)
 {
-    uint8_t buf[MAX_HOLE];
+    int      result;
+    uint8_t  buf[MAX_HOLE];
     uint64_t pos = 0;
     uint64_t start = bd_tell(bd);
     uint64_t orig;
@@ -840,7 +879,8 @@ static uint64_t align_to_next_packet(BLURAY *bd, uint8_t *pkt)
 
     while (1)
     {
-        if (bd_read(bd, buf+off, sizeof(buf)-off) == sizeof(buf)-off)
+        result = bd_read(bd, buf + off, sizeof(buf) - off);
+        if (result == sizeof(buf) - off)
         {
             const uint8_t *bp = buf;
             int i;
@@ -861,6 +901,10 @@ static uint64_t align_to_next_packet(BLURAY *bd, uint8_t *pkt)
             memcpy(buf, buf + sizeof(buf) - off, off);
             start += sizeof(buf) - off;
         }
+        else if (result < 0)
+        {
+            return -1;
+        }
         else
         {
             return 0;
@@ -873,9 +917,14 @@ static uint64_t align_to_next_packet(BLURAY *bd, uint8_t *pkt)
     bd_seek(bd, off);
     while (off > bd_tell(bd))
     {
-        if (bd_read(bd, buf, 192) != 192)
+        result = bd_read(bd, buf, 192);
+        if (result < 0)
         {
-            break;
+            return -1;
+        }
+        else if (result != 192)
+        {
+            return 0;
         }
     }
     return start - orig + pos;
@@ -904,12 +953,16 @@ static int next_packet( BLURAY *bd, uint8_t *pkt )
         // lost sync - back up to where we started then try to re-establish.
         uint64_t pos = bd_tell(bd);
         uint64_t pos2 = align_to_next_packet(bd, pkt);
-        if ( pos2 == 0 )
+        if (pos2 < 0)
         {
-            hb_log( "next_packet: eof while re-establishing sync @ %"PRId64, pos );
+            return -1;
+        }
+        else if (pos2 == 0)
+        {
+            hb_log("next_packet: eof while re-establishing sync @ %"PRIu64"", pos );
             return 0;
         }
-        hb_log( "next_packet: sync lost @ %"PRId64", regained after %"PRId64" bytes",
+        hb_log("next_packet: sync lost @ %"PRIu64", regained after %"PRIu64" bytes",
                  pos, pos2 );
     }
 }
