@@ -1,6 +1,6 @@
 /* deccc608sub.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -24,16 +24,6 @@ static int debug_608 = 0;
 static int cc_channel = 1;
 static int subs_delay = 0;
 
-/*
- * Get the time of the last buffer that we have received.
- */
-static int64_t get_last_pts(struct s_write *wb)
-{
-    return wb->last_pts;
-}
-
-#define fatal(N, ...) // N
-
 int rowdata[] = {11,-1,1,2,3,4,12,13,14,15,5,6,7,8,9,10};
 // Relationship between the first PAC byte and the row number
 
@@ -141,9 +131,9 @@ static int general_608_init (struct s_write *wb)
     wb->new_channel = 1;
     wb->in_xds_mode = 0;
 
-    wb->hb_buffer = NULL;
-    wb->hb_last_buffer = NULL;
+    hb_buffer_list_clear(&wb->list);
     wb->last_pts = 0;
+    wb->last_scr_sequence = 0;
     return 0;
 }
 
@@ -162,10 +152,7 @@ static void general_608_close (struct s_write *wb)
     if( wb->subline ) {
         free(wb->subline);
     }
-
-    if( wb->hb_buffer ) {
-        hb_buffer_close( &wb->hb_buffer );
-    }
+    hb_buffer_list_close(&wb->list);
 }
 
 
@@ -563,6 +550,17 @@ static unsigned encode_line(unsigned char *buffer, unsigned char *text)
     return bytes;
 }
 
+static unsigned stuff_space(unsigned char *buffer, int space)
+{
+    int ii;
+    for (ii = 0; ii < space; ii++)
+    {
+        *buffer++ = '\\';
+        *buffer++ = 'h';
+    }
+    return space * 2;
+}
+
 static void find_limit_characters(unsigned char *line, int *first_non_blank,
                                   int *last_non_blank)
 {
@@ -734,10 +732,11 @@ static struct eia608_screen *get_writing_buffer(struct s_write *wb)
         case MODE_ROLLUP_2: // Write directly to screen
         case MODE_ROLLUP_3:
         case MODE_ROLLUP_4:
+        case MODE_TEXT:
             use_buffer = get_current_visible_buffer(wb);
             break;
         default:
-            fatal (EXIT_BUG_BUG, "Caption mode has an illegal value at get_writing_buffer(), this is a bug.\n");
+            hb_error("Caption mode has an illegal value at get_writing_buffer(), this is a bug.");
     }
     return use_buffer;
 }
@@ -747,17 +746,20 @@ static void write_char(const unsigned char c, struct s_write *wb)
     if (wb->data608->mode != MODE_TEXT)
     {
         struct eia608_screen * use_buffer = get_writing_buffer(wb);
-        /* hb_log ("\rWriting char [%c] at %s:%d:%d\n",c,
-        use_buffer == &wb->data608->buffer1?"B1":"B2",
-        wb->data608->cursor_row,wb->data608->cursor_column); */
-        use_buffer->characters[wb->data608->cursor_row][wb->data608->cursor_column] = c;
-        use_buffer->colors[wb->data608->cursor_row][wb->data608->cursor_column] = wb->data608->color;
-        use_buffer->fonts[wb->data608->cursor_row][wb->data608->cursor_column] = wb->data608->font;
-        use_buffer->row_used[wb->data608->cursor_row] = 1;
-        use_buffer->empty = 0;
-        if (wb->data608->cursor_column < 31)
-            wb->data608->cursor_column++;
-        use_buffer->dirty = 1;
+        if (use_buffer != NULL)
+        {
+            /* hb_log ("\rWriting char [%c] at %s:%d:%d\n",c,
+            use_buffer == &wb->data608->buffer1?"B1":"B2",
+            wb->data608->cursor_row,wb->data608->cursor_column); */
+            use_buffer->characters[wb->data608->cursor_row][wb->data608->cursor_column] = c;
+            use_buffer->colors[wb->data608->cursor_row][wb->data608->cursor_column] = wb->data608->color;
+            use_buffer->fonts[wb->data608->cursor_row][wb->data608->cursor_column] = wb->data608->font;
+            use_buffer->row_used[wb->data608->cursor_row] = 1;
+            use_buffer->empty = 0;
+            if (wb->data608->cursor_column < 31)
+                wb->data608->cursor_column++;
+            use_buffer->dirty = 1;
+        }
     }
 
 }
@@ -797,8 +799,7 @@ static int write_cc_buffer_as_ssa(struct eia608_screen *data,
     int wrote_something = 0;
     int i;
     int64_t ms_start = wb->data608->current_visible_start_ms;
-    //int64_t ms_end = get_last_pts(wb) + subs_delay;
-    int row = -1, col = -1;
+    //int64_t ms_end = wb->last_pts + subs_delay;
 
     ms_start += subs_delay;
     if (ms_start<0) // Drop screens that because of subs_delay start too early
@@ -820,6 +821,8 @@ static int write_cc_buffer_as_ssa(struct eia608_screen *data,
      * and have a big bottom line (strip spaces from any joined lines).
      */
     int rows = 0, columns = 0;
+    int min_row = 15, max_row = 0;
+    int min_col = 41, max_col = 0;
     for (i = 0; i < 15; i++)
     {
         if (data->row_used[i])
@@ -830,76 +833,109 @@ static int write_cc_buffer_as_ssa(struct eia608_screen *data,
             find_limit_characters(data->characters[i], &first, &last);
             if (last - first + 1 > columns)
                 columns = last - first + 1;
+            if (min_col > first)
+                min_col = first;
+            if (min_row > i)
+                min_row = i;
+            if (max_col < last)
+                max_col = last;
+            if (max_row < i)
+                max_row = i;
         }
     }
 
     wb->prev_font_style = FONT_REGULAR;
     wb->prev_font_color = COL_WHITE;
     wb->enc_buffer_used = 0;
+
+    int cropped_width, cropped_height, font_size;
+    int cell_width, cell_height;
+    int safe_x, safe_y;
+    int min_safe_x, min_safe_y;
+    double aspect;
+
+    cropped_height = wb->height - wb->crop[0] - wb->crop[1];
+    cropped_width = wb->width - wb->crop[2] - wb->crop[3];
+    aspect = (double)wb->width * wb->par.num /
+                    (wb->height * wb->par.den);
+
+    // CC grid is 16 rows by 32 columns (for 4:3 video)
+    // Our SSA resolution is the title resolution
+    // Translate CC grid to SSA coordinates
+    // The numbers are tweaked to keep things off the very
+    // edges of the screen and in the "safe" zone
+    int screen_columns = 32;
+    if (aspect >= 1.6)
+    {
+        // If the display aspect is close to or greater than 16:9
+        // then width of screen is 42 columns (see CEA-708)
+        screen_columns = 42;
+    }
+    font_size = wb->height * .8 * .08;
+
+    safe_x = 0.1 * wb->width;
+    safe_y = 0.1 * wb->height;
+    min_safe_x = 0.025 * cropped_width;
+    min_safe_y = 0.025 * cropped_height;
+    cell_height = (wb->height - 2 * safe_y) / 16; 
+    cell_width  = (wb->width  - 2 * safe_x) / screen_columns; 
+
+    char *pos;
+    int y, x, top;
+    int col = min_col;
+    if (aspect >= 1.6)
+    {
+        // If the display aspect is close to or greater than 16:9
+        // center the CC in about a 4:3 region
+        col += 5;
+    }
+    y = cell_height * (min_row + 1 + rows) + safe_y - wb->crop[0];
+    x = cell_width * col + safe_x - wb->crop[2];
+    top = y - rows * font_size;
+
+    if (top < min_safe_y)
+        y = (rows * font_size) + min_safe_y;
+    if (y > cropped_height - min_safe_y)
+        y = cropped_height - min_safe_y;
+    if (x + columns * cell_width > cropped_width - min_safe_x)
+        x = cropped_width - columns * cell_width - min_safe_x;
+    if (x < min_safe_x)
+        x = min_safe_x;
+    pos = hb_strdup_printf("{\\an1\\pos(%d,%d)}", x, y);
+
     int line = 1;
     for (i = 0; i < 15; i++)
     {
         if (data->row_used[i])
         {
+            int first, last;
             // Get position for this CC
-            if (row == -1)
-            {
-                int last, x, y, top, safe_zone, cell_width, cell_height;
-                int cropped_width, cropped_height, font_size;
-                char *pos;
-
-                row = i;
-                find_limit_characters(data->characters[i], &col, &last);
-
-                // CC grid is 16 rows by 62 colums
-                // Our SSA resolution is the title resolution
-                // Tranlate CC grid to SSA coordinates
-                // The numbers are tweaked to keep things off the very
-                // edges of the screen and in the "safe" zone
-                cropped_height = wb->height - wb->crop[0] - wb->crop[1];
-                cropped_width = wb->width - wb->crop[2] - wb->crop[3];
-                font_size = cropped_height * .066;
-
-                safe_zone = cropped_height * 0.025; 
-                cell_height = (wb->height - 2 * safe_zone) / 16; 
-                cell_width = (wb->width - 2 * safe_zone) / 32; 
-
-                // Calculate position assuming the position defines
-                // the baseline of the text which is lower left corner
-                // of bottom row of characters
-                y = cell_height * (row + 1 + rows) + safe_zone - wb->crop[0];
-                top = y - rows * font_size;
-                x = cell_width * col + safe_zone - wb->crop[2];
-                if (top < safe_zone)
-                    y = (rows * font_size) + safe_zone;
-                if (y > cropped_height - safe_zone)
-                    y = cropped_height - safe_zone;
-                if (x + columns * cell_width > cropped_width - safe_zone)
-                    x = cropped_width - columns * cell_width - safe_zone;
-                if (x < safe_zone)
-                    x = safe_zone;
-                pos = hb_strdup_printf("{\\a1\\pos(%d,%d)}", x, y);
-                wb->enc_buffer_used += encode_line(
-                        wb->enc_buffer + wb->enc_buffer_used, (uint8_t*)pos);
-                free(pos);
-            }
+            find_limit_characters(data->characters[i], &first, &last);
 
             /*
              * The intention was to use a newline but QT doesn't like it,
              * old code still here just in case..
              */
+            int space = first - min_col;
             if (line == 1) {
+                wb->enc_buffer_used += encode_line(
+                        wb->enc_buffer + wb->enc_buffer_used, (uint8_t*)pos);
+                wb->enc_buffer_used += stuff_space(
+                        wb->enc_buffer + wb->enc_buffer_used, space);
                 wb->enc_buffer_used += get_decoder_line_encoded(wb,
                         wb->enc_buffer + wb->enc_buffer_used, i, data);
                 line = 2;
             } else {
                 wb->enc_buffer_used += encode_line(
                         wb->enc_buffer + wb->enc_buffer_used, (uint8_t*)"\\N");
+                wb->enc_buffer_used += stuff_space(
+                        wb->enc_buffer + wb->enc_buffer_used, space);
                 wb->enc_buffer_used += get_decoder_line_encoded(wb,
                         wb->enc_buffer + wb->enc_buffer_used, i, data);
             }
         }
     }
+    free(pos);
     if (wb->enc_buffer_used && wb->enc_buffer[0] != 0 && data->dirty)
     {
         hb_buffer_t *buffer;
@@ -911,37 +947,23 @@ static int write_cc_buffer_as_ssa(struct eia608_screen *data,
         buffer->s.frametype = HB_FRAME_SUBTITLE;
         buffer->s.start = ms_start;
         buffer->s.stop = AV_NOPTS_VALUE;
+        buffer->s.scr_sequence = wb->data608->current_visible_scr_sequence;
         sprintf((char*)buffer->data, "%d,,Default,,0,0,0,,", ++wb->line);
         len = strlen((char*)buffer->data);
         memcpy(buffer->data + len, wb->enc_buffer, wb->enc_buffer_used);
-        if (wb->hb_last_buffer)
-        {
-            wb->hb_last_buffer->next = buffer;
-        }
-        else
-        {
-            wb->hb_buffer = buffer;
-        }
-        wb->hb_last_buffer = buffer;
+        hb_buffer_list_append(&wb->list, buffer);
         wrote_something=1;
         wb->clear_sub_needed = 1;
     }
     else if (wb->clear_sub_needed)
     {
-        hb_buffer_t *buffer = hb_buffer_init(1);
+        hb_buffer_t *buffer = hb_buffer_init(0);
         buffer->s.frametype = HB_FRAME_SUBTITLE;
-        buffer->s.start = ms_start;
-        buffer->s.stop = ms_start;
-        buffer->data[0] = 0;
-        if (wb->hb_last_buffer != NULL)
-        {
-            wb->hb_last_buffer->next = buffer;
-        }
-        else
-        {
-            wb->hb_buffer = buffer;
-        }
-        wb->hb_last_buffer = buffer;
+        buffer->s.flags     = HB_BUF_FLAG_EOS;
+        buffer->s.start     = ms_start;
+        buffer->s.stop      = ms_start;
+        buffer->s.scr_sequence = wb->data608->current_visible_scr_sequence;
+        hb_buffer_list_append(&wb->list, buffer);
         wb->clear_sub_needed = 0;
     }
     if (debug_608)
@@ -1193,9 +1215,12 @@ static void handle_command(unsigned char c1, const unsigned char c2,
             {
                 struct eia608_screen *data;
                 data = get_writing_buffer(wb);
-                wb->data608->cursor_column--;
-                data->characters[wb->data608->cursor_row][wb->data608->cursor_column] = ' ';
-                data->dirty = 1;
+                if (data != NULL)
+                {
+                    wb->data608->cursor_column--;
+                    data->characters[wb->data608->cursor_row][wb->data608->cursor_column] = ' ';
+                    data->dirty = 1;
+                }
             }
             break;
         case COM_TABOFFSET1:
@@ -1214,11 +1239,13 @@ static void handle_command(unsigned char c1, const unsigned char c2,
             break;
         case COM_RESUMECAPTIONLOADING:
             wb->data608->mode=MODE_POPUP;
-            wb->data608->current_visible_start_ms = get_last_pts(wb);
+            wb->data608->current_visible_start_ms = wb->last_pts;
+            wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             break;
         case COM_RESUMETEXTDISPLAY:
             wb->data608->mode=MODE_TEXT;
-            wb->data608->current_visible_start_ms = get_last_pts(wb);
+            wb->data608->current_visible_start_ms = wb->last_pts;
+            wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             break;
         case COM_ROLLUP2:
             if (wb->data608->rollup_base_row + 1 < 2)
@@ -1242,7 +1269,8 @@ static void handle_command(unsigned char c1, const unsigned char c2,
                 handle_command(0x14, 0x2D, wb);
                 wb->rollup_cr = 1;
             }
-            wb->data608->current_visible_start_ms = get_last_pts(wb);
+            wb->data608->current_visible_start_ms = wb->last_pts;
+            wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             wb->data608->mode=MODE_ROLLUP_2;
             erase_memory (wb, 0);
             wb->data608->cursor_column = 0;
@@ -1269,7 +1297,8 @@ static void handle_command(unsigned char c1, const unsigned char c2,
                 handle_command(0x14, 0x2D, wb);
                 wb->rollup_cr = 1;
             }
-            wb->data608->current_visible_start_ms = get_last_pts(wb);
+            wb->data608->current_visible_start_ms = wb->last_pts;
+            wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             wb->data608->mode=MODE_ROLLUP_3;
             erase_memory (wb, 0);
             wb->data608->cursor_column = 0;
@@ -1296,7 +1325,8 @@ static void handle_command(unsigned char c1, const unsigned char c2,
                 handle_command(0x14, 0x2D, wb);
                 wb->rollup_cr = 1;
             }
-            wb->data608->current_visible_start_ms = get_last_pts(wb);
+            wb->data608->current_visible_start_ms = wb->last_pts;
+            wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             wb->data608->mode = MODE_ROLLUP_4;
             wb->data608->cursor_column = 0;
             wb->data608->cursor_row = wb->data608->rollup_base_row;
@@ -1310,14 +1340,16 @@ static void handle_command(unsigned char c1, const unsigned char c2,
             if (wb->rollup_cr && is_current_row_empty(wb))
             {
                 wb->rollup_cr = 0;
-                wb->data608->current_visible_start_ms = get_last_pts(wb);
+                wb->data608->current_visible_start_ms = wb->last_pts;
+                wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
                 break;
             }
             if (write_cc_buffer(wb))
                 wb->data608->screenfuls_counter++;
             roll_up(wb);
             wb->data608->cursor_column = 0;
-            wb->data608->current_visible_start_ms = get_last_pts(wb);
+            wb->data608->current_visible_start_ms = wb->last_pts;
+            wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             break;
         case COM_ERASENONDISPLAYEDMEMORY:
             erase_memory (wb,0);
@@ -1335,7 +1367,8 @@ static void handle_command(unsigned char c1, const unsigned char c2,
 
             // the last pts is the time to remove the previously 
             // displayed CC from the display
-            wb->data608->current_visible_start_ms = get_last_pts(wb);
+            wb->data608->current_visible_start_ms = wb->last_pts;
+            wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
 
             // Write "clear" subtitle if necessary
             struct eia608_screen *data;
@@ -1349,7 +1382,8 @@ static void handle_command(unsigned char c1, const unsigned char c2,
             if (wb->data608->mode == MODE_POPUP)
             {
                 swap_visible_buffer(wb);
-                wb->data608->current_visible_start_ms = get_last_pts(wb);
+                wb->data608->current_visible_start_ms = wb->last_pts;
+                wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             }
             if (write_cc_buffer(wb))
                 wb->data608->screenfuls_counter++;
@@ -1470,12 +1504,13 @@ static void handle_pac(unsigned char c1, unsigned char c2, struct s_write *wb)
             return;
         }
     }
-    int color=pac2_attribs[c2][0];
-    int font=pac2_attribs[c2][1];
+    wb->data608->color=pac2_attribs[c2][0];
+    wb->data608->font=pac2_attribs[c2][1];
     int indent=pac2_attribs[c2][2];
     if (debug_608)
-        hb_log ("  --  Position: %d:%d, color: %s,  font: %s\n",row,
-            indent,color_text[color][0],font_text[font]);
+        hb_log ("  --  Position: %d:%d, color: %s,  font: %s\n", row, indent,
+                color_text[wb->data608->color][0],
+                font_text[wb->data608->font]);
 
     // CC spec says to the preferred method to handle a roll-up base row
     // that causes the display to scroll off the top of the screen is to 
@@ -1617,7 +1652,7 @@ static int disCommand(unsigned char hi, unsigned char lo, struct s_write *wb)
                 handle_pac (hi,lo,wb);
             break;
         case 0x17:
-            if (lo>=0x21 && lo<=0x22)
+            if (lo>=0x21 && lo<=0x23)
                 handle_command (hi,lo,wb);
             if (lo>=0x2e && lo<=0x2f)
                 handle_text_attr (hi,lo,wb);
@@ -1702,7 +1737,7 @@ static void process608(const unsigned char *data, int length,
 
             if ( debug_608 && !textprinted && wb->data608->channel==cc_channel )
             {   // Current FTS information after the characters are shown
-                //hb_log("Current FTS: %s\n", print_mstime(get_last_pts()));
+                //hb_log("Current FTS: %s\n", print_mstime(wb->last_pts));
             }
 
             if ((wb->data608->mode == MODE_ROLLUP_2 ||
@@ -1713,7 +1748,8 @@ static void process608(const unsigned char *data, int length,
                 // If we are showing rollup on the fly (direct_rollup)
                 // write a buffer now
                 write_cc_buffer(wb);
-                wb->data608->current_visible_start_ms = get_last_pts(wb);
+                wb->data608->current_visible_start_ms = wb->last_pts;
+                wb->data608->current_visible_scr_sequence = wb->last_scr_sequence;
             }
         }
     }
@@ -1741,9 +1777,10 @@ static int decccInit( hb_work_object_t * w, hb_job_t * job )
 
         if( pv->cc608 )
         {
-            pv->cc608->width = job->title->width;
-            pv->cc608->height = job->title->height;
+            pv->cc608->width = job->title->geometry.width;
+            pv->cc608->height = job->title->geometry.height;
             memcpy(pv->cc608->crop, job->crop, sizeof(int[4]));
+            pv->cc608->par = job->title->geometry.par;
             retval = general_608_init(pv->cc608);
             if( !retval )
             {
@@ -1752,21 +1789,28 @@ static int decccInit( hb_work_object_t * w, hb_job_t * job )
                 {
                     retval = 1;
                 }
-                init_eia608(pv->cc608->data608);
+                else
+                {
+                    init_eia608(pv->cc608->data608);
+                }
             }
+
+            // When rendering subs, we need to push rollup subtitles out
+            // asap (instead of waiting for a completed line) so that we
+            // do not miss the frame that they should be rendered over.
+            pv->cc608->direct_rollup = w->subtitle->config.dest == RENDERSUB;
+        }
+
+        if (!retval)
+        {
+            // Generate generic SSA Script Info.
+            int height = job->title->geometry.height - job->crop[0] - job->crop[1];
+            int width = job->title->geometry.width - job->crop[2] - job->crop[3];
+            int safe_height = 0.8 * job->title->geometry.height;
+            hb_subtitle_add_ssa_header(w->subtitle, HB_FONT_MONO,
+                                       .08 * safe_height, width, height);
         }
     }
-    if (!retval)
-    {
-        // Generate generic SSA Script Info.
-        int height = job->title->height - job->crop[0] - job->crop[1];
-        int width = job->title->width - job->crop[2] - job->crop[3];
-        hb_subtitle_add_ssa_header(w->subtitle, width, height);
-    }
-    // When rendering subs, we need to push rollup subtitles out
-    // asap (instead of waiting for a completed line) so that we
-    // do not miss the frame that they should be rendered over.
-    pv->cc608->direct_rollup = w->subtitle->config.dest == RENDERSUB;
     return retval;
 }
 
@@ -1776,46 +1820,40 @@ static int decccWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
     hb_work_private_t * pv = w->private_data;
     hb_buffer_t * in = *buf_in;
 
-    if ( in->size <= 0 )
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
         /* EOF on input stream - send it downstream & say that we're done */
         handle_end_of_data(pv->cc608);
         /*
          * Grab any pending buffer and output them with the EOF on the end
          */
-        if (pv->cc608->hb_last_buffer) {
-            pv->cc608->hb_last_buffer->next = in;
-            *buf_out = pv->cc608->hb_buffer;
-            *buf_in = NULL;
-            pv->cc608->hb_buffer = NULL;
-            pv->cc608->hb_last_buffer = NULL;
-        } else {
-            *buf_out = in;
-            *buf_in = NULL;
-        }
+        *buf_in = NULL;
+        hb_buffer_list_append(&pv->cc608->list, in);
+        *buf_out = hb_buffer_list_clear(&pv->cc608->list);
         return HB_WORK_DONE;
     }
 
     pv->cc608->last_pts = in->s.start;
+    pv->cc608->last_scr_sequence = in->s.scr_sequence;
     process608(in->data, in->size, pv->cc608);
 
     /*
      * If there is one waiting then pass it on
      */
-    *buf_out = pv->cc608->hb_buffer;
-
-    pv->cc608->hb_buffer = NULL;
-    pv->cc608->hb_last_buffer = NULL;
-
+    *buf_out = hb_buffer_list_clear(&pv->cc608->list);
     return HB_WORK_OK;
 }
 
 static void decccClose( hb_work_object_t * w )
 {
     hb_work_private_t * pv = w->private_data;
-    general_608_close( pv->cc608 );
-    free( pv->cc608->data608 );
-    free( pv->cc608 );
+
+    if (pv)
+    {
+        general_608_close( pv->cc608 );
+        free( pv->cc608->data608 );
+        free( pv->cc608 );
+    }
     free( w->private_data );
 }
 
