@@ -1,6 +1,6 @@
 /* decsrtsub.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -180,18 +180,24 @@ void hb_srt_to_ssa(hb_buffer_t *sub_in, int line)
             ssa = (char*)sub_in->data;
             if (srt[ii] == '\r')
             {
-                ssa[pos++] = '\\';
-                ssa[pos++] = 'N';
-                ii++;
-                if (srt[ii] == '\n')
+                if (srt[ii + 1] == '\n')
                 {
                     ii++;
                 }
+                if (srt[ii + 1] != 0)
+                {
+                    ssa[pos++] = '\\';
+                    ssa[pos++] = 'N';
+                }
+                ii++;
             }
             else if (srt[ii] == '\n')
             {
-                ssa[pos++] = '\\';
-                ssa[pos++] = 'N';
+                if (srt[ii + 1] != 0)
+                {
+                    ssa[pos++] = '\\';
+                    ssa[pos++] = 'N';
+                }
                 ii++;
             }
             else
@@ -251,10 +257,7 @@ static int utf8_fill( hb_work_private_t * pv )
             pv->end = bytes;
             if( bytes == 0 )
             {
-                if( conversion )
-                    return 1;
-                else
-                    return 0;
+                return conversion;
             }
         }
 
@@ -263,8 +266,10 @@ static int utf8_fill( hb_work_private_t * pv )
         in_size = pv->end - pv->pos;
 
         retval = iconv( pv->iconv_context, &p, &in_size, &q, &out_size);
-        if( q != pv->utf8_buf + pv->utf8_pos )
+        if (q != pv->utf8_buf + pv->utf8_end)
+        {
             conversion = 1;
+        }
 
         pv->utf8_end = q - pv->utf8_buf;
         pv->pos = p - pv->buf;
@@ -275,6 +280,11 @@ static int utf8_fill( hb_work_private_t * pv )
             if (buf[0] == 0xef && buf[1] == 0xbb && buf[2] == 0xbf)
             {
                 pv->utf8_pos = 3;
+                if (pv->utf8_pos == pv->utf8_end)
+                {
+                    // If only the bom was converted, indicate no conversion
+                    conversion = 0;
+                }
             }
             pv->utf8_bom_skipped = 1;
         }
@@ -288,10 +298,7 @@ static int utf8_fill( hb_work_private_t * pv )
             bytes = fread( pv->buf + pv->end, 1, 1024 - pv->end, pv->file );
             if( bytes == 0 )
             {
-                if( !conversion )
-                    return 0;
-                else
-                    return 1;
+                return conversion;
             }
             pv->end += bytes;
         } else if ( ( retval == -1 ) && ( errno == EILSEQ ) )
@@ -417,7 +424,7 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
             int  size, len;
 
             // If the current line is empty, we assume this is the
-            //	seperation betwene two entries. In case we are wrong,
+            //	separation between two entries. In case we are wrong,
             //	the mistake is corrected in the next state.
             if (strcmp(line_buffer, "\n") == 0 || strcmp(line_buffer, "\r\n") == 0) {
                 pv->current_state = k_state_potential_new_entry;
@@ -448,7 +455,7 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
                  * Well.. looks like we are in the wrong mode.. lets add the
                  * newline we misinterpreted...
                  */
-                strncat(pv->current_entry.text, " ", 1024);
+                strncat(pv->current_entry.text, " ", sizeof(pv->current_entry.text) - strlen(pv->current_entry.text) - 1);
                 pv->current_state = k_state_inEntry_or_new;
                 continue;
             }
@@ -473,14 +480,14 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
             resync = 0;
             if (*pv->current_entry.text != '\0')
             {
-                long length;
+                long length = 0;
                 char *p, *q;
                 uint64_t start_time = ( pv->current_entry.start +
                                         pv->subtitle->config.offset ) * 90;
                 uint64_t stop_time = ( pv->current_entry.stop +
                                        pv->subtitle->config.offset ) * 90;
 
-                if( !( start_time > pv->start_time && stop_time < pv->stop_time ) )
+                if( !( start_time >= pv->start_time && stop_time < pv->stop_time ) )
                 {
                     hb_deep_log( 3, "Discarding SRT at time start %"PRId64", stop %"PRId64, start_time, stop_time);
                     memset( &pv->current_entry, 0, sizeof( srt_entry_t ) );
@@ -489,27 +496,25 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
                     continue;
                 }
 
-                length = strlen( pv->current_entry.text );
-
                 for (q = p = pv->current_entry.text; *p != '\0'; p++)
                 {
-                    if (*p == '\r')
+                    if (*p == '\r' || *p == '\n')
                     {
-                        if (*(p + 1) == '\n' || *(p + 1) == '\r' ||
-                            *(p + 1) == '\0')
+                        // sanitize newline codes to a single '\n'
+                        // and delete any newline at the end of the subtitle
+                        if (*(p + 1) != '\n' && *(p + 1) != '\r' &&
+                            *(p + 1) != '\0')
                         {
-                            // followed by line break or last character, skip it
-                            length--;
-                            continue;
+                            *q   = '\n';
+                            q++;
+                            length++;
                         }
-                        // replace '\r' with '\n'
-                        *q   = '\n';
-                        q++;
                     }
                     else
                     {
                         *q = *p;
                         q++;
+                        length++;
                     }
                 }
                 *q = '\0';
@@ -539,40 +544,38 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
     hb_buffer_t *buffer = NULL;
     if (*pv->current_entry.text != '\0')
     {
-        long length;
+        long length = 0;
         char *p, *q;
         uint64_t start_time = ( pv->current_entry.start +
                                 pv->subtitle->config.offset ) * 90;
         uint64_t stop_time = ( pv->current_entry.stop +
                                pv->subtitle->config.offset ) * 90;
 
-        if( !( start_time > pv->start_time && stop_time < pv->stop_time ) )
+        if( !( start_time >= pv->start_time && stop_time < pv->stop_time ) )
         {
             hb_deep_log( 3, "Discarding SRT at time start %"PRId64", stop %"PRId64, start_time, stop_time);
             memset( &pv->current_entry, 0, sizeof( srt_entry_t ) );
             return NULL;
         }
 
-        length = strlen( pv->current_entry.text );
-
         for (q = p = pv->current_entry.text; *p != '\0'; p++)
         {
-            if (*p == '\r')
+            if (*p == '\r' || *p == '\n')
             {
-                if (*(p + 1) == '\n' || *(p + 1) == '\r' || *(p + 1) == '\0')
+                // sanitize newline codes to a single '\n'
+                // and delete any newline at the end of the subtitle
+                if (*(p + 1) != '\n' && *(p + 1) != '\r' && *(p + 1) != '\0')
                 {
-                    // followed by line break or last character, skip it
-                    length--;
-                    continue;
+                    *q   = '\n';
+                    q++;
+                    length++;
                 }
-                // replace '\r' with '\n'
-                *q   = '\n';
-                q++;
             }
             else
             {
                 *q = *p;
                 q++;
+                length++;
             }
         }
         *q = '\0';
@@ -598,114 +601,133 @@ static hb_buffer_t *srt_read( hb_work_private_t *pv )
 
 static int decsrtInit( hb_work_object_t * w, hb_job_t * job )
 {
-    int retval = 1;
     hb_work_private_t * pv;
-    hb_buffer_t *buffer;
     int i;
     hb_chapter_t * chapter;
 
     pv = calloc( 1, sizeof( hb_work_private_t ) );
-    if( pv )
+    if (pv == NULL)
     {
-        w->private_data = pv;
+        goto fail;
+    }
 
-        pv->job = job;
+    w->private_data = pv;
 
-        buffer = hb_buffer_init( 0 );
-        hb_fifo_push( w->fifo_in, buffer);
+    pv->job = job;
+    pv->current_state = k_state_potential_new_entry;
+    pv->number_of_entries = 0;
+    pv->last_entry_number = 0;
+    pv->current_time = 0;
+    pv->subtitle = w->subtitle;
 
-        pv->current_state = k_state_potential_new_entry;
-        pv->number_of_entries = 0;
-        pv->last_entry_number = 0;
-        pv->current_time = 0;
-        pv->subtitle = w->subtitle;
-
-        /*
-         * Figure out the start and stop times from teh chapters being
-         * encoded - drop subtitle not in this range.
-         */
-        pv->start_time = 0;
-        for( i = 1; i < job->chapter_start; ++i )
+    /*
+     * Figure out the start and stop times from the chapters being
+     * encoded - drop subtitle not in this range.
+     */
+    pv->start_time = 0;
+    for( i = 1; i < job->chapter_start; ++i )
+    {
+        chapter = hb_list_item( job->list_chapter, i - 1 );
+        if( chapter )
         {
-            chapter = hb_list_item( job->list_chapter, i - 1 );
-            if( chapter )
-            {
-                pv->start_time += chapter->duration;
-            } else {
-                hb_error( "Could not locate chapter %d for SRT start time", i );
-                retval = 0;
-            }
-        }
-        pv->stop_time = pv->start_time;
-        for( i = job->chapter_start; i <= job->chapter_end; ++i )
-        {
-            chapter = hb_list_item( job->list_chapter, i - 1 );
-            if( chapter )
-            {
-                pv->stop_time += chapter->duration;
-            } else {
-                hb_error( "Could not locate chapter %d for SRT start time", i );
-                retval = 0;
-            }
-        }
-
-        hb_deep_log( 3, "SRT Start time %"PRId64", stop time %"PRId64, pv->start_time, pv->stop_time);
-
-        pv->iconv_context = iconv_open( "utf-8", pv->subtitle->config.src_codeset );
-
-
-        if( pv->iconv_context == (iconv_t) -1 )
-        {
-            hb_error("Could not open the iconv library with those file formats\n");
-
+            pv->start_time += chapter->duration;
         } else {
-            memset( &pv->current_entry, 0, sizeof( srt_entry_t ) );
-
-            pv->file = hb_fopen(w->subtitle->config.src_filename, "r");
-
-            if( !pv->file )
-            {
-                hb_error("Could not open the SRT subtitle file '%s'\n",
-                         w->subtitle->config.src_filename);
-            } else {
-                retval = 0;
-            }
+            hb_error( "Could not locate chapter %d for SRT start time", i );
         }
     }
-    if (!retval)
+    pv->stop_time = pv->start_time;
+    for( i = job->chapter_start; i <= job->chapter_end; ++i )
     {
-        // Generate generic SSA Script Info.
-        int height = job->title->height - job->crop[0] - job->crop[1];
-        int width = job->title->width - job->crop[2] - job->crop[3];
-        hb_subtitle_add_ssa_header(w->subtitle, width, height);
+        chapter = hb_list_item( job->list_chapter, i - 1 );
+        if( chapter )
+        {
+            pv->stop_time += chapter->duration;
+        } else {
+            hb_error( "Could not locate chapter %d for SRT start time", i );
+        }
     }
-    return retval;
+
+    hb_deep_log(3, "SRT Start time %"PRId64", stop time %"PRId64,
+                pv->start_time, pv->stop_time);
+
+    if (job->pts_to_start != 0)
+    {
+        pv->start_time = AV_NOPTS_VALUE;
+    }
+
+    pv->iconv_context = iconv_open( "utf-8", pv->subtitle->config.src_codeset );
+    if( pv->iconv_context == (iconv_t) -1 )
+    {
+        hb_error("Could not open the iconv library with those file formats\n");
+        goto fail;
+    } else {
+        memset( &pv->current_entry, 0, sizeof( srt_entry_t ) );
+
+        pv->file = hb_fopen(w->subtitle->config.src_filename, "r");
+
+        if( !pv->file )
+        {
+            hb_error("Could not open the SRT subtitle file '%s'\n",
+                     w->subtitle->config.src_filename);
+            goto fail;
+        }
+    }
+
+    // Generate generic SSA Script Info.
+    int height = job->title->geometry.height - job->crop[0] - job->crop[1];
+    int width = job->title->geometry.width - job->crop[2] - job->crop[3];
+    hb_subtitle_add_ssa_header(w->subtitle, HB_FONT_SANS,
+                               .066 * job->title->geometry.height,
+                               width, height);
+    return 0;
+
+fail:
+    if (pv != NULL)
+    {
+        if (pv->iconv_context != (iconv_t) -1)
+        {
+            iconv_close(pv->iconv_context);
+        }
+        if (pv->file != NULL)
+        {
+            fclose(pv->file);
+        }
+        free(pv);
+    }
+    return 1;
 }
 
 static int decsrtWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
                        hb_buffer_t ** buf_out )
 {
     hb_work_private_t * pv = w->private_data;
-    hb_buffer_t * in = *buf_in;
     hb_buffer_t * out = NULL;
 
-    out = srt_read( pv );
-    if( out )
+    if (pv->job->reader_pts_offset == AV_NOPTS_VALUE)
     {
-        hb_srt_to_ssa(out, ++pv->line);
-
-        /*
-         * Keep a buffer in our input fifo so that we get run.
-         */
-        hb_fifo_push( w->fifo_in, in);
-        *buf_in = NULL;
-        *buf_out = out;
-    } else {
+        // We need to wait for reader to initialize it's pts offset so that
+        // we know where to start reading SRTs.
         *buf_out = NULL;
         return HB_WORK_OK;
     }
-
-    return HB_WORK_OK;
+    if (pv->start_time == AV_NOPTS_VALUE)
+    {
+        pv->start_time = pv->job->reader_pts_offset;
+        if (pv->job->pts_to_stop > 0)
+        {
+            pv->stop_time = pv->job->pts_to_start + pv->job->pts_to_stop;
+        }
+    }
+    out = srt_read( pv );
+    if (out != NULL)
+    {
+        hb_srt_to_ssa(out, ++pv->line);
+        *buf_out = out;
+        return HB_WORK_OK;
+    } else {
+        *buf_out = hb_buffer_eof_init();
+        return HB_WORK_DONE;
+    }
 }
 
 static void decsrtClose( hb_work_object_t * w )
