@@ -1,6 +1,6 @@
 /* detelecine.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -103,15 +103,22 @@ static int hb_detelecine_work( hb_filter_object_t * filter,
 
 static void hb_detelecine_close( hb_filter_object_t * filter );
 
+static const char detelecine_template[] =
+    "skip-left=^"HB_INT_REG"$:skip-right=^"HB_INT_REG"$:"
+    "skip-top=^"HB_INT_REG"$:skip-bottom=^"HB_INT_REG"$:"
+    "strict-breaks=^"HB_BOOL_REG"$:plane=^([012])$:parity=^([01])$:"
+    "disable=^"HB_BOOL_REG"$";
+
 hb_filter_object_t hb_filter_detelecine =
 {
-    .id            = HB_FILTER_DETELECINE,
-    .enforce_order = 1,
-    .name          = "Detelecine (pullup)",
-    .settings      = NULL,
-    .init          = hb_detelecine_init,
-    .work          = hb_detelecine_work,
-    .close         = hb_detelecine_close,
+    .id                = HB_FILTER_DETELECINE,
+    .enforce_order     = 1,
+    .name              = "Detelecine (pullup)",
+    .settings          = NULL,
+    .init              = hb_detelecine_init,
+    .work              = hb_detelecine_work,
+    .close             = hb_detelecine_close,
+    .settings_template = detelecine_template,
 };
 
 /*
@@ -826,17 +833,14 @@ static int hb_detelecine_init( hb_filter_object_t * filter,
     ctx->metric_plane  = 0;
     ctx->parity = -1;
     
-    if( filter->settings )
-    {
-        sscanf( filter->settings, "%d:%d:%d:%d:%d:%d:%d",
-                &ctx->junk_left,
-                &ctx->junk_right,
-                &ctx->junk_top,
-                &ctx->junk_bottom,
-                &ctx->strict_breaks,
-                &ctx->metric_plane,
-                &ctx->parity );
-    }
+    // "Skip" array [top, bottom, left, right]
+    hb_dict_extract_int(&ctx->junk_top, filter->settings, "skip-top");
+    hb_dict_extract_int(&ctx->junk_bottom, filter->settings, "skip-bottom");
+    hb_dict_extract_int(&ctx->junk_left, filter->settings, "skip-left");
+    hb_dict_extract_int(&ctx->junk_right, filter->settings, "skip-right");
+    hb_dict_extract_int(&ctx->strict_breaks, filter->settings, "strict-breaks");
+    hb_dict_extract_int(&ctx->metric_plane, filter->settings, "plane");
+    hb_dict_extract_int(&ctx->parity, filter->settings, "parity");
 
     ctx->format = PULLUP_FMT_Y;
     ctx->nplanes = 4;
@@ -846,19 +850,20 @@ static int hb_detelecine_init( hb_filter_object_t * filter,
     ctx->bpp[0] = ctx->bpp[1] = ctx->bpp[2] = 8;
     ctx->background[1] = ctx->background[2] = 128;
 
-    ctx->w[0]      = init->width;
-    ctx->h[0]      = hb_image_height( init->pix_fmt, init->height, 0 );
-    ctx->stride[0] = hb_image_stride( init->pix_fmt, init->width, 0 );
+    ctx->w[0]      = init->geometry.width;
+    ctx->h[0]      = hb_image_height( init->pix_fmt, init->geometry.height, 0 );
+    ctx->stride[0] = hb_image_stride( init->pix_fmt, init->geometry.width, 0 );
 
-    ctx->w[1]      = init->width >> 1;
-    ctx->h[1]      = hb_image_height( init->pix_fmt, init->height, 1 );
-    ctx->stride[1] = hb_image_stride( init->pix_fmt, init->width, 1 );
+    ctx->w[1]      = init->geometry.width >> 1;
+    ctx->h[1]      = hb_image_height( init->pix_fmt, init->geometry.height, 1 );
+    ctx->stride[1] = hb_image_stride( init->pix_fmt, init->geometry.width, 1 );
 
-    ctx->w[1]      = init->width >> 1;
-    ctx->h[2]      = hb_image_height( init->pix_fmt, init->height, 2 );
-    ctx->stride[2] = hb_image_stride( init->pix_fmt, init->width, 2 );
+    ctx->w[1]      = init->geometry.width >> 1;
+    ctx->h[2]      = hb_image_height( init->pix_fmt, init->geometry.height, 2 );
+    ctx->stride[2] = hb_image_stride( init->pix_fmt, init->geometry.width, 2 );
 
-    ctx->w[3]      = ((init->width+15)/16) * ((init->height+15)/16);
+    ctx->w[3]      = ((init->geometry.width + 15) / 16) *
+                     ((init->geometry.height + 15) / 16);
     ctx->h[3]      = 2;
     ctx->stride[3] = ctx->w[3];
 
@@ -871,6 +876,8 @@ static int hb_detelecine_init( hb_filter_object_t * filter,
     pv->pullup_fakecount = 1;
     pv->pullup_skipflag = 0;
 
+    init->job->use_detelecine = 1;
+
     return 0;
 }
 
@@ -900,7 +907,7 @@ static int hb_detelecine_work( hb_filter_object_t * filter,
     hb_filter_private_t * pv = filter->private_data;
     hb_buffer_t * in = *buf_in, * out;
 
-    if ( in->size <= 0 )
+    if (in->s.flags & HB_BUF_FLAG_EOF)
     {
         *buf_out = in;
         *buf_in = NULL;
@@ -1020,8 +1027,6 @@ static int hb_detelecine_work( hb_filter_object_t * filter,
     pullup_release_frame( frame );
 
     out->s = in->s;
-    hb_buffer_move_subs( out, in );
-
     *buf_out = out;
 
 output_frame:
