@@ -1,6 +1,6 @@
 /* dvd.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -15,7 +15,7 @@
 #include "dvdread/ifo_print.h"
 #include "dvdread/nav_read.h"
 
-static hb_dvd_t    * hb_dvdread_init( char * path );
+static hb_dvd_t    * hb_dvdread_init( hb_handle_t * h, char * path );
 static void          hb_dvdread_close( hb_dvd_t ** _d );
 static char        * hb_dvdread_name( char * path );
 static int           hb_dvdread_title_count( hb_dvd_t * d );
@@ -106,7 +106,7 @@ static char * hb_dvdread_name( char * path )
  ***********************************************************************
  *
  **********************************************************************/
-hb_dvd_t * hb_dvdread_init( char * path )
+hb_dvd_t * hb_dvdread_init( hb_handle_t * h, char * path )
 {
     hb_dvd_t * e;
     hb_dvdread_t * d;
@@ -115,6 +115,7 @@ hb_dvd_t * hb_dvdread_init( char * path )
 
     e = calloc( sizeof( hb_dvd_t ), 1 );
     d = &(e->dvdread);
+    d->h = h;
 
     /*
      * Convert UTF-8 path to current code page on Windows
@@ -146,7 +147,7 @@ hb_dvd_t * hb_dvdread_init( char * path )
     /* Open main IFO */
     if( !( d->vmg = ifoOpen( d->reader, 0 ) ) )
     {
-        hb_error( "dvd: ifoOpen failed" );
+        hb_log( "dvd: not a dvd - trying as a stream/file instead" );
         goto fail;
     }
 
@@ -172,6 +173,117 @@ static int hb_dvdread_title_count( hb_dvd_t * e )
     return d->vmg->tt_srpt->nr_of_srpts;
 }
 
+static void add_subtitle( hb_list_t * list_subtitle, int position,
+                          iso639_lang_t * lang, int lang_extension,
+                          uint32_t * palette, int style )
+{
+    hb_subtitle_t * subtitle;
+    int ii, count;
+
+    count = hb_list_count(list_subtitle);
+    for (ii = 0; ii < count; ii++)
+    {
+        subtitle = hb_list_item(list_subtitle, ii);
+        if (((subtitle->id >> 8) & 0x1f) == position)
+        {
+            // The subtitle is already in the list
+            return;
+        }
+    }
+
+    subtitle        = calloc(sizeof(hb_subtitle_t), 1);
+    subtitle->track = count;
+    subtitle->id    = ((0x20 + position) << 8) | 0xbd;
+    snprintf(subtitle->lang, sizeof( subtitle->lang ), "%s",
+             strlen(lang->native_name) ? lang->native_name : lang->eng_name);
+    snprintf(subtitle->iso639_2, sizeof( subtitle->iso639_2 ), "%s",
+             lang->iso639_2);
+    subtitle->format         = PICTURESUB;
+    subtitle->source         = VOBSUB;
+    subtitle->config.dest    = RENDERSUB;
+    subtitle->stream_type    = 0xbd;
+    subtitle->substream_type = 0x20 + position;
+    subtitle->codec          = WORK_DECVOBSUB;
+
+    memcpy(subtitle->palette, palette, 16 * sizeof(uint32_t));
+    subtitle->palette_set = 1;
+
+    switch (lang_extension)
+    {
+        case 1:
+            subtitle->attributes = HB_SUBTITLE_ATTR_NORMAL;
+            break;
+        case 2:
+            subtitle->attributes = HB_SUBTITLE_ATTR_LARGE;
+            strcat(subtitle->lang, " Large Type");
+            break;
+        case 3:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CHILDREN;
+            strcat(subtitle->lang, " Children");
+            break;
+        case 5:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CC;
+            strcat(subtitle->lang, " Closed Caption");
+            break;
+        case 6:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CC | HB_SUBTITLE_ATTR_LARGE;
+            strcat(subtitle->lang, " Closed Caption, Large Type");
+            break;
+        case 7:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CC |
+                                   HB_SUBTITLE_ATTR_CHILDREN;
+            strcat(subtitle->lang, " Closed Caption, Children");
+            break;
+        case 9:
+            subtitle->attributes = HB_SUBTITLE_ATTR_FORCED;
+            strcat(subtitle->lang, " Forced");
+            break;
+        case 13:
+            subtitle->attributes = HB_SUBTITLE_ATTR_COMMENTARY;
+            strcat(subtitle->lang, " Director's Commentary");
+            break;
+        case 14:
+            subtitle->attributes = HB_SUBTITLE_ATTR_COMMENTARY |
+                                   HB_SUBTITLE_ATTR_LARGE;
+            strcat(subtitle->lang, " Director's Commentary, Large Type");
+            break;
+        case 15:
+            subtitle->attributes = HB_SUBTITLE_ATTR_COMMENTARY |
+                                   HB_SUBTITLE_ATTR_CHILDREN;
+            strcat(subtitle->lang, " Director's Commentary, Children");
+        default:
+            subtitle->attributes = HB_SUBTITLE_ATTR_UNKNOWN;
+            break;
+    }
+    switch (style)
+    {
+        case HB_VOBSUB_STYLE_4_3:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_4_3;
+            strcat(subtitle->lang, " (4:3)");
+            break;
+        case HB_VOBSUB_STYLE_WIDE:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_WIDE;
+            strcat(subtitle->lang, " (Wide Screen)");
+            break;
+        case HB_VOBSUB_STYLE_LETTERBOX:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_LETTERBOX;
+            strcat(subtitle->lang, " (Letterbox)");
+            break;
+        case HB_VOBSUB_STYLE_PANSCAN:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_PANSCAN;
+            strcat(subtitle->lang, " (Pan & Scan)");
+            break;
+    }
+    strcat(subtitle->lang, " [");
+    strcat(subtitle->lang, hb_subsource_name(subtitle->source));
+    strcat(subtitle->lang, "]");
+
+    hb_log("scan: id=0x%x, lang=%s, 3cc=%s ext=%i", subtitle->id,
+           subtitle->lang, subtitle->iso639_2, lang_extension);
+
+    hb_list_add(list_subtitle, subtitle);
+}
+
 /***********************************************************************
  * hb_dvdread_title_scan
  **********************************************************************/
@@ -431,7 +543,24 @@ static hb_title_t * hb_dvdread_title_scan( hb_dvd_t * e, int t, uint64_t min_dur
 
         lang = lang_for_code( lang_code );
 
-        audio->config.lang.type = lang_extension;
+        switch ( lang_extension )
+        {
+            case 1:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_NORMAL;
+                break;
+            case 2:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_VISUALLY_IMPAIRED;
+                break;
+            case 3:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_COMMENTARY;
+                break;
+            case 4:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_ALT_COMMENTARY;
+                break;
+            default:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_NONE;
+                break;
+        }
 
         snprintf( audio->config.lang.simple,
                   sizeof( audio->config.lang.simple ), "%s",
@@ -450,14 +579,17 @@ static hb_title_t * hb_dvdread_title_scan( hb_dvd_t * e, int t, uint64_t min_dur
     /* Check for subtitles */
     for( i = 0; i < vts->vtsi_mat->nr_of_vts_subp_streams; i++ )
     {
-        hb_subtitle_t * subtitle;
-        int spu_control;
-        int position;
+        int             spu_control, pos, lang_ext = 0;
         iso639_lang_t * lang;
-        int lang_extension = 0;
 
         hb_log( "scan: checking subtitle %d", i + 1 );
 
+        // spu_control
+        // 0x80000000 - Subtitle enabled
+        // 0x1f000000 - Position mask for 4:3 aspect subtitle track
+        // 0x001f0000 - Position mask for Wide Screen subtitle track
+        // 0x00001f00 - Position mask for Letterbox subtitle track
+        // 0x0000001f - Position mask for Pan&Scan subtitle track
         spu_control =
             vts->vts_pgcit->pgci_srp[pgc_id-1].pgc->subp_control[i];
 
@@ -467,86 +599,49 @@ static hb_title_t * hb_dvdread_title_scan( hb_dvd_t * e, int t, uint64_t min_dur
             continue;
         }
 
-        if( vts->vtsi_mat->vts_video_attr.display_aspect_ratio )
+        lang_ext = vts->vtsi_mat->vts_subp_attr[i].code_extension;
+        lang     = lang_for_code(vts->vtsi_mat->vts_subp_attr[i].lang_code);
+
+        // display_aspect_ratio
+        // 0     = 4:3
+        // 3     = 16:9
+        // other = invalid
+        if (vts->vtsi_mat->vts_video_attr.display_aspect_ratio)
         {
-            switch( vts->vtsi_mat->vts_video_attr.permitted_df )
+            // Add Wide Screen subtitle.
+            pos = (spu_control >> 16) & 0x1F;
+            add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                       vts->vts_pgcit->pgci_srp[pgc_id-1].pgc->palette,
+                       HB_VOBSUB_STYLE_WIDE);
+
+            // permitted_df
+            // 1 - Letterbox not permitted
+            // 2 - Pan&Scan not permitted
+            // 3 - Letterbox and Pan&Scan not permitted
+            if (!(vts->vtsi_mat->vts_video_attr.permitted_df & 1))
             {
-                case 1:
-                    position = spu_control & 0xFF;
-                    break;
-                case 2:
-                    position = ( spu_control >> 8 ) & 0xFF;
-                    break;
-                default:
-                    position = ( spu_control >> 16 ) & 0xFF;
+                // Letterbox permitted.  Add Letterbox subtitle.
+                pos = (spu_control >> 8) & 0x1F;
+                add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                           vts->vts_pgcit->pgci_srp[pgc_id-1].pgc->palette,
+                           HB_VOBSUB_STYLE_LETTERBOX);
+            }
+            if (!(vts->vtsi_mat->vts_video_attr.permitted_df & 2))
+            {
+                // Pan&Scan permitted.  Add Pan&Scan subtitle.
+                pos = spu_control & 0x1F;
+                add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                           vts->vts_pgcit->pgci_srp[pgc_id-1].pgc->palette,
+                           HB_VOBSUB_STYLE_PANSCAN);
             }
         }
         else
         {
-            position = ( spu_control >> 24 ) & 0x7F;
+            pos = (spu_control >> 24) & 0x1F;
+            add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                       vts->vts_pgcit->pgci_srp[pgc_id-1].pgc->palette,
+                       HB_VOBSUB_STYLE_4_3);
         }
-
-        lang_extension = vts->vtsi_mat->vts_subp_attr[i].code_extension;
-
-        lang = lang_for_code( vts->vtsi_mat->vts_subp_attr[i].lang_code );
-
-        subtitle = calloc( sizeof( hb_subtitle_t ), 1 );
-        subtitle->track = i+1;
-        subtitle->id = ( ( 0x20 + position ) << 8 ) | 0xbd;
-        snprintf( subtitle->lang, sizeof( subtitle->lang ), "%s",
-             strlen(lang->native_name) ? lang->native_name : lang->eng_name);
-        snprintf( subtitle->iso639_2, sizeof( subtitle->iso639_2 ), "%s",
-                  lang->iso639_2);
-        subtitle->format = PICTURESUB;
-        subtitle->source = VOBSUB;
-        subtitle->config.dest   = RENDERSUB;  // By default render (burn-in) the VOBSUB.
-        subtitle->stream_type = 0xbd;
-        subtitle->substream_type = 0x20 + position;
-        subtitle->codec = WORK_DECVOBSUB;
-
-        subtitle->type = lang_extension;
-        
-        memcpy( subtitle->palette,
-            vts->vts_pgcit->pgci_srp[pgc_id-1].pgc->palette,
-            16 * sizeof( uint32_t ) );
-        subtitle->palette_set = 1;
-
-        switch( lang_extension )
-        {  
-            case 2:
-                strcat( subtitle->lang, " (Caption with bigger size character)" );
-                break;
-            case 3: 
-                strcat( subtitle->lang, " (Caption for Children)" );
-                break;
-            case 5:
-                strcat( subtitle->lang, " (Closed Caption)" );
-                break;
-            case 6:
-                strcat( subtitle->lang, " (Closed Caption with bigger size character)" );
-                break;
-            case 7:
-                strcat( subtitle->lang, " (Closed Caption for Children)" );
-                break;
-            case 9:
-                strcat( subtitle->lang, " (Forced Caption)" );
-                break;
-            case 13:
-                strcat( subtitle->lang, " (Director's Commentary)" );
-                break;
-            case 14:
-                strcat( subtitle->lang, " (Director's Commentary with bigger size character)" );
-                break;
-            case 15:
-                strcat( subtitle->lang, " (Director's Commentary for Children)" );
-            default:
-                break;
-        }
-
-        hb_log( "scan: id=0x%x, lang=%s, 3cc=%s ext=%i", subtitle->id,
-                subtitle->lang, subtitle->iso639_2, lang_extension );
-
-        hb_list_add( title->list_subtitle, subtitle );
     }
 
     /* Chapters */
@@ -623,17 +718,20 @@ static hb_title_t * hb_dvdread_title_scan( hb_dvd_t * e, int t, uint64_t min_dur
     switch( vts->vtsi_mat->vts_video_attr.display_aspect_ratio )
     {
         case 0:
-            title->container_aspect = 4. / 3.;
+            title->container_dar.num = 4;
+            title->container_dar.den = 3;
             break;
         case 3:
-            title->container_aspect = 16. / 9.;
+            title->container_dar.num = 16;
+            title->container_dar.den = 9;
             break;
         default:
             hb_log( "scan: unknown aspect" );
             goto fail;
     }
 
-    hb_log( "scan: aspect = %g", title->container_aspect );
+    hb_log("scan: aspect = %d:%d",
+           title->container_dar.num, title->container_dar.den);
 
     /* This title is ok so far */
     goto cleanup;
@@ -887,6 +985,7 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
                 if ( d->cell_cur > d->cell_end )
                 {
                     hb_buffer_close( &b );
+                    hb_set_work_error(d->h, HB_ERROR_READ);
                     return NULL;
                 }
                 d->in_cell = 0;
@@ -985,8 +1084,9 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
             /* Wasn't a valid VOBU, try next block */
             if( ++error > 1024 )
             {
-                hb_log( "dvd: couldn't find a VOBU after 1024 blocks" );
+                hb_error( "dvd: couldn't find a VOBU after 1024 blocks" );
                 hb_buffer_close( &b );
+                hb_set_work_error(d->h, HB_ERROR_READ);
                 return NULL;
             }
 
@@ -1050,11 +1150,7 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
                 d->cur_vob_id = dsi_pack.dsi_gi.vobu_vob_idn;
                 d->cur_cell_id = dsi_pack.dsi_gi.vobu_c_idn;
 
-                if( d->cell_overlap )
-                {
-                    b->s.new_chap = hb_dvdread_is_break( d );
-                    d->cell_overlap = 0;
-                }
+                d->cell_overlap = 0;
             }
         }
 
@@ -1089,6 +1185,10 @@ static hb_buffer_t * hb_dvdread_read( hb_dvd_t * e )
         }
         d->pack_len--;
     }
+    if (b != NULL)
+    {
+        b->s.new_chap = hb_dvdread_is_break( d );
+    }
 
     d->block++;
 
@@ -1143,9 +1243,7 @@ static int hb_dvdread_is_break( hb_dvdread_t * d )
     pgc_t * pgc;
     int     cell;
 
-    for( i = nr_of_ptts - 1;
-         i > 0;
-         i-- )
+    for (i = nr_of_ptts - 1; i >= 0; i--)
     {
         /* Get pgc for chapter (i+1) */
         pgc_id = d->ifo->vts_ptt_srpt->title[d->ttn-1].ptt[i].pgcn;
@@ -1153,11 +1251,11 @@ static int hb_dvdread_is_break( hb_dvdread_t * d )
         pgc    = d->ifo->vts_pgcit->pgci_srp[pgc_id-1].pgc;
         cell   = pgc->program_map[pgn-1] - 1;
 
-        if( cell <= d->cell_start )
+        if( cell < d->cell_start )
             break;
 
         // This must not match against the start cell.
-        if( pgc->cell_playback[cell].first_sector == d->block && cell != d->cell_start )
+        if (pgc->cell_playback[cell].first_sector == d->block)
         {
             return i + 1;
         }
@@ -1266,9 +1364,9 @@ char * hb_dvd_name( char * path )
     return dvd_methods->name(path);
 }
 
-hb_dvd_t * hb_dvd_init( char * path )
+hb_dvd_t * hb_dvd_init( hb_handle_t * h, char * path )
 {
-    return dvd_methods->init(path);
+    return dvd_methods->init(h, path);
 }
 
 int hb_dvd_title_count( hb_dvd_t * d )
