@@ -1,6 +1,6 @@
 /* dvdnav.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -19,7 +19,7 @@
 #define DVD_READ_CACHE 1
 
 static char        * hb_dvdnav_name( char * path );
-static hb_dvd_t    * hb_dvdnav_init( char * path );
+static hb_dvd_t    * hb_dvdnav_init( hb_handle_t * h, char * path );
 static int           hb_dvdnav_title_count( hb_dvd_t * d );
 static hb_title_t  * hb_dvdnav_title_scan( hb_dvd_t * d, int t, uint64_t min_duration );
 static int           hb_dvdnav_start( hb_dvd_t * d, hb_title_t *title, int chapter );
@@ -148,7 +148,7 @@ fail:
  ***********************************************************************
  *
  **********************************************************************/
-static hb_dvd_t * hb_dvdnav_init( char * path )
+static hb_dvd_t * hb_dvdnav_init( hb_handle_t * h, char * path )
 {
     hb_dvd_t * e;
     hb_dvdnav_t * d;
@@ -157,6 +157,7 @@ static hb_dvd_t * hb_dvdnav_init( char * path )
 
     e = calloc( sizeof( hb_dvd_t ), 1 );
     d = &(e->dvdnav);
+    d->h = h;
 
     /*
      * Convert UTF-8 path to current code page on Windows
@@ -316,6 +317,117 @@ PttDuration(ifo_handle_t *ifo, int ttn, int pttn, int *blocks, int *last_pgcn)
     return duration;
 }
 
+static void add_subtitle( hb_list_t * list_subtitle, int position,
+                          iso639_lang_t * lang, int lang_extension,
+                          uint32_t * palette, int style )
+{
+    hb_subtitle_t * subtitle;
+    int ii, count;
+
+    count = hb_list_count(list_subtitle);
+    for (ii = 0; ii < count; ii++)
+    {
+        subtitle = hb_list_item(list_subtitle, ii);
+        if (((subtitle->id >> 8) & 0x1f) == position)
+        {
+            // The subtitle is already in the list
+            return;
+        }
+    }
+
+    subtitle        = calloc(sizeof(hb_subtitle_t), 1);
+    subtitle->track = count;
+    subtitle->id    = ((0x20 + position) << 8) | 0xbd;
+    snprintf(subtitle->lang, sizeof( subtitle->lang ), "%s",
+             strlen(lang->native_name) ? lang->native_name : lang->eng_name);
+    snprintf(subtitle->iso639_2, sizeof( subtitle->iso639_2 ), "%s",
+             lang->iso639_2);
+    subtitle->format         = PICTURESUB;
+    subtitle->source         = VOBSUB;
+    subtitle->config.dest    = RENDERSUB;
+    subtitle->stream_type    = 0xbd;
+    subtitle->substream_type = 0x20 + position;
+    subtitle->codec          = WORK_DECVOBSUB;
+
+    memcpy(subtitle->palette, palette, 16 * sizeof(uint32_t));
+    subtitle->palette_set = 1;
+
+    switch (lang_extension)
+    {
+        case 1:
+            subtitle->attributes = HB_SUBTITLE_ATTR_NORMAL;
+            break;
+        case 2:
+            subtitle->attributes = HB_SUBTITLE_ATTR_LARGE;
+            strcat(subtitle->lang, " Large Type");
+            break;
+        case 3:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CHILDREN;
+            strcat(subtitle->lang, " Children");
+            break;
+        case 5:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CC;
+            strcat(subtitle->lang, " Closed Caption");
+            break;
+        case 6:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CC | HB_SUBTITLE_ATTR_LARGE;
+            strcat(subtitle->lang, " Closed Caption, Large Type");
+            break;
+        case 7:
+            subtitle->attributes = HB_SUBTITLE_ATTR_CC |
+                                   HB_SUBTITLE_ATTR_CHILDREN;
+            strcat(subtitle->lang, " Closed Caption, Children");
+            break;
+        case 9:
+            subtitle->attributes = HB_SUBTITLE_ATTR_FORCED;
+            strcat(subtitle->lang, " Forced");
+            break;
+        case 13:
+            subtitle->attributes = HB_SUBTITLE_ATTR_COMMENTARY;
+            strcat(subtitle->lang, " Director's Commentary");
+            break;
+        case 14:
+            subtitle->attributes = HB_SUBTITLE_ATTR_COMMENTARY |
+                                   HB_SUBTITLE_ATTR_LARGE;
+            strcat(subtitle->lang, " Director's Commentary, Large Type");
+            break;
+        case 15:
+            subtitle->attributes = HB_SUBTITLE_ATTR_COMMENTARY |
+                                   HB_SUBTITLE_ATTR_CHILDREN;
+            strcat(subtitle->lang, " Director's Commentary, Children");
+        default:
+            subtitle->attributes = HB_SUBTITLE_ATTR_UNKNOWN;
+            break;
+    }
+    switch (style)
+    {
+        case HB_VOBSUB_STYLE_4_3:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_4_3;
+            strcat(subtitle->lang, " (4:3)");
+            break;
+        case HB_VOBSUB_STYLE_WIDE:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_WIDE;
+            strcat(subtitle->lang, " (Wide Screen)");
+            break;
+        case HB_VOBSUB_STYLE_LETTERBOX:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_LETTERBOX;
+            strcat(subtitle->lang, " (Letterbox)");
+            break;
+        case HB_VOBSUB_STYLE_PANSCAN:
+            subtitle->attributes |= HB_SUBTITLE_ATTR_PANSCAN;
+            strcat(subtitle->lang, " (Pan & Scan)");
+            break;
+    }
+    strcat(subtitle->lang, " [");
+    strcat(subtitle->lang, hb_subsource_name(subtitle->source));
+    strcat(subtitle->lang, "]");
+
+    hb_log("scan: id=0x%x, lang=%s, 3cc=%s ext=%i", subtitle->id,
+           subtitle->lang, subtitle->iso639_2, lang_extension);
+
+    hb_list_add(list_subtitle, subtitle);
+}
+
 /***********************************************************************
  * hb_dvdnav_title_scan
  **********************************************************************/
@@ -334,6 +446,7 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
     uint64_t       duration, longest;
     int            longest_pgcn, longest_pgn, longest_pgcn_end;
     const char   * name;
+    unsigned char  unused[1024];
     const char   * codec_name;
 
     hb_log( "scan: scanning title %d", t );
@@ -344,8 +457,13 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
     {
         strncpy( title->name, name, sizeof( title->name ) );
     }
-    else
+
+    if (strlen(title->name) == 0)
     {
+        if( DVDUDFVolumeInfo( d->reader, title->name, sizeof( title->name ),
+                             unused, sizeof( unused ) ) )
+        {
+
         char * p_cur, * p_last = d->path;
         for( p_cur = d->path; *p_cur; p_cur++ )
         {
@@ -358,6 +476,7 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
         char *dot_term = strrchr(title->name, '.');
         if (dot_term)
             *dot_term = '\0';
+        }
     }
 
     /* VTS which our title is in */
@@ -583,6 +702,24 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
         {
             continue;
         }
+        switch ( lang_extension )
+        {
+            case 1:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_NORMAL;
+                break;
+            case 2:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_VISUALLY_IMPAIRED;
+                break;
+            case 3:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_COMMENTARY;
+                break;
+            case 4:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_ALT_COMMENTARY;
+                break;
+            default:
+                audio->config.lang.attributes = HB_AUDIO_ATTR_NONE;
+                break;
+        }
 
         /* Check for duplicate tracks */
         audio_tmp = NULL;
@@ -604,7 +741,6 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
 
         lang = lang_for_code( lang_code );
 
-        audio->config.lang.type = lang_extension;
 
         snprintf( audio->config.lang.simple,
                   sizeof( audio->config.lang.simple ), "%s",
@@ -623,14 +759,17 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
     /* Check for subtitles */
     for( i = 0; i < ifo->vtsi_mat->nr_of_vts_subp_streams; i++ )
     {
-        hb_subtitle_t * subtitle;
-        int spu_control;
-        int position;
+        int             spu_control, pos, lang_ext = 0;
         iso639_lang_t * lang;
-        int lang_extension = 0;
 
         hb_log( "scan: checking subtitle %d", i + 1 );
 
+        // spu_control
+        // 0x80000000 - Subtitle enabled
+        // 0x1f000000 - Position mask for 4:3 aspect subtitle track
+        // 0x001f0000 - Position mask for Wide Screen subtitle track
+        // 0x00001f00 - Position mask for Letterbox subtitle track
+        // 0x0000001f - Position mask for Pan&Scan subtitle track
         spu_control =
             ifo->vts_pgcit->pgci_srp[title_pgcn-1].pgc->subp_control[i];
 
@@ -640,86 +779,49 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
             continue;
         }
 
-        if( ifo->vtsi_mat->vts_video_attr.display_aspect_ratio )
+        lang_ext = ifo->vtsi_mat->vts_subp_attr[i].code_extension;
+        lang     = lang_for_code(ifo->vtsi_mat->vts_subp_attr[i].lang_code);
+
+        // display_aspect_ratio
+        // 0     = 4:3
+        // 3     = 16:9
+        // other = invalid
+        if (ifo->vtsi_mat->vts_video_attr.display_aspect_ratio)
         {
-            switch( ifo->vtsi_mat->vts_video_attr.permitted_df )
+            // Add Wide Screen subtitle.
+            pos = (spu_control >> 16) & 0x1F;
+            add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                       ifo->vts_pgcit->pgci_srp[title_pgcn-1].pgc->palette,
+                       HB_VOBSUB_STYLE_WIDE);
+
+            // permitted_df
+            // 1 - Letterbox not permitted
+            // 2 - Pan&Scan not permitted
+            // 3 - Letterbox and Pan&Scan not permitted
+            if (!(ifo->vtsi_mat->vts_video_attr.permitted_df & 1))
             {
-                case 1:
-                    position = spu_control & 0xFF;
-                    break;
-                case 2:
-                    position = ( spu_control >> 8 ) & 0xFF;
-                    break;
-                default:
-                    position = ( spu_control >> 16 ) & 0xFF;
+                // Letterbox permitted.  Add Letterbox subtitle.
+                pos = (spu_control >> 8) & 0x1F;
+                add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                           ifo->vts_pgcit->pgci_srp[title_pgcn-1].pgc->palette,
+                           HB_VOBSUB_STYLE_LETTERBOX);
+            }
+            if (!(ifo->vtsi_mat->vts_video_attr.permitted_df & 2))
+            {
+                // Pan&Scan permitted.  Add Pan&Scan subtitle.
+                pos = spu_control & 0x1F;
+                add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                           ifo->vts_pgcit->pgci_srp[title_pgcn-1].pgc->palette,
+                           HB_VOBSUB_STYLE_PANSCAN);
             }
         }
         else
         {
-            position = ( spu_control >> 24 ) & 0x7F;
+            pos = (spu_control >> 24) & 0x1F;
+            add_subtitle(title->list_subtitle, pos, lang, lang_ext,
+                       ifo->vts_pgcit->pgci_srp[title_pgcn-1].pgc->palette,
+                           HB_VOBSUB_STYLE_4_3);
         }
-
-        lang_extension = ifo->vtsi_mat->vts_subp_attr[i].code_extension;
-
-        lang = lang_for_code( ifo->vtsi_mat->vts_subp_attr[i].lang_code );
-
-        subtitle = calloc( sizeof( hb_subtitle_t ), 1 );
-        subtitle->track = i+1;
-        subtitle->id = ( ( 0x20 + position ) << 8 ) | 0xbd;
-        snprintf( subtitle->lang, sizeof( subtitle->lang ), "%s",
-             strlen(lang->native_name) ? lang->native_name : lang->eng_name);
-        snprintf( subtitle->iso639_2, sizeof( subtitle->iso639_2 ), "%s",
-                  lang->iso639_2);
-        subtitle->format = PICTURESUB;
-        subtitle->source = VOBSUB;
-        subtitle->config.dest   = RENDERSUB;  // By default render (burn-in) the VOBSUB.
-        subtitle->stream_type = 0xbd;
-        subtitle->substream_type = 0x20 + position;
-        subtitle->codec = WORK_DECVOBSUB;
-
-        subtitle->type = lang_extension;
-        
-        memcpy( subtitle->palette,
-            ifo->vts_pgcit->pgci_srp[title_pgcn-1].pgc->palette,
-            16 * sizeof( uint32_t ) );
-        subtitle->palette_set = 1;
-
-        switch( lang_extension )
-        {  
-            case 2:
-                strcat( subtitle->lang, " (Caption with bigger size character)" );
-                break;
-            case 3: 
-                strcat( subtitle->lang, " (Caption for Children)" );
-                break;
-            case 5:
-                strcat( subtitle->lang, " (Closed Caption)" );
-                break;
-            case 6:
-                strcat( subtitle->lang, " (Closed Caption with bigger size character)" );
-                break;
-            case 7:
-                strcat( subtitle->lang, " (Closed Caption for Children)" );
-                break;
-            case 9:
-                strcat( subtitle->lang, " (Forced Caption)" );
-                break;
-            case 13:
-                strcat( subtitle->lang, " (Director's Commentary)" );
-                break;
-            case 14:
-                strcat( subtitle->lang, " (Director's Commentary with bigger size character)" );
-                break;
-            case 15:
-                strcat( subtitle->lang, " (Director's Commentary for Children)" );
-            default:
-                break;
-        }
-
-        hb_log( "scan: id=0x%x, lang=%s, 3cc=%s ext=%i", subtitle->id,
-                subtitle->lang, subtitle->iso639_2, lang_extension );
-
-        hb_list_add( title->list_subtitle, subtitle );
     }
 
     /* Chapters */
@@ -809,17 +911,20 @@ static hb_title_t * hb_dvdnav_title_scan( hb_dvd_t * e, int t, uint64_t min_dura
     switch( ifo->vtsi_mat->vts_video_attr.display_aspect_ratio )
     {
         case 0:
-            title->container_aspect = 4. / 3.;
+            title->container_dar.num = 4;
+            title->container_dar.den = 3;
             break;
         case 3:
-            title->container_aspect = 16. / 9.;
+            title->container_dar.num = 16;
+            title->container_dar.den = 9;
             break;
         default:
             hb_log( "scan: unknown aspect" );
             goto fail;
     }
 
-    hb_log( "scan: aspect = %g", title->container_aspect );
+    hb_log("scan: aspect = %d:%d",
+           title->container_dar.num, title->container_dar.den);
 
     /* This title is ok so far */
     goto cleanup;
@@ -1270,7 +1375,7 @@ static int try_menu(
                 break;
             }
         }
-        // Sometimes the menu is preceeded by a intro that just
+        // Sometimes the menu is preceded by a intro that just
         // gets restarted when hitting the menu button. So
         // try skipping with the skip forward button.  Then
         // try hitting the menu again.
@@ -1312,7 +1417,7 @@ static int hb_dvdnav_main_feature( hb_dvd_t * e, hb_list_t * list_title )
             longest_duration_fallback = title->duration;
             longest_fallback = title->index;
         }
-        if ( title->duration > 90000L * 60 * 30 )
+        if ( title->duration > 90000LL * 60 * 30 )
         {
             avg_duration += title->duration;
             avg_cnt++;
@@ -1408,7 +1513,7 @@ static int hb_dvdnav_main_feature( hb_dvd_t * e, hb_list_t * list_title )
         longest = longest_title;
     }
     if ((float)longest_duration_fallback * 0.7 > longest_duration &&
-        longest_duration < 90000L * 60 * 30 )
+        longest_duration < 90000LL * 60 * 30 )
     {
         float factor = (float)avg_duration / longest_duration;
         if ( factor > 1 )
@@ -1617,6 +1722,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 hb_error( "dvd: dvdnav_sector_search failed - %s",
                         dvdnav_err_to_string(d->dvdnav) );
                 hb_buffer_close( &b );
+                hb_set_work_error(d->h, HB_ERROR_READ);
                 return NULL;
             }
             error_count++;
@@ -1624,6 +1730,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
             {
                 hb_error("dvdnav: Error, too many consecutive read errors");
                 hb_buffer_close( &b );
+                hb_set_work_error(d->h, HB_ERROR_READ);
                 return NULL;
             }
             continue;
@@ -1633,11 +1740,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
         case DVDNAV_BLOCK_OK:
             // We have received a regular block of the currently playing
             // MPEG stream.
-
-            // The muxers expect to only get chapter 2 and above
-            // They write chapter 1 when chapter 2 is detected.
-            if (chapter > 1)
-                b->s.new_chap = chapter;
+            b->s.new_chap = chapter;
             chapter = 0;
             error_count = 0;
             return b;
@@ -1716,6 +1819,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 {
                     // Transition to another title signals that we are done.
                     hb_buffer_close( &b );
+                    hb_deep_log(2, "dvdnav: vts change, found next title");
                     return NULL;
                 }
             }
@@ -1739,6 +1843,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 {
                     // Transition to another title signals that we are done.
                     hb_buffer_close( &b );
+                    hb_deep_log(2, "dvdnav: cell change, found next title");
                     return NULL;
                 }
                 c = FindChapterIndex(d->list_chapter, pgcn, pgn);
@@ -1749,6 +1854,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                         // Some titles end with a 'link' back to the beginning so
                         // a transition to an earlier chapter means we're done.
                         hb_buffer_close( &b );
+                        hb_deep_log(2, "dvdnav: cell change, previous chapter");
                         return NULL;
                     }
                     chapter = d->chapter = c;
@@ -1756,6 +1862,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
                 else if ( cell_event->cellN <= d->cell )
                 {
                     hb_buffer_close( &b );
+                    hb_deep_log(2, "dvdnav: cell change, previous cell");
                     return NULL;
                 }
                 d->cell = cell_event->cellN;
@@ -1774,12 +1881,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
 
             // mpegdemux expects to get these.  I don't think it does
             // anything useful with them however.
-
-            // The muxers expect to only get chapter 2 and above
-            // They write chapter 1 when chapter 2 is detected.
-            if (chapter > 1)
-                b->s.new_chap = chapter;
-            chapter = 0;
+            b->s.new_chap = chapter;
             return b;
 
             break;
@@ -1798,6 +1900,7 @@ static hb_buffer_t * hb_dvdnav_read( hb_dvd_t * e )
             */
             d->stopped = 1;
             hb_buffer_close( &b );
+            hb_deep_log(2, "dvdnav: stop");
             return NULL;
 
         default:
