@@ -1,6 +1,6 @@
 /* fifo.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -8,15 +8,29 @@
  */
 
 #include "hb.h"
-#include "openclwrapper.h"
+#ifdef USE_QSV
+#include "qsv_libav.h"
+#endif
 
 #ifndef SYS_DARWIN
+#if defined( SYS_FREEBSD ) || defined ( __FreeBSD__ )
+#include <stdlib.h>
+#else
 #include <malloc.h>
 #endif
+#endif
 
 #define FIFO_TIMEOUT 200
 //#define HB_FIFO_DEBUG 1
+// defining HB_BUFFER_DEBUG and HB_NO_BUFFER_POOL allows tracking
+// buffer memory leaks using valgrind.  The source of the leak
+// can be determined with "valgrind --leak-check=full"
 //#define HB_BUFFER_DEBUG 1
+//#define HB_NO_BUFFER_POOL 1
+
+#if defined(HB_BUFFER_DEBUG)
+#include <assert.h>
+#endif
 
 /* Fifo */
 struct hb_fifo_s
@@ -26,6 +40,7 @@ struct hb_fifo_s
     int            wait_full;
     hb_cond_t    * cond_empty;
     int            wait_empty;
+    hb_cond_t    * cond_alert_full;
     uint32_t       capacity;
     uint32_t       thresh;
     uint32_t       size;
@@ -64,13 +79,19 @@ struct hb_buffer_pools_s
 {
     int64_t allocated;
     hb_lock_t *lock;
+#if !defined(HB_NO_BUFFER_POOL)
     hb_fifo_t *pool[MAX_BUFFER_POOLS];
+#endif
 #if defined(HB_BUFFER_DEBUG)
     hb_list_t *alloc_list;
 #endif
 } buffers;
 
 
+#if defined(HB_BUFFER_DEBUG)
+static int hb_fifo_contains( hb_fifo_t *f, hb_buffer_t *b );
+#endif
+
 void hb_buffer_pool_init( void )
 {
     buffers.lock = hb_lock_init();
@@ -80,6 +101,7 @@ void hb_buffer_pool_init( void )
     buffers.alloc_list = hb_list_init();
 #endif
 
+#if !defined(HB_NO_BUFFER_POOL)
     /* we allocate pools for sizes 2^10 through 2^25. requests larger than
      * 2^25 will get passed through to malloc. */
     int i;
@@ -99,6 +121,7 @@ void hb_buffer_pool_init( void )
         buffers.pool[i] = hb_fifo_init(BUFFER_POOL_MAX_ELEMENTS, 1);
         buffers.pool[i]->buffer_size = 1 << i;
     }
+#endif
 }
 
 #if defined(HB_FIFO_DEBUG)
@@ -144,6 +167,7 @@ static void fifo_list_rem( hb_fifo_t * f )
     }
 }
 
+#if !defined(HB_NO_BUFFER_POOL)
 // These routines are useful for finding and debugging problems
 // with the fifos and buffer pools
 static void buffer_pool_validate( hb_fifo_t * f )
@@ -241,13 +265,12 @@ void fifo_list_validate( void )
     }
 }
 #endif
+#endif
 
 void hb_buffer_pool_free( void )
 {
     int i;
-    int count;
     int64_t freed = 0;
-    hb_buffer_t *b;
 
     hb_lock(buffers.lock);
 
@@ -261,6 +284,9 @@ void hb_buffer_pool_free( void )
     }
 #endif
 
+#if !defined(HB_NO_BUFFER_POOL)
+    hb_buffer_t * b;
+    int           count;
     for( i = BUFFER_POOL_FIRST; i <= BUFFER_POOL_LAST; ++i)
     {
         count = 0;
@@ -269,20 +295,7 @@ void hb_buffer_pool_free( void )
             if( b->data )
             {
                 freed += b->alloc;
-
-                if (b->cl.buffer != NULL)
-                {
-                    /* OpenCL */
-                    if (hb_cl_free_mapped_buffer(b->cl.buffer, b->data) == 0)
-                    {
-                        hb_log("hb_buffer_pool_free: bad free: %p -> buffer %p map %p",
-                               b, b->cl.buffer, b->data);
-                    }
-                }
-                else
-                {
-                    free(b->data);
-                }
+                free(b->data);
             }
             free( b );
             count++;
@@ -293,6 +306,18 @@ void hb_buffer_pool_free( void )
                     buffers.pool[i]->buffer_size);
         }
     }
+#endif
+
+#if defined(HB_BUFFER_DEBUG) && defined(HB_NO_BUFFER_POOL)
+    // defining HB_BUFFER_DEBUG and HB_NO_BUFFER_POOL allows tracking
+    // buffer memory leaks using valgrind.  The source of the leak
+    // can be determined with "valgrind --leak-check=full"
+    for (i = 0; i < hb_list_count(buffers.alloc_list); i++)
+    {
+        hb_buffer_t *b = hb_list_item(buffers.alloc_list, i);
+        hb_list_rem(buffers.alloc_list, b);
+    }
+#endif
 
     hb_deep_log( 2, "Allocated %"PRId64" bytes of buffers on this pass and Freed %"PRId64" bytes, "
            "%"PRId64" bytes leaked", buffers.allocated, freed, buffers.allocated - freed);
@@ -302,6 +327,7 @@ void hb_buffer_pool_free( void )
 
 static hb_fifo_t *size_to_pool( int size )
 {
+#if !defined(HB_NO_BUFFER_POOL)
     int i;
     for ( i = BUFFER_POOL_FIRST; i <= BUFFER_POOL_LAST; ++i )
     {
@@ -310,10 +336,11 @@ static hb_fifo_t *size_to_pool( int size )
             return buffers.pool[i];
         }
     }
+#endif
     return NULL;
 }
 
-hb_buffer_t * hb_buffer_init_internal( int size , int needsMapped )
+hb_buffer_t * hb_buffer_init_internal( int size )
 {
     hb_buffer_t * b;
     // Certain libraries (hrm ffmpeg) expect buffers passed to them to
@@ -328,20 +355,6 @@ hb_buffer_t * hb_buffer_init_internal( int size , int needsMapped )
     {
         b = hb_fifo_get( buffer_pool );
 
-        /* OpenCL */
-        if (b != NULL && needsMapped && b->cl.buffer == NULL)
-        {
-            // We need a mapped OpenCL buffer and that is not
-            // what we got out of the pool.
-            // Ditch it; it will get replaced with what we need.
-            if (b->data != NULL)
-            {
-                free(b->data);
-            }
-            free(b);
-            b = NULL;
-        }
-
         if( b )
         {
             /*
@@ -350,23 +363,14 @@ hb_buffer_t * hb_buffer_init_internal( int size , int needsMapped )
              */
             uint8_t *data = b->data;
 
-            /* OpenCL */
-            cl_mem buffer       = b->cl.buffer;
-            cl_event last_event = b->cl.last_event;
-            int loc             = b->cl.buffer_location;
-
             memset( b, 0, sizeof(hb_buffer_t) );
-            b->alloc = buffer_pool->buffer_size;
-            b->size = size;
-            b->data = data;
-            b->s.start = AV_NOPTS_VALUE;
-            b->s.stop = AV_NOPTS_VALUE;
+            b->alloc          = buffer_pool->buffer_size;
+            b->size           = size;
+            b->data           = data;
+            b->s.start        = AV_NOPTS_VALUE;
+            b->s.stop         = AV_NOPTS_VALUE;
             b->s.renderOffset = AV_NOPTS_VALUE;
-
-            /* OpenCL */
-            b->cl.buffer          = buffer;
-            b->cl.last_event      = last_event;
-            b->cl.buffer_location = loc;
+            b->s.scr_sequence = -1;
 
 #if defined(HB_BUFFER_DEBUG)
             hb_lock(buffers.lock);
@@ -391,34 +395,14 @@ hb_buffer_t * hb_buffer_init_internal( int size , int needsMapped )
 
     if (size)
     {
-        /* OpenCL */
-        b->cl.last_event      = NULL;
-        b->cl.buffer_location = HOST;
-
-        /* OpenCL */
-        if (needsMapped)
-        {
-            int status = hb_cl_create_mapped_buffer(&b->cl.buffer, &b->data, b->alloc);
-            if (!status)
-            {
-                hb_error("Failed to map CL buffer");
-                free(b);
-                return NULL;
-            }
-        }
-        else
-        {
-            b->cl.buffer = NULL;
-
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined( SYS_MINGW )
-            b->data  = malloc( b->alloc );
+#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
+        b->data  = malloc( b->alloc );
 #elif defined( SYS_CYGWIN )
-            /* FIXME */
-            b->data  = malloc( b->alloc + 17 );
+        /* FIXME */
+        b->data  = malloc( b->alloc + 17 );
 #else
-            b->data  = memalign( 16, b->alloc );
+        b->data  = memalign( 16, b->alloc );
 #endif
-        }
 
         if( !b->data )
         {
@@ -426,13 +410,17 @@ hb_buffer_t * hb_buffer_init_internal( int size , int needsMapped )
             free( b );
             return NULL;
         }
+#if defined(HB_BUFFER_DEBUG)
+        memset(b->data, 0, b->size);
+#endif
         hb_lock(buffers.lock);
         buffers.allocated += b->alloc;
         hb_unlock(buffers.lock);
     }
-    b->s.start = AV_NOPTS_VALUE;
-    b->s.stop = AV_NOPTS_VALUE;
+    b->s.start        = AV_NOPTS_VALUE;
+    b->s.stop         = AV_NOPTS_VALUE;
     b->s.renderOffset = AV_NOPTS_VALUE;
+    b->s.scr_sequence = -1;
 #if defined(HB_BUFFER_DEBUG)
     hb_lock(buffers.lock);
     hb_list_add(buffers.alloc_list, b);
@@ -443,7 +431,14 @@ hb_buffer_t * hb_buffer_init_internal( int size , int needsMapped )
 
 hb_buffer_t * hb_buffer_init( int size )
 {
-    return hb_buffer_init_internal(size, 0);
+    return hb_buffer_init_internal(size);
+}
+
+hb_buffer_t * hb_buffer_eof_init(void)
+{
+    hb_buffer_t * buf = hb_buffer_init(0);
+    buf->s.flags = HB_BUF_FLAG_EOF;
+    return buf;
 }
 
 void hb_buffer_realloc( hb_buffer_t * b, int size )
@@ -451,7 +446,11 @@ void hb_buffer_realloc( hb_buffer_t * b, int size )
     if ( size > b->alloc || b->data == NULL )
     {
         uint32_t orig = b->data != NULL ? b->alloc : 0;
-        size = size_to_pool( size )->buffer_size;
+        hb_fifo_t *buffer_pool = size_to_pool(size);
+        if (buffer_pool != NULL)
+        {
+            size = buffer_pool->buffer_size;
+        }
         b->data  = realloc( b->data, size );
         b->alloc = size;
 
@@ -542,6 +541,11 @@ void hb_buffer_init_planes( hb_buffer_t * b )
     const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(b->f.fmt);
     int p;
 
+    if (desc == NULL)
+    {
+        return;
+    }
+
     uint8_t has_plane[4] = {0,};
 
     for( p = 0; p < 4; p++ )
@@ -560,6 +564,10 @@ hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int width, int height )
     int p;
     uint8_t has_plane[4] = {0,};
 
+    if (desc == NULL)
+    {
+        return NULL;
+    }
     for( p = 0; p < 4; p++ )
     {
         has_plane[desc->comp[p].plane] = 1;
@@ -575,8 +583,7 @@ hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int width, int height )
         }
     }
 
-    /* OpenCL */
-    buf = hb_buffer_init_internal(size , hb_use_buffers());
+    buf = hb_buffer_init_internal(size);
 
     if( buf == NULL )
         return NULL;
@@ -598,6 +605,10 @@ void hb_video_buffer_realloc( hb_buffer_t * buf, int width, int height )
     int p;
     uint8_t has_plane[4] = {0,};
 
+    if (desc == NULL)
+    {
+        return;
+    }
     for( p = 0; p < 4; p++ )
     {
         has_plane[desc->comp[p].plane] = 1;
@@ -631,21 +642,11 @@ void hb_buffer_swap_copy( hb_buffer_t *src, hb_buffer_t *dst )
     int      size  = dst->size;
     int      alloc = dst->alloc;
 
-    /* OpenCL */
-    cl_mem buffer       = dst->cl.buffer;
-    cl_event last_event = dst->cl.last_event;
-    int loc             = dst->cl.buffer_location;
-
     *dst = *src;
 
     src->data  = data;
     src->size  = size;
     src->alloc = alloc;
-
-    /* OpenCL */
-    src->cl.buffer          = buffer;
-    src->cl.last_event      = last_event;
-    src->cl.buffer_location = loc;
 }
 
 // Frees the specified buffer list.
@@ -655,6 +656,29 @@ void hb_buffer_close( hb_buffer_t ** _b )
 
     while( b )
     {
+#ifdef USE_QSV
+        // Reclaim QSV resources before dropping the buffer.
+        // when decoding without QSV, the QSV atom will be NULL.
+        if (b->qsv_details.qsv_atom != NULL && b->qsv_details.ctx != NULL)
+        {
+            hb_qsv_stage *stage = hb_qsv_get_last_stage(b->qsv_details.qsv_atom);
+            if (stage != NULL)
+            {
+                hb_qsv_wait_on_sync(b->qsv_details.ctx, stage);
+                if (stage->out.sync->in_use > 0)
+                {
+                    ff_qsv_atomic_dec(&stage->out.sync->in_use);
+                }
+                if (stage->out.p_surface->Data.Locked > 0)
+                {
+                    ff_qsv_atomic_dec(&stage->out.p_surface->Data.Locked);
+                }
+            }
+            hb_qsv_flush_stages(b->qsv_details.ctx->pipes,
+                                (hb_qsv_list**)&b->qsv_details.qsv_atom);
+        }
+#endif
+
         hb_buffer_t * next = b->next;
         hb_fifo_t *buffer_pool = size_to_pool( b->alloc );
 
@@ -665,11 +689,15 @@ void hb_buffer_close( hb_buffer_t ** _b )
         hb_list_rem(buffers.alloc_list, b);
         hb_unlock(buffers.lock);
 #endif
-        // Close any attached subtitle buffers
-        hb_buffer_close( &b->sub );
-
         if( buffer_pool && b->data && !hb_fifo_is_full( buffer_pool ) )
         {
+#if defined(HB_BUFFER_DEBUG)
+            if (hb_fifo_contains(buffer_pool, b))
+            {
+                hb_error("hb_buffer_close: buffer %p already freed", b);
+                assert(0);
+            }
+#endif
             hb_fifo_push_head( buffer_pool, b );
             b = next;
             continue;
@@ -678,19 +706,7 @@ void hb_buffer_close( hb_buffer_t ** _b )
         // free the buf 
         if( b->data )
         {
-            if (b->cl.buffer != NULL)
-            {
-                /* OpenCL */
-                if (hb_cl_free_mapped_buffer(b->cl.buffer, b->data) == 0)
-                {
-                    hb_log("hb_buffer_pool_free: bad free %p -> buffer %p map %p",
-                           b, b->cl.buffer, b->data);
-                }
-            }
-            else
-            {
-                free(b->data);
-            }
+            free(b->data);
             hb_lock(buffers.lock);
             buffers.allocated -= b->alloc;
             hb_unlock(buffers.lock);
@@ -702,23 +718,78 @@ void hb_buffer_close( hb_buffer_t ** _b )
     *_b = NULL;
 }
 
-void hb_buffer_move_subs( hb_buffer_t * dst, hb_buffer_t * src )
+hb_image_t * hb_image_init(int pix_fmt, int width, int height)
 {
-    // Note that dst takes ownership of the subtitles
-    dst->sub       = src->sub;
-    src->sub       = NULL;
+    const AVPixFmtDescriptor *desc = av_pix_fmt_desc_get(pix_fmt);
+    int p;
+    uint8_t has_plane[4] = {0,};
 
-#ifdef USE_QSV
-	memcpy(&dst->qsv_details, &src->qsv_details, sizeof(src->qsv_details));
+    if (desc == NULL)
+    {
+        return NULL;
+    }
+    for (p = 0; p < 4; p++)
+    {
+        has_plane[desc->comp[p].plane] = 1;
+    }
+
+    int size = 0;
+    for (p = 0; p < 4; p++)
+    {
+        if (has_plane[p])
+        {
+            size += hb_image_stride( pix_fmt, width, p ) * 
+                    hb_image_height_stride( pix_fmt, height, p );
+        }
+    }
+
+    hb_image_t *image = calloc(1, sizeof(hb_image_t));
+    if (image == NULL)
+    {
+        return NULL;
+    }
+#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
+    image->data  = malloc(size);
+#elif defined( SYS_CYGWIN )
+    /* FIXME */
+    image->data  = malloc(size + 17);
+#else
+    image->data  = memalign(16, size);
 #endif
+    if (image->data == NULL)
+    {
+        free(image);
+        return NULL;
+    }
+    image->format = pix_fmt;
+    image->width = width;
+    image->height = height;
+    memset(image->data, 0, size);
 
+    uint8_t * plane = image->data;
+    for (p = 0; p < 4; p++)
+    {
+        if (has_plane[p])
+        {
+            image->plane[p].data = plane;
+            image->plane[p].stride = hb_image_stride(pix_fmt, width, p );
+            image->plane[p].height_stride =
+                                    hb_image_height_stride(pix_fmt, height, p );
+            image->plane[p].width  = hb_image_width(pix_fmt, width, p );
+            image->plane[p].height = hb_image_height(pix_fmt, height, p );
+            image->plane[p].size   =
+                        image->plane[p].stride * image->plane[p].height_stride;
+            plane += image->plane[p].size;
+        }
+    }
+    return image;
 }
 
 hb_image_t * hb_buffer_to_image(hb_buffer_t *buf)
 {
     hb_image_t *image = calloc(1, sizeof(hb_image_t));
 
-#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined( SYS_MINGW )
+#if defined( SYS_DARWIN ) || defined( SYS_FREEBSD ) || defined ( __FreeBSD__ ) || defined( SYS_MINGW )
     image->data  = malloc( buf->size );
 #elif defined( SYS_CYGWIN )
     /* FIXME */
@@ -784,6 +855,11 @@ hb_fifo_t * hb_fifo_init( int capacity, int thresh )
     return f;
 }
 
+void hb_fifo_register_full_cond( hb_fifo_t * f, hb_cond_t * c )
+{
+    f->cond_alert_full = c;
+}
+
 int hb_fifo_size_bytes( hb_fifo_t * f )
 {
     int ret = 0;
@@ -975,6 +1051,8 @@ void hb_fifo_push_wait( hb_fifo_t * f, hb_buffer_t * b )
     if( f->size >= f->capacity )
     {
         f->wait_full = 1;
+        if (f->cond_alert_full != NULL)
+            hb_cond_broadcast( f->cond_alert_full );
         hb_cond_timedwait( f->cond_full, f->lock, FIFO_TIMEOUT );
     }
     if( f->size > 0 )
@@ -1009,6 +1087,11 @@ void hb_fifo_push( hb_fifo_t * f, hb_buffer_t * b )
     }
 
     hb_lock( f->lock );
+    if (f->size >= f->capacity &&
+        f->cond_alert_full != NULL)
+    {
+        hb_cond_broadcast( f->cond_alert_full );
+    }
     if( f->size > 0 )
     {
         f->last->next = b;
@@ -1044,6 +1127,11 @@ void hb_fifo_push_head( hb_fifo_t * f, hb_buffer_t * b )
     }
 
     hb_lock( f->lock );
+    if (f->size >= f->capacity &&
+        f->cond_alert_full != NULL)
+    {
+        hb_cond_broadcast( f->cond_alert_full );
+    }
 
     /*
      * If there are a chain of buffers prepend the lot
@@ -1070,29 +1158,6 @@ void hb_fifo_push_head( hb_fifo_t * f, hb_buffer_t * b )
     hb_unlock( f->lock );
 }
 
-// Pushes a list of packets onto the specified FIFO as a single element.
-void hb_fifo_push_list_element( hb_fifo_t *fifo, hb_buffer_t *buffer_list )
-{
-    hb_buffer_t *container = hb_buffer_init( 0 );
-    // XXX: Using an arbitrary hb_buffer_t pointer (other than 'next')
-    //      to carry the list inside a single "container" buffer
-    container->sub = buffer_list;
-    
-    hb_fifo_push( fifo, container );
-}
-
-// Removes a list of packets from the specified FIFO that were stored as a single element.
-hb_buffer_t *hb_fifo_get_list_element( hb_fifo_t *fifo )
-{
-    hb_buffer_t *container = hb_fifo_get( fifo );
-    // XXX: Using an arbitrary hb_buffer_t pointer (other than 'next')
-    //      to carry the list inside a single "container" buffer
-    hb_buffer_t *buffer_list = container->sub;
-    hb_buffer_close( &container );
-    
-    return buffer_list;
-}
-
 void hb_fifo_close( hb_fifo_t ** _f )
 {
     hb_fifo_t   * f = *_f;
@@ -1136,3 +1201,19 @@ void hb_fifo_flush( hb_fifo_t * f )
 
 }
 
+#if defined(HB_BUFFER_DEBUG)
+static int hb_fifo_contains( hb_fifo_t *f, hb_buffer_t *b )
+{
+    hb_buffer_t * tmp = f->first;
+
+    while (tmp != NULL)
+    {
+        if (b == tmp)
+        {
+            return 1;
+        }
+        tmp = tmp->next;
+    }
+    return 0;
+}
+#endif
