@@ -1,6 +1,6 @@
 /* hbffmpeg.h
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -15,6 +15,7 @@
 #include "libavutil/opt.h"
 #include "libavutil/avutil.h"
 #include "libavutil/downmix_info.h"
+#include "libavutil/display.h"
 #include "libswscale/swscale.h"
 #include "libavresample/avresample.h"
 #include "common.h"
@@ -24,12 +25,18 @@
 void hb_avcodec_init(void);
 int  hb_avcodec_open(AVCodecContext *, AVCodec *, AVDictionary **, int);
 int  hb_avcodec_close(AVCodecContext *);
+const char* const* hb_av_preset_get_names(int encoder);
 
 uint64_t hb_ff_mixdown_xlat(int hb_mixdown, int *downmix_mode);
 void     hb_ff_set_sample_fmt(AVCodecContext *, AVCodec *, enum AVSampleFormat);
 
+int hb_ff_get_colorspace(int color_matrix);
+
 struct SwsContext*
 hb_sws_get_context(int srcW, int srcH, enum AVPixelFormat srcFormat,
                    int dstW, int dstH, enum AVPixelFormat dstFormat,
-                   int flags);
-int hb_avpicture_fill(AVPicture *pic, hb_buffer_t *buf);
+                   int flags, int colorspace);
+
+hb_buffer_t * hb_avframe_to_video_buffer(AVFrame *frame, AVRational time_base);
+void hb_avframe_set_video_buffer_flags(hb_buffer_t * buf, AVFrame *frame,
+                                       AVRational time_base);
