@@ -1,6 +1,6 @@
 /* internal.h
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -9,6 +9,9 @@
 
 #include "hbffmpeg.h"
 #include "extras/cl.h"
+#ifdef USE_QSV
+#include "qsv_libav.h"
+#endif
 
 /***********************************************************************
  * common.c
@@ -40,6 +43,8 @@ void         hb_title_close( hb_title_t ** );
  **********************************************************************/
 int  hb_get_pid( hb_handle_t * );
 void hb_set_state( hb_handle_t *, hb_state_t * );
+void hb_set_work_error( hb_handle_t * h, hb_error_code err );
+void hb_job_setup_passes(hb_handle_t *h, hb_job_t *job, hb_list_t *list_pass);
 
 /***********************************************************************
  * fifo.c
@@ -53,7 +58,7 @@ void hb_set_state( hb_handle_t *, hb_state_t * );
  */
 struct hb_buffer_settings_s
 {
-    enum { AUDIO_BUF, VIDEO_BUF, SUBTITLE_BUF, FRAME_BUF, OTHER_BUF } type;
+    enum { OTHER_BUF, AUDIO_BUF, VIDEO_BUF, SUBTITLE_BUF, FRAME_BUF } type;
 
     int           id;           // ID of the track that the packet comes from
     int64_t       start;        // start time of frame
@@ -61,6 +66,9 @@ struct hb_buffer_settings_s
     int64_t       stop;         // stop time of frame
     int64_t       renderOffset; // DTS used by b-frame offsets in muxmp4
     int64_t       pcr;
+    int           scr_sequence; // The SCR sequence that this buffer's 
+                                // timestamps are referenced to
+    int           split;
     uint8_t       discontinuity;
     int           new_chap;     // Video packets: if non-zero, is the index of the chapter whose boundary was crossed
 
@@ -71,10 +79,31 @@ struct hb_buffer_settings_s
 #define HB_FRAME_P        0x10
 #define HB_FRAME_B        0x20
 #define HB_FRAME_BREF     0x40
-#define HB_FRAME_KEY      0x0F
-#define HB_FRAME_REF      0xF0
+#define HB_FRAME_MASK_KEY 0x0F
+#define HB_FRAME_MASK_REF 0xF0
     uint8_t       frametype;
+
+// Picture flags used by filters
+#ifndef PIC_FLAG_TOP_FIELD_FIRST
+#define PIC_FLAG_TOP_FIELD_FIRST    0x0008
+#endif
+#ifndef PIC_FLAG_PROGRESSIVE_FRAME
+#define PIC_FLAG_PROGRESSIVE_FRAME  0x0010
+#endif
+#ifndef PIC_FLAG_REPEAT_FIRST_FIELD
+#define PIC_FLAG_REPEAT_FIRST_FIELD 0x0100
+#endif
+#define PIC_FLAG_REPEAT_FRAME       0x0200
+#define HB_BUF_FLAG_EOF             0x0400
+#define HB_BUF_FLAG_EOS             0x0800
+#define HB_FLAG_FRAMETYPE_KEY       0x1000
+#define HB_FLAG_FRAMETYPE_REF       0x2000
     uint16_t      flags;
+
+#define HB_COMB_NONE  0
+#define HB_COMB_LIGHT 1
+#define HB_COMB_HEAVY 2
+    uint8_t       combed;
 };
 
 struct hb_image_format_s
@@ -84,6 +113,8 @@ struct hb_image_format_s
     int           width;
     int           height;
     int           fmt;
+    int           window_width;
+    int           window_height;
 };
 
 struct hb_buffer_s
@@ -93,19 +124,6 @@ struct hb_buffer_s
     uint8_t *     data;     // packet data
     int           offset;   // used internally by packet lists (hb_list_t)
 
-    /*
-     * Corresponds to the order that this packet was read from the demuxer.
-     * 
-     * It is important that video decoder work-objects pass this value through
-     * from their input packets to the output packets they generate. Otherwise
-     * RENDERSUB subtitles (especially VOB subtitles) will break.
-     * 
-     * Subtitle decoder work-objects that output a renderable subtitle
-     * format (ex: PICTURESUB) must also be careful to pass the sequence number
-     * through for the same reason.
-     */
-    int64_t       sequence;
-
     hb_buffer_settings_t s;
     hb_image_format_t f;
 
@@ -119,34 +137,19 @@ struct hb_buffer_s
         int           size;
     } plane[4]; // 3 Color components + alpha
 
+#ifdef USE_QSV
     struct qsv
     {
-        void *qsv_atom;
-        void *filter_details;
+        void           * qsv_atom;
+        void           * filter_details;
+        hb_qsv_context * ctx;
     } qsv_details;
-
-    /* OpenCL */
-    struct cl_data
-    {
-        cl_mem buffer;
-        cl_event last_event;
-        enum { HOST, DEVICE } buffer_location;
-    } cl;
+#endif
 
     // libav may attach AV_PKT_DATA_PALETTE side data to some AVPackets
     // Store this data here when read and pass to decoder.
     hb_buffer_t * palette;
 
-    // PICTURESUB subtitle packets:
-
-    // Video packets (after processing by the hb_sync_video work-object):
-    //   A (copy of a) PICTURESUB subtitle packet that needs to be burned into 
-    //   this video packet by the vobsub renderer filter
-    //
-    //   Subtitles that are simply passed thru are NOT attached to the 
-    //   associated video packets.
-    hb_buffer_t * sub;
-
     // Packets in a list:
     //   the next packet in the list
     hb_buffer_t * next;
@@ -156,6 +159,7 @@ void hb_buffer_pool_init( void );
 void hb_buffer_pool_free( void );
 
 hb_buffer_t * hb_buffer_init( int size );
+hb_buffer_t * hb_buffer_eof_init( void );
 hb_buffer_t * hb_frame_buffer_init( int pix_fmt, int w, int h);
 void          hb_buffer_init_planes( hb_buffer_t * b );
 void          hb_buffer_realloc( hb_buffer_t *, int size );
@@ -165,10 +169,14 @@ void          hb_buffer_close( hb_buffer_t ** );
 hb_buffer_t * hb_buffer_dup( const hb_buffer_t * src );
 int           hb_buffer_copy( hb_buffer_t * dst, const hb_buffer_t * src );
 void          hb_buffer_swap_copy( hb_buffer_t *src, hb_buffer_t *dst );
-void          hb_buffer_move_subs( hb_buffer_t * dst, hb_buffer_t * src );
+hb_image_t  * hb_image_init(int pix_fmt, int width, int height);
 hb_image_t  * hb_buffer_to_image(hb_buffer_t *buf);
+int           hb_picture_fill(uint8_t *data[], int stride[], hb_buffer_t *b);
+int           hb_picture_crop(uint8_t *data[], int stride[], hb_buffer_t *b,
+                              int top, int left);
 
 hb_fifo_t   * hb_fifo_init( int capacity, int thresh );
+void          hb_fifo_register_full_cond( hb_fifo_t * f, hb_cond_t * c );
 int           hb_fifo_size( hb_fifo_t * );
 int           hb_fifo_size_bytes( hb_fifo_t * );
 int           hb_fifo_is_full( hb_fifo_t * );
@@ -182,8 +190,6 @@ void          hb_fifo_push( hb_fifo_t *, hb_buffer_t * );
 void          hb_fifo_push_wait( hb_fifo_t *, hb_buffer_t * );
 int           hb_fifo_full_wait( hb_fifo_t * f );
 void          hb_fifo_push_head( hb_fifo_t *, hb_buffer_t * );
-void          hb_fifo_push_list_element( hb_fifo_t *fifo, hb_buffer_t *buffer_list );
-hb_buffer_t * hb_fifo_get_list_element( hb_fifo_t *fifo );
 void          hb_fifo_close( hb_fifo_t ** );
 void          hb_fifo_flush( hb_fifo_t * f );
 
@@ -204,7 +210,7 @@ static inline int hb_image_width(int pix_fmt, int width, int plane)
 
     if (desc != NULL && (plane == 1 || plane == 2))
     {
-        // The wacky arithmatic assures rounding up.
+        // The wacky arithmetic assures rounding up.
         width = -((-width) >> desc->log2_chroma_w);
     }
 
@@ -231,7 +237,7 @@ static inline int hb_image_height(int pix_fmt, int height, int plane)
 
     if (desc != NULL && (plane == 1 || plane == 2))
     {
-        // The wacky arithmatic assures rounding up.
+        // The wacky arithmetic assures rounding up.
         height = -((-height) >> desc->log2_chroma_h);
     }
 
@@ -246,9 +252,8 @@ static inline hb_buffer_t * hb_video_buffer_init( int width, int height )
 }
 
 /***********************************************************************
- * Threads: update.c, scan.c, work.c, reader.c, muxcommon.c
+ * Threads: scan.c, work.c, reader.c, muxcommon.c
  **********************************************************************/
-hb_thread_t * hb_update_init( int * build, char * version );
 hb_thread_t * hb_scan_init( hb_handle_t *, volatile int * die, 
                             const char * path, int title_index, 
                             hb_title_set_t * title_set, int preview_count, 
@@ -256,10 +261,13 @@ hb_thread_t * hb_scan_init( hb_handle_t *, volatile int * die,
 hb_thread_t * hb_work_init( hb_list_t * jobs,
                             volatile int * die, hb_error_code * error, hb_job_t ** job );
 void ReadLoop( void * _w );
+void hb_work_loop( void * );
 hb_work_object_t * hb_muxer_init( hb_job_t * );
-hb_work_object_t * hb_get_work( int );
-hb_work_object_t * hb_codec_decoder( int );
-hb_work_object_t * hb_codec_encoder( int );
+hb_work_object_t * hb_get_work( hb_handle_t *, int );
+hb_work_object_t * hb_audio_decoder( hb_handle_t *, int );
+hb_work_object_t * hb_audio_encoder( hb_handle_t *, int );
+hb_work_object_t * hb_video_decoder( hb_handle_t *, int, int );
+hb_work_object_t * hb_video_encoder( hb_handle_t *, int );
 
 /***********************************************************************
  * sync.c
@@ -274,15 +282,14 @@ typedef struct {
     int64_t scr_delta;
     int64_t last_pts;       /* last pts we saw */
     int     scr_changes;    /* number of SCR discontinuities */
-    int     dts_drops;      /* number of drops because DTS too far from SCR */
     int     new_chap;
 } hb_psdemux_t;
 
-typedef void (*hb_muxer_t)(hb_buffer_t *, hb_list_t *, hb_psdemux_t*);
+typedef void (*hb_muxer_t)(hb_buffer_t *, hb_buffer_list_t *, hb_psdemux_t*);
 
-void hb_demux_ps( hb_buffer_t * ps_buf, hb_list_t * es_list, hb_psdemux_t * );
-void hb_demux_ts( hb_buffer_t * ps_buf, hb_list_t * es_list, hb_psdemux_t * );
-void hb_demux_null( hb_buffer_t * ps_buf, hb_list_t * es_list, hb_psdemux_t * );
+void hb_demux_ps(hb_buffer_t * ps_buf, hb_buffer_list_t * es_list, hb_psdemux_t *);
+void hb_demux_ts(hb_buffer_t * ps_buf, hb_buffer_list_t * es_list, hb_psdemux_t *);
+void hb_demux_null(hb_buffer_t * ps_buf, hb_buffer_list_t * es_list, hb_psdemux_t *);
 
 extern const hb_muxer_t hb_demux[];
 
@@ -291,10 +298,11 @@ extern const hb_muxer_t hb_demux[];
  **********************************************************************/
 typedef struct hb_batch_s hb_batch_t;
 
-hb_batch_t  * hb_batch_init( char * path );
+hb_batch_t  * hb_batch_init( hb_handle_t *h, char * path );
 void          hb_batch_close( hb_batch_t ** _d );
 int           hb_batch_title_count( hb_batch_t * d );
-hb_title_t  * hb_batch_title_scan( hb_batch_t * d, int t );
+hb_title_t  * hb_batch_title_scan( hb_batch_t * d, int t,
+                                   uint64_t min_duration );
 
 /***********************************************************************
  * dvd.c
@@ -303,7 +311,7 @@ typedef struct hb_bd_s hb_bd_t;
 typedef union  hb_dvd_s hb_dvd_t;
 typedef struct hb_stream_s hb_stream_t;
 
-hb_dvd_t *   hb_dvd_init( char * path );
+hb_dvd_t *   hb_dvd_init( hb_handle_t * h, char * path );
 int          hb_dvd_title_count( hb_dvd_t * );
 hb_title_t * hb_dvd_title_scan( hb_dvd_t *, int title, uint64_t min_duration );
 int          hb_dvd_start( hb_dvd_t *, hb_title_t *title, int chapter );
@@ -317,7 +325,7 @@ int          hb_dvd_angle_count( hb_dvd_t * d );
 void         hb_dvd_set_angle( hb_dvd_t * d, int angle );
 int          hb_dvd_main_feature( hb_dvd_t * d, hb_list_t * list_title );
 
-hb_bd_t     * hb_bd_init( char * path );
+hb_bd_t     * hb_bd_init( hb_handle_t *h, char * path );
 int           hb_bd_title_count( hb_bd_t * d );
 hb_title_t  * hb_bd_title_scan( hb_bd_t * d, int t, uint64_t min_duration );
 int           hb_bd_start( hb_bd_t * d, hb_title_t *title );
@@ -331,9 +339,10 @@ void          hb_bd_close( hb_bd_t ** _d );
 void          hb_bd_set_angle( hb_bd_t * d, int angle );
 int           hb_bd_main_feature( hb_bd_t * d, hb_list_t * list_title );
 
-hb_stream_t * hb_bd_stream_open( hb_title_t *title );
+hb_stream_t * hb_bd_stream_open( hb_handle_t *h, hb_title_t *title );
 void hb_ts_stream_reset(hb_stream_t *stream);
-hb_stream_t * hb_stream_open( char * path, hb_title_t *title, int scan );
+hb_stream_t * hb_stream_open(hb_handle_t *h, char * path,
+                             hb_title_t *title, int scan);
 void		 hb_stream_close( hb_stream_t ** );
 hb_title_t * hb_stream_title_scan( hb_stream_t *, hb_title_t *);
 hb_buffer_t * hb_stream_read( hb_stream_t * );
@@ -342,7 +351,8 @@ int          hb_stream_seek_ts( hb_stream_t * stream, int64_t ts );
 int          hb_stream_seek_chapter( hb_stream_t *, int );
 int          hb_stream_chapter( hb_stream_t * );
 
-hb_buffer_t * hb_ts_decode_pkt( hb_stream_t *stream, const uint8_t * pkt );
+hb_buffer_t * hb_ts_decode_pkt( hb_stream_t *stream, const uint8_t * pkt,
+                                int chapter, int discontinuity );
 void hb_stream_set_need_keyframe( hb_stream_t *stream, int need_keyframe );
 
 
@@ -356,8 +366,12 @@ void hb_stream_set_need_keyframe( hb_stream_t *stream, int need_keyframe );
  * Work objects
  **********************************************************************/
 #define HB_CONFIG_MAX_SIZE (2*8192)
-union hb_esconfig_u
+struct hb_esconfig_s
 {
+    int init_delay;
+
+    union
+    {
 
     struct
     {
@@ -371,7 +385,6 @@ union hb_esconfig_u
 	    int       sps_length;
 	    uint8_t  pps[HB_CONFIG_MAX_SIZE];
 	    int       pps_length;
-        int      init_delay;
 	} h264;
 
     struct
@@ -396,6 +409,7 @@ union hb_esconfig_u
         uint8_t headers[3][HB_CONFIG_MAX_SIZE];
         char *language;
     } vorbis;
+    };
 };
 
 enum
@@ -403,6 +417,7 @@ enum
     WORK_NONE = 0,
     WORK_SYNC_VIDEO,
     WORK_SYNC_AUDIO,
+    WORK_SYNC_SUBTITLE,
     WORK_DECCC608,
     WORK_DECVOBSUB,
     WORK_DECSRTSUB,
@@ -430,15 +445,22 @@ enum
 };
 
 extern hb_filter_object_t hb_filter_detelecine;
+extern hb_filter_object_t hb_filter_comb_detect;
+extern hb_filter_object_t hb_filter_decomb;
 extern hb_filter_object_t hb_filter_deinterlace;
+extern hb_filter_object_t hb_filter_vfr;
 extern hb_filter_object_t hb_filter_deblock;
 extern hb_filter_object_t hb_filter_denoise;
 extern hb_filter_object_t hb_filter_nlmeans;
-extern hb_filter_object_t hb_filter_decomb;
-extern hb_filter_object_t hb_filter_rotate;
-extern hb_filter_object_t hb_filter_crop_scale;
 extern hb_filter_object_t hb_filter_render_sub;
-extern hb_filter_object_t hb_filter_vfr;
+extern hb_filter_object_t hb_filter_crop_scale;
+extern hb_filter_object_t hb_filter_rotate;
+extern hb_filter_object_t hb_filter_grayscale;
+extern hb_filter_object_t hb_filter_pad;
+extern hb_filter_object_t hb_filter_lapsharp;
+extern hb_filter_object_t hb_filter_unsharp;
+extern hb_filter_object_t hb_filter_avfilter;
+extern hb_filter_object_t hb_filter_mt_frame;
 
 #ifdef USE_QSV
 extern hb_filter_object_t hb_filter_qsv;
@@ -446,19 +468,6 @@ extern hb_filter_object_t hb_filter_qsv_pre;
 extern hb_filter_object_t hb_filter_qsv_post;
 #endif
 
-// Picture flags used by filters
-#ifndef PIC_FLAG_REPEAT_FIRST_FIELD
-#define PIC_FLAG_REPEAT_FIRST_FIELD 256
-#endif
-#ifndef PIC_FLAG_TOP_FIELD_FIRST
-#define PIC_FLAG_TOP_FIELD_FIRST 8
-#endif
-#ifndef PIC_FLAG_PROGRESSIVE_FRAME
-#define PIC_FLAG_PROGRESSIVE_FRAME 16
-#endif
-
-#define PIC_FLAG_REPEAT_FRAME 512
-
 extern hb_work_object_t * hb_objects;
 
 #define HB_WORK_IDLE     0
@@ -485,8 +494,39 @@ DECLARE_MUX( mp4 );
 DECLARE_MUX( mkv );
 DECLARE_MUX( avformat );
 
-void hb_muxmp4_process_subtitle_style( uint8_t *input,
-                                       uint8_t *output,
-                                       uint8_t *style, uint16_t *stylesize );
+void hb_muxmp4_process_subtitle_style(int        height,
+                                      uint8_t  * input, uint8_t  ** output,
+                                      uint8_t ** style, uint16_t  * stylesize);
 
 void hb_deinterlace(hb_buffer_t *dst, hb_buffer_t *src);
+void hb_avfilter_combine( hb_list_t * list );
+char * hb_append_filter_string(char * graph_str, char * filter_str);
+
+struct hb_chapter_queue_item_s
+{
+    int64_t start;
+    int     new_chap;
+};
+
+struct hb_chapter_queue_s
+{
+    hb_list_t   * list_chapter;
+};
+
+typedef struct hb_chapter_queue_item_s hb_chapter_queue_item_t;
+typedef struct hb_chapter_queue_s hb_chapter_queue_t;
+
+hb_chapter_queue_t * hb_chapter_queue_init(void);
+void                 hb_chapter_queue_close(hb_chapter_queue_t **_q);
+void                 hb_chapter_enqueue(hb_chapter_queue_t *q, hb_buffer_t *b);
+void                 hb_chapter_dequeue(hb_chapter_queue_t *q, hb_buffer_t *b);
+
+/* Font names used for rendering subtitles */
+#if defined(SYS_MINGW)
+#define HB_FONT_MONO "Lucida Console"
+#define HB_FONT_SANS "sans-serif"
+#else
+#define HB_FONT_MONO "monospace"
+#define HB_FONT_SANS "sans-serif"
+#endif
+
