@@ -1,6 +1,6 @@
 /* muxcommon.c
 
-   Copyright (c) 2003-2016 HandBrake Team
+   Copyright (c) 2003-2018 HandBrake Team
    This file is part of the HandBrake source code
    Homepage: <http://handbrake.fr/>.
    It may be used under the terms of the GNU General Public License v2.
@@ -43,7 +43,6 @@ typedef struct
 typedef struct
 {
     hb_lock_t       * mutex;
-    int               ref;
     int               done;
     hb_mux_object_t * m;
     double            pts;        // end time of next muxing chunk
@@ -60,9 +59,10 @@ typedef struct
 
 struct hb_work_private_s
 {
-    hb_job_t * job;
-    int        track;
-    hb_mux_t * mux;
+    hb_job_t  * job;
+    int         track;
+    hb_mux_t  * mux;
+    hb_list_t * list_work;
 };
 
 
@@ -366,14 +366,15 @@ static int muxWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
         return HB_WORK_DONE;
     }
 
-    if ( buf->size <= 0 )
+    if (buf->s.flags & HB_BUF_FLAG_EOF)
     {
         // EOF - mark this track as done
         hb_buffer_close( &buf );
         hb_bitvec_set(mux->eof, pv->track);
         hb_bitvec_set(mux->rdy, pv->track);
     }
-    else if ((job->pass != 0 && job->pass != 2) ||
+    else if ((job->pass_id != HB_PASS_ENCODE &&
+              job->pass_id != HB_PASS_ENCODE_2ND) ||
              hb_bitvec_bit(mux->eof, pv->track))
     {
         hb_buffer_close( &buf );
@@ -443,6 +444,7 @@ static int muxWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
             if ( i >= mux->ntracks )
             {
                 mux->done = 1;
+                *w->done = 1;
                 hb_unlock( mux->mutex );
                 hb_bitvec_free(&more);
                 return HB_WORK_DONE;
@@ -456,143 +458,171 @@ static int muxWork( hb_work_object_t * w, hb_buffer_t ** buf_in,
     return HB_WORK_OK;
 }
 
-void muxClose( hb_work_object_t * w )
+static void muxFlush(hb_mux_t * mux)
 {
-    hb_work_private_t * pv = w->private_data;
-    hb_mux_t    * mux = pv->mux;
-    hb_job_t    * job = pv->job;
-    hb_track_t  * track;
-    int           i;
+    int ii, done = 0;
+
+    while (!done)
+    {
+        done = 1;
+        for (ii = 0; ii < mux->ntracks; ii++)
+        {
+            OutputTrackChunk(mux, ii, mux->m);
+            if (mux->track[ii]->mf.out != mux->track[ii]->mf.in)
+            {
+                // track buffer is not empty
+                done = 0;
+            }
+        }
+        mux->pts += mux->interleave;
+    }
+}
+
+static void muxClose( hb_work_object_t * muxer )
+{
+    hb_work_private_t * pv = muxer->private_data;
+    if (pv == NULL)
+    {
+        // Not initialized
+        return;
+    }
+
+    hb_mux_t          * mux = pv->mux;
+    hb_job_t          * job = pv->job;
+    hb_track_t        * track;
+    hb_work_object_t  * w;
+    int                 i;
 
     hb_lock( mux->mutex );
-    if ( --mux->ref == 0 )
+    muxFlush(mux);
+
+    // Update state before closing muxer.  Closing the muxer
+    // may initiate optimization which can take a while and
+    // we want the muxing state to be visible while this is
+    // happening.
+    if( job->pass_id == HB_PASS_ENCODE ||
+        job->pass_id == HB_PASS_ENCODE_2ND )
     {
-        // Update state before closing muxer.  Closing the muxer
-        // may initiate optimization which can take a while and
-        // we want the muxing state to be visible while this is
-        // happening.
-        if( job->pass == 0 || job->pass == 2 )
-        {
-            /* Update the UI */
-            hb_state_t state;
-            state.state = HB_STATE_MUXING;
-            state.param.muxing.progress = 0;
-            hb_set_state( job->h, &state );
-        }
-
-        if( mux->m )
-        {
-            mux->m->end( mux->m );
-            free( mux->m );
-        }
-
-        // we're all done muxing -- print final stats and cleanup.
-        if( job->pass == 0 || job->pass == 2 )
-        {
-            hb_stat_t sb;
-            uint64_t bytes_total, frames_total;
-
-            if (!hb_stat(job->file, &sb))
-            {
-                hb_deep_log( 2, "mux: file size, %"PRId64" bytes", (uint64_t) sb.st_size );
-
-                bytes_total  = 0;
-                frames_total = 0;
-                for( i = 0; i < mux->ntracks; ++i )
-                {
-                    track = mux->track[i];
-                    hb_log( "mux: track %d, %"PRId64" frames, %"PRId64" bytes, %.2f kbps, fifo %d",
-                            i, track->frames, track->bytes,
-                            90000.0 * track->bytes / mux->pts / 125,
-                            track->mf.flen );
-                    if( !i && job->vquality < 0 )
-                    {
-                        /* Video */
-                        hb_deep_log( 2, "mux: video bitrate error, %+"PRId64" bytes",
-                                (int64_t)(track->bytes - mux->pts * job->vbitrate * 125 / 90000) );
-                    }
-                    bytes_total  += track->bytes;
-                    frames_total += track->frames;
-                }
-
-                if( bytes_total && frames_total )
-                {
-                    hb_deep_log( 2, "mux: overhead, %.2f bytes per frame",
-                            (float) ( sb.st_size - bytes_total ) /
-                            frames_total );
-                }
-            }
-        }
-
-        for( i = 0; i < mux->ntracks; ++i )
-        {
-            hb_buffer_t * b;
-            track = mux->track[i];
-            while ( (b = mf_pull( mux, i )) != NULL )
-            {
-                hb_buffer_close( &b );
-            }
-            if( track->mux_data )
-            {
-                free( track->mux_data );
-                free( track->mf.fifo );
-            }
-            free( track );
-        }
-        free(mux->track);
-        hb_unlock( mux->mutex );
-        hb_lock_close( &mux->mutex );
-        hb_bitvec_free(&mux->eof);
-        hb_bitvec_free(&mux->rdy);
-        hb_bitvec_free(&mux->allEof);
-        hb_bitvec_free(&mux->allRdy);
-        free( mux );
+        /* Update the UI */
+        hb_state_t state;
+        state.state = HB_STATE_MUXING;
+        state.param.muxing.progress = 0;
+        hb_set_state( job->h, &state );
     }
-    else
+
+    if( mux->m )
     {
-        hb_unlock( mux->mutex );
+        mux->m->end( mux->m );
+        free( mux->m );
     }
+
+    // we're all done muxing -- print final stats and cleanup.
+    if( job->pass_id == HB_PASS_ENCODE ||
+        job->pass_id == HB_PASS_ENCODE_2ND )
+    {
+        hb_stat_t sb;
+        uint64_t bytes_total, frames_total;
+
+        if (!hb_stat(job->file, &sb))
+        {
+            hb_deep_log( 2, "mux: file size, %"PRId64" bytes", (uint64_t) sb.st_size );
+
+            bytes_total  = 0;
+            frames_total = 0;
+            for( i = 0; i < mux->ntracks; ++i )
+            {
+                track = mux->track[i];
+                hb_log( "mux: track %d, %"PRId64" frames, %"PRId64" bytes, %.2f kbps, fifo %d",
+                        i, track->frames, track->bytes,
+                        90000.0 * track->bytes / mux->pts / 125,
+                        track->mf.flen );
+                if (!i && job->vquality <= HB_INVALID_VIDEO_QUALITY)
+                {
+                    /* Video */
+                    hb_deep_log( 2, "mux: video bitrate error, %+"PRId64" bytes",
+                            (int64_t)(track->bytes - mux->pts * job->vbitrate * 125 / 90000) );
+                }
+                bytes_total  += track->bytes;
+                frames_total += track->frames;
+            }
+
+            if( bytes_total && frames_total )
+            {
+                hb_deep_log( 2, "mux: overhead, %.2f bytes per frame",
+                        (float) ( sb.st_size - bytes_total ) /
+                        frames_total );
+            }
+        }
+    }
+
+    for (i = 0; i < mux->ntracks; ++i)
+    {
+        hb_buffer_t * b;
+        track = mux->track[i];
+        while ( (b = mf_pull( mux, i )) != NULL )
+        {
+            hb_buffer_close( &b );
+        }
+        if( track->mux_data )
+        {
+            free( track->mux_data );
+            free( track->mf.fifo );
+        }
+        free( track );
+    }
+    free(mux->track);
+    hb_unlock( mux->mutex );
+    hb_lock_close( &mux->mutex );
+    hb_bitvec_free(&mux->eof);
+    hb_bitvec_free(&mux->rdy);
+    hb_bitvec_free(&mux->allEof);
+    hb_bitvec_free(&mux->allRdy);
+    free( mux );
+
+    // Close mux work threads
+    while ((w = hb_list_item(pv->list_work, 0)))
+    {
+        hb_list_rem(pv->list_work, w);
+        if (w->thread != NULL)
+        {
+            hb_thread_close( &w->thread );
+        }
+        free(w->private_data);
+        free(w);
+    }
+    hb_list_close(&pv->list_work);
     free( pv );
-    w->private_data = NULL;
+    muxer->private_data = NULL;
 }
 
-static void mux_loop( void * _w )
+static int muxInit( hb_work_object_t * muxer, hb_job_t * job )
 {
-    hb_work_object_t  * w = _w;
-    hb_work_private_t * pv = w->private_data;
-    hb_job_t          * job = pv->job;
-    hb_buffer_t       * buf_in;
+    muxer->private_data = calloc( sizeof( hb_work_private_t ), 1 );
+    hb_work_private_t * pv = muxer->private_data;
 
-    while ( !*job->die && w->status != HB_WORK_DONE )
+    hb_mux_t         * mux = calloc( sizeof( hb_mux_t ), 1 );
+    int                i;
+    hb_work_object_t * w;
+
+    /* Get a real muxer */
+    if( job->pass_id == HB_PASS_ENCODE || job->pass_id == HB_PASS_ENCODE_2ND )
     {
-        buf_in = hb_fifo_get_wait( w->fifo_in );
-        if ( pv->mux->done )
-            break;
-        if ( buf_in == NULL )
-            continue;
-        if ( *job->die )
+        switch( job->mux )
         {
-            if( buf_in )
-            {
-                hb_buffer_close( &buf_in );
-            }
-            break;
-        }
-
-        w->status = w->work( w, &buf_in, NULL );
-        if( buf_in )
-        {
-            hb_buffer_close( &buf_in );
+            case HB_MUX_AV_MP4:
+            case HB_MUX_AV_MKV:
+                mux->m = hb_mux_avformat_init( job );
+                break;
+            default:
+                hb_error( "No muxer selected, exiting" );
+                free(mux);
+                *job->done_error = HB_ERROR_INIT;
+                *job->die = 1;
+                return -1;
         }
     }
-}
 
-hb_work_object_t * hb_muxer_init( hb_job_t * job )
-{
-    int           i;
-    hb_mux_t    * mux = calloc( sizeof( hb_mux_t ), 1 );
-    hb_work_object_t  * w;
-    hb_work_object_t  * muxer;
+    pv->list_work = hb_list_init();
 
     // The bit vectors must be allocated before hb_thread_init for the
     // audio and subtitle muxer jobs below.
@@ -608,24 +638,11 @@ hb_work_object_t * hb_muxer_init( hb_job_t * job )
     // set up to interleave track data in blocks of 1 video frame time.
     // (the best case for buffering and playout latency). The container-
     // specific muxers can reblock this into bigger chunks if necessary.
-    mux->interleave = 90000. * (double)job->vrate_base / (double)job->vrate;
+    mux->interleave = 90000. * (double)job->vrate.den / job->vrate.num;
     mux->pts = mux->interleave;
 
-    /* Get a real muxer */
-    if( job->pass == 0 || job->pass == 2)
+    if( job->pass_id == HB_PASS_ENCODE || job->pass_id == HB_PASS_ENCODE_2ND )
     {
-        switch( job->mux )
-        {
-        case HB_MUX_AV_MP4:
-        case HB_MUX_AV_MKV:
-            mux->m = hb_mux_avformat_init( job );
-            break;
-        default:
-            hb_error( "No muxer selected, exiting" );
-            *job->done_error = HB_ERROR_INIT;
-            *job->die = 1;
-            return NULL;
-        }
         /* Create file, write headers */
         if( mux->m )
         {
@@ -634,60 +651,50 @@ hb_work_object_t * hb_muxer_init( hb_job_t * job )
     }
 
     /* Initialize the work objects that will receive fifo data */
-
-    muxer = hb_get_work( WORK_MUX );
-    muxer->private_data = calloc( sizeof( hb_work_private_t ), 1 );
-    muxer->private_data->job = job;
-    muxer->private_data->mux = mux;
-    mux->ref++;
-    muxer->private_data->track = mux->ntracks;
+    pv->job = job;
+    pv->mux = mux;
+    pv->track = mux->ntracks;
     muxer->fifo_in = job->fifo_mpeg4;
     add_mux_track( mux, job->mux_data, 1 );
-    muxer->done = &muxer->private_data->mux->done;
 
-    for( i = 0; i < hb_list_count( job->list_audio ); i++ )
+    for (i = 0; i < hb_list_count(job->list_audio); i++)
     {
         hb_audio_t  *audio = hb_list_item( job->list_audio, i );
 
-        w = hb_get_work( WORK_MUX );
-        w->private_data = calloc( sizeof( hb_work_private_t ), 1 );
+        w = hb_get_work(job->h, WORK_MUX);
+        w->private_data = calloc(sizeof(hb_work_private_t), 1);
         w->private_data->job = job;
         w->private_data->mux = mux;
-        mux->ref++;
         w->private_data->track = mux->ntracks;
         w->fifo_in = audio->priv.fifo_out;
-        add_mux_track( mux, audio->priv.mux_data, 1 );
-        w->done = &job->done;
-        hb_list_add( job->list_work, w );
-        w->thread = hb_thread_init( w->name, mux_loop, w, HB_NORMAL_PRIORITY );
+        add_mux_track(mux, audio->priv.mux_data, 1);
+        hb_list_add(pv->list_work, w);
     }
 
-    for( i = 0; i < hb_list_count( job->list_subtitle ); i++ )
+    for (i = 0; i < hb_list_count(job->list_subtitle); i++)
     {
         hb_subtitle_t  *subtitle = hb_list_item( job->list_subtitle, i );
 
         if (subtitle->config.dest != PASSTHRUSUB)
             continue;
 
-        w = hb_get_work( WORK_MUX );
-        w->private_data = calloc( sizeof( hb_work_private_t ), 1 );
+        w = hb_get_work(job->h, WORK_MUX);
+        w->private_data = calloc(sizeof(hb_work_private_t), 1);
         w->private_data->job = job;
         w->private_data->mux = mux;
-        mux->ref++;
         w->private_data->track = mux->ntracks;
         w->fifo_in = subtitle->fifo_out;
-        add_mux_track( mux, subtitle->mux_data, 0 );
-        w->done = &job->done;
-        hb_list_add( job->list_work, w );
-        w->thread = hb_thread_init( w->name, mux_loop, w, HB_NORMAL_PRIORITY );
+        add_mux_track(mux, subtitle->mux_data, 0);
+        hb_list_add(pv->list_work, w);
     }
-    return muxer;
-}
 
-// muxInit does nothing because the muxer has a special initializer
-// that takes care of initializing all muxer work objects
-static int muxInit( hb_work_object_t * w, hb_job_t * job )
-{
+    /* Launch processing threads */
+    for (i = 0; i < hb_list_count(pv->list_work); i++)
+    {
+        w = hb_list_item(pv->list_work, i);
+        w->done = muxer->done;
+        w->thread = hb_thread_init(w->name, hb_work_loop, w, HB_LOW_PRIORITY);
+    }
     return 0;
 }
 
@@ -704,20 +711,58 @@ hb_work_object_t hb_muxer =
                      HB_STYLE_FLAG_ITALIC |   \
                      HB_STYLE_FLAG_UNDERLINE)
 
+struct output_buf_s
+{
+    int       alloc;
+    int       size;
+    uint8_t * buf;
+};
+
 typedef struct style_context_s
 {
-    uint8_t             * style_atoms;
+    struct output_buf_s   style_atoms;
     int                   style_atom_count;
     hb_subtitle_style_t   current_style;
     int                   style_start;
+    int                   height;
 } style_context_t;
 
-static void update_style_atoms(style_context_t *ctx, int stop)
+static int check_realloc_output(struct output_buf_s * output, int size)
 {
-    uint8_t *style_entry;
-    uint8_t face = 0;
+    if (output->alloc < size)
+    {
+        uint8_t * tmp;
 
-    style_entry = ctx->style_atoms + 10 + (12 * ctx->style_atom_count);
+        output->alloc = size + 1024;
+        output->size  = size;
+        tmp = realloc(output->buf, output->alloc);
+        if (tmp == NULL)
+        {
+            hb_error("realloc failed!");
+            free(output->buf);
+            output->size = 0;
+            output->alloc = 0;
+            output->buf   = NULL;
+            return 0;
+        }
+        output->buf = tmp;
+    }
+    return 1;
+}
+
+static int update_style_atoms(style_context_t *ctx, int stop)
+{
+    uint8_t * style_entry;
+    uint8_t   face = 0;
+    int       font_size;
+    int       pos  = 10 + (12 * ctx->style_atom_count);
+    int       size = 10 + (12 * (ctx->style_atom_count + 1));
+
+    if (!check_realloc_output(&ctx->style_atoms, size))
+    {
+        return 0;
+    }
+    style_entry = ctx->style_atoms.buf + pos;
 
     if (ctx->current_style.flags & HB_STYLE_FLAG_BOLD)
         face |= 1;
@@ -730,20 +775,31 @@ static void update_style_atoms(style_context_t *ctx, int stop)
     style_entry[1]  = ctx->style_start & 0xff;
     style_entry[2]  = (stop >> 8) & 0xff;               // endChar
     style_entry[3]  = stop & 0xff;
-    style_entry[4]  = 0;    // font-ID msb
-    style_entry[5]  = 1;    // font-ID lsb
-    style_entry[6]  = face; // face-style-flags
-    style_entry[7]  = 24;   // font-size
+    style_entry[4]  = 0;            // font-ID msb
+    style_entry[5]  = 1;            // font-ID lsb
+    style_entry[6]  = face;         // face-style-flags
+    font_size       = 0.05 * ctx->height;
+    if (font_size < 12)
+    {
+        font_size = 12;
+    }
+    else if (font_size > 255)
+    {
+        font_size = 255;
+    }
+    style_entry[7]  = font_size;    // font-size
     style_entry[8]  = (ctx->current_style.fg_rgb >> 16) & 0xff; // r
     style_entry[9]  = (ctx->current_style.fg_rgb >> 8)  & 0xff; // g
     style_entry[10] = (ctx->current_style.fg_rgb)       & 0xff; // b
     style_entry[11] = ctx->current_style.fg_alpha;              // a
 
     ctx->style_atom_count++;
+
+    return 1;
 }
 
-static void update_style(style_context_t *ctx,
-                         hb_subtitle_style_t *style, int pos)
+static int update_style(style_context_t *ctx,
+                        hb_subtitle_style_t *style, int pos)
 {
     if (ctx->style_start < pos)
     {
@@ -752,7 +808,10 @@ static void update_style(style_context_t *ctx,
             ctx->current_style.fg_rgb   != style->fg_rgb               ||
             ctx->current_style.fg_alpha != style->fg_alpha)
         {
-            update_style_atoms(ctx, pos - 1);
+            if (!update_style_atoms(ctx, pos - 1))
+            {
+                return 0;
+            }
             ctx->current_style = *style;
             ctx->style_start = pos;
         }
@@ -762,32 +821,44 @@ static void update_style(style_context_t *ctx,
         ctx->current_style = *style;
         ctx->style_start = pos;
     }
+    return 1;
 }
 
-static void style_context_init(style_context_t *ctx, uint8_t *style_atoms)
+static void style_context_init(style_context_t *ctx)
 {
     memset(ctx, 0, sizeof(*ctx));
-    ctx->style_atoms = style_atoms;
-    ctx->style_start = INT_MAX;
+    ctx->style_atoms.buf   = NULL;
+    ctx->style_atoms.size  = 0;
+    ctx->style_atoms.alloc = 0;
+    ctx->style_start       = INT_MAX;
 }
 
 /*
  * Copy the input to output removing markup and adding markup to the style
  * atom where appropriate.
  */
-void hb_muxmp4_process_subtitle_style(uint8_t *input,
-                                      uint8_t *output,
-                                      uint8_t *style_atoms, uint16_t *stylesize)
+void hb_muxmp4_process_subtitle_style(int        height,
+                                      uint8_t  * input,
+                                      uint8_t ** out_buf,
+                                      uint8_t ** out_style_atoms,
+                                      uint16_t * stylesize)
 {
-    uint16_t utf8_count = 0;         // utf8 count from start of subtitle
-    int consumed, in_pos = 0, out_pos = 0, len, ii;
-    style_context_t ctx;
-    hb_subtitle_style_t style;
-    char *text, *tmp;
+    uint16_t              utf8_count = 0; // utf8 count from start of subtitle
+    int                   consumed, in_pos = 0, out_pos = 0, len, ii;
+    style_context_t       ctx;
+    hb_subtitle_style_t   style;
+    struct output_buf_s   output;
+    char                * text, * tmp;
 
-    *stylesize = 0;
-    style_context_init(&ctx, style_atoms);
+    output.buf       = NULL;
+    output.alloc     = 0;
+    output.size      = 0;
+    *out_buf         = NULL;
+    *out_style_atoms = NULL;
+    *stylesize       = 0;
 
+    style_context_init(&ctx);
+    ctx.height = height;
     hb_ssa_style_init(&style);
 
     // Skip past the SSA preamble
@@ -801,6 +872,11 @@ void hb_muxmp4_process_subtitle_style(uint8_t *input,
     }
     in_pos = text - (char*)input;
 
+    // Always allocate enough for empty string
+    if (!check_realloc_output(&output, 1))
+    {
+        goto fail;
+    }
     while (input[in_pos] != '\0')
     {
         text = hb_ssa_to_text((char*)input + in_pos, &consumed, &style);
@@ -819,32 +895,49 @@ void hb_muxmp4_process_subtitle_style(uint8_t *input,
             }
             len++;
         }
-        strcpy((char*)output+out_pos, text);
+        if (!check_realloc_output(&output, out_pos + len + 1))
+        {
+            goto fail;
+        }
+        strcpy((char*)output.buf + out_pos, text);
         free(text);
         out_pos += len;
         in_pos += consumed;
-        update_style(&ctx, &style, out_pos - utf8_count);
+        if (!update_style(&ctx, &style, out_pos - utf8_count))
+        {
+            goto fail;
+        }
     }
     // Return to default style at end of line, flushes any pending
     // style changes
     hb_ssa_style_init(&style);
-    update_style(&ctx, &style, out_pos - utf8_count);
+    if (!update_style(&ctx, &style, out_pos - utf8_count))
+    {
+        goto fail;
+    }
 
     // null terminate output string
-    output[out_pos] = 0;
+    output.buf[out_pos] = 0;
 
     if (ctx.style_atom_count > 0)
     {
         *stylesize = 10 + (ctx.style_atom_count * 12);
 
-        memcpy(style_atoms + 4, "styl", 4);
+        memcpy(ctx.style_atoms.buf + 4, "styl", 4);
 
-        style_atoms[0] = 0;
-        style_atoms[1] = 0;
-        style_atoms[2] = (*stylesize >> 8) & 0xff;
-        style_atoms[3] = *stylesize & 0xff;
-        style_atoms[8] = (ctx.style_atom_count >> 8) & 0xff;
-        style_atoms[9] = ctx.style_atom_count & 0xff;
+        ctx.style_atoms.buf[0] = 0;
+        ctx.style_atoms.buf[1] = 0;
+        ctx.style_atoms.buf[2] = (*stylesize >> 8) & 0xff;
+        ctx.style_atoms.buf[3] = *stylesize & 0xff;
+        ctx.style_atoms.buf[8] = (ctx.style_atom_count >> 8) & 0xff;
+        ctx.style_atoms.buf[9] = ctx.style_atom_count & 0xff;
+        *out_style_atoms = ctx.style_atoms.buf;
     }
+    *out_buf = output.buf;
+    return;
+
+fail:
+    free(output.buf);
+    free(ctx.style_atoms.buf);
 }
 
