@@ -1,7 +1,8 @@
 ###############################################################################
 ##
-## This script is coded for minimum version of Python 2.4 .
-## Pyhthon3 is incompatible.
+## This script is coded for minimum version of Python 2.7 .
+##
+## Python3 is incompatible.
 ##
 ## Authors: konablend
 ##
@@ -9,16 +10,18 @@
 
 import fnmatch
 import glob
+import json
 import optparse
 import os
 import platform
+import random
 import re
+import string
 import subprocess
 import sys
 import time
 from datetime import datetime, timedelta
 
-from optparse import OptionGroup
 from optparse import OptionGroup
 from optparse import OptionParser
 from sys import stderr
@@ -81,13 +84,13 @@ class Configure( object ):
     def infof( self, format, *args ):
         line = format % args
         self._log_verbose.append( line )
-        if cfg.verbose >= Configure.OUT_INFO:
+        if self.verbose >= Configure.OUT_INFO:
             self._log_info.append( line )
             stdout.write( line )
     def verbosef( self, format, *args ):
         line = format % args
         self._log_verbose.append( line )
-        if cfg.verbose >= Configure.OUT_VERBOSE:
+        if self.verbose >= Configure.OUT_VERBOSE:
             stdout.write( line )
 
     ## doc is ready to be populated
@@ -96,15 +99,19 @@ class Configure( object ):
         self.build_final  = os.curdir
         self.src_final    = self._final_dir( self.build_dir, self.src_dir )
         self.prefix_final = self._final_dir( self.build_dir, self.prefix_dir )
+        if host.match( '*-*-darwin*' ):
+            self.xcode_prefix_final = self._final_dir( self.build_dir, self.xcode_prefix_dir )
 
-        cfg.infof( 'compute: makevar SRC/    = %s\n', self.src_final )
-        cfg.infof( 'compute: makevar BUILD/  = %s\n', self.build_final )
-        cfg.infof( 'compute: makevar PREFIX/ = %s\n', self.prefix_final )
+        self.infof( 'compute: makevar SRC/    = %s\n', self.src_final )
+        self.infof( 'compute: makevar BUILD/  = %s\n', self.build_final )
+        self.infof( 'compute: makevar PREFIX/ = %s\n', self.prefix_final )
+        if host.match( '*-*-darwin*' ):
+            self.infof( 'compute: makevar XCODE.prefix/ = %s\n', self.xcode_prefix_final )
 
     ## perform chdir and enable log recording
     def chdir( self ):
         if os.path.abspath( self.build_dir ) == os.path.abspath( self.src_dir ):
-            cfg.errln( 'build (scratch) directory must not be the same as top-level source root!' )
+            self.errln( 'build (scratch) directory must not be the same as top-level source root!' )
 
         if self.build_dir != os.curdir:
             if os.path.exists( self.build_dir ):
@@ -127,7 +134,7 @@ class Configure( object ):
         dir = os.path.dirname( args[0] )
         if len(args) > 1 and args[1].find('w') != -1:
             self.mkdirs( dir )
-        m = re.match( '^(.*)\.tmp$', args[0] )
+        m = re.match( '^(.*)\.tmp\..{8}$', args[0] )
         if m:
             self.infof( 'write: %s\n', m.group(1) )
         else:
@@ -136,21 +143,21 @@ class Configure( object ):
         try:
             return open( *args )
         except Exception, x:
-            cfg.errln( 'open failure: %s', x )
+            self.errln( 'open failure: %s', x )
 
     def record_log( self ):
         if not self._record:
             return
         self._record = False
         self.verbose = Configure.OUT_QUIET
-        file = cfg.open( 'log/config.info.txt', 'w' )
+        log_info_file = self.open( 'log/config.info.txt', 'w' )
         for line in self._log_info:
-            file.write( line )
-        file.close()
-        file = cfg.open( 'log/config.verbose.txt', 'w' )
+            log_info_file.write( line )
+        log_info_file.close()
+        log_verbose_file = self.open( 'log/config.verbose.txt', 'w' )
         for line in self._log_verbose:
-            file.write( line )
-        file.close()
+            log_verbose_file.write( line )
+        log_verbose_file.close()
 
     ## Find executable by searching path.
     ## On success, returns full pathname of executable.
@@ -161,11 +168,7 @@ class Configure( object ):
                 return name
             return None
 
-        if not os.environ.has_key( 'PATH' ) or os.environ[ 'PATH' ] == '':
-            path = os.defpath
-        else:
-            path = os.environ['PATH']
-
+        path = os.getenv( 'PATH' ) or os.defpath
         for dir in path.split( os.pathsep ):
             f = os.path.join( dir, name )
             if os.access( f, os.X_OK ):
@@ -195,6 +198,8 @@ class Configure( object ):
         self.src_dir    = os.path.normpath( options.src )
         self.build_dir  = os.path.normpath( options.build )
         self.prefix_dir = os.path.normpath( options.prefix )
+        if host.match( '*-*-darwin*' ):
+            self.xcode_prefix_dir = os.path.normpath( options.xcode_prefix )
         if options.sysroot != None:
                 self.sysroot_dir = os.path.normpath( options.sysroot )
         else:
@@ -209,6 +214,10 @@ class Configure( object ):
         if os.path.abspath( self.src_dir ) == os.path.abspath( self.build_dir ):
             self.build_dir = os.path.join( self.build_dir, 'build' )
 
+    ## generate a temporary filename - not worried about race conditions
+    def mktmpname( self, filename ):
+        return filename + '.tmp.' + ''.join(random.choice(string.ascii_lowercase + string.digits) for _ in range(8))
+
 ###############################################################################
 ##
 ## abstract action
@@ -327,9 +336,8 @@ class CCProbe( Action ):
 
     def _action( self ):
         ## write program file
-        file = open( 'conftest.c', 'w' )
-        file.write( self.test_file )
-        file.close()
+        with open( 'conftest.c', 'w' ) as out_file:
+            out_file.write( self.test_file )
         ## pipe and redirect stderr to stdout; effects communicate result
         pipe = subprocess.Popen( '%s -c -o conftest.o conftest.c' % self.command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT )
 
@@ -369,9 +377,8 @@ class LDProbe( Action ):
 
     def _action( self ):
         ## write program file
-        file = open( 'conftest.c', 'w' )
-        file.write( self.test_file )
-        file.close()
+        with open( 'conftest.c', 'w' ) as out_file:
+            out_file.write( self.test_file )
         ## pipe and redirect stderr to stdout; effects communicate result
         pipe = subprocess.Popen( '%s -o conftest conftest.c %s' % (self.command, self.lib), shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT )
 
@@ -416,10 +423,7 @@ class HostTupleProbe( ShellProbe, list ):
         super( HostTupleProbe, self ).__init__( 'host tuple', '%s/config.guess' % (cfg.dir), abort=True, head=True )
 
     def _parseSession( self ):
-        if len(self.session):
-            self.spec = self.session[0]
-        else:
-            self.spec = ''
+        self.spec = self.session[0] if self.session else ''
 
         ## grok GNU host tuples
         m = re.match( HostTupleProbe.GNU_TUPLE_RE, self.spec )
@@ -490,7 +494,7 @@ class BuildAction( Action, list ):
             if self.match( '*mingw*' ):
                 self.systemf = 'MinGW'
             elif self.systemf:
-                self.systemf[0] = self.systemf[0].upper()
+                self.systemf = self.systemf.capitalize()
             self.title = '%s %s' % (build.systemf,self.machine)
         else:
             self.title = '%s %s' % (build.systemf,arch.mode.mode)
@@ -577,6 +581,9 @@ class ArchAction( Action ):
             pass
         elif host.match( '*-*-solaris*' ):
             pass
+        elif host.match( '*-*-freebsd.*' ):
+            self.mode['i386']   = 'i386-portsbuild-freebsd%s' % (host.release)
+            self.mode['amd64'] = 'amd64-portsbuild-freebsd%s' % (host.release)
         else:
             self.msg_pass = 'WARNING'
 
@@ -617,8 +624,8 @@ class CoreProbe( Action ):
         ## clamp
         if self.count < 1:
             self.count = 1
-        elif self.count > 32:
-            self.count = 32
+        elif self.count > 64:
+            self.count = 64
 
         if options.launch:
             if options.launch_jobs == 0:
@@ -700,7 +707,7 @@ class RepoProbe( ShellProbe ):
         self.rev       = 0
         self.hash      = 'deadbeaf'
         self.shorthash = 'deadbea'
-        self.date      = datetime(1, 1, 1)
+        self.date      = None
         self.official  = 0
         self.type      = 'developer'
 
@@ -744,11 +751,10 @@ class RepoProbe( ShellProbe ):
                 self.shorthash = value[:7]
 
         # type-classification via repository URL
-        official_url = 'https://github.com/HandBrake/HandBrake.git' # HTTPS
-        if self.url == 'git@github.com:HandBrake/HandBrake.git':    # SSH
-            self.url = official_url
+        if self.url == project.url_repo_ssh:
+            self.url = project.url_repo  # official repo, SSH to HTTPS
 
-        if self.url == official_url:
+        if self.url == project.url_repo:
             self.official = 1
             if not options.snapshot and self.hash == self.tag_hash:
                 self.type = 'release'
@@ -768,12 +774,11 @@ class RepoProbe( ShellProbe ):
         try:
             hvp = os.path.join( cfg.src_dir, 'version.txt' )
             if os.path.isfile( hvp ) and os.path.getsize( hvp ) > 0:
-                file = open( hvp, 'r' )
-                self.session = file.readlines()
-                file.close()
+                with open( hvp, 'r' ) as in_file:
+                    self.session = in_file.readlines()
                 if self.session:
                     self._parseSession()
-            if self.rev != 0:
+            if self.hash is not empty and self.hash is not 'deadbeaf':
                 cfg.infof( '(pass)\n' )
             else:
                 cfg.infof( '(fail)\n' )
@@ -796,6 +801,8 @@ class Project( Action ):
         self.acro_lower    = 'hb'
         self.acro_upper    = 'HB'
         self.url_website   = 'https://handbrake.fr'
+        self.url_repo      = 'https://github.com/HandBrake/HandBrake.git'
+        self.url_repo_ssh  = 'git@github.com:HandBrake/HandBrake.git'
         self.url_community = 'https://forum.handbrake.fr'
         self.url_irc       = 'irc://irc.freenode.net/handbrake'
 
@@ -816,9 +823,12 @@ class Project( Action ):
         else:
             url_arch = ''
 
-        suffix = ''
+        if repo.date is None:
+            cfg.errln( '%s is missing version information it needs to build properly.\nClone the official git repository at %s\nor download an official source archive from %s\n', self.name, self.url_repo, self.url_website )
+            sys.exit( 1 )
+
         if repo.tag != '':
-            m = re.match( '^([0-9]+)\.([0-9]+)\.([0-9]+)-?(.*)?$', repo.tag )
+            m = re.match( '^([0-9]+)\.([0-9]+)\.([0-9]+)-?(.+)?$', repo.tag )
             if not m:
                 cfg.errln( 'Invalid repo tag format %s\n', repo.tag )
                 sys.exit( 1 )
@@ -826,7 +836,8 @@ class Project( Action ):
             self.vmajor = int(vmajor)
             self.vminor = int(vminor)
             self.vpoint = int(vpoint)
-            self.suffix = suffix
+            if suffix:
+                self.suffix = suffix
 
         if repo.type != 'release' or options.snapshot:
             self.version = repo.date.strftime("%Y%m%d%H%M%S")
@@ -841,10 +852,10 @@ class Project( Action ):
 
             url_ctype = '_unstable'
             url_ntype = 'unstable'
-            self.build = time.strftime('%Y%m%d') + '01'
+            self.build = time.strftime('%Y%m%d', now) + '01'
             self.title = '%s %s (%s)' % (self.name,self.version,self.build)
         else:
-            m = re.match('^([a-zA-Z]+)\.([0-9]+)$', suffix)
+            m = re.match('^([a-zA-Z]+)\.([0-9]+)$', self.suffix)
             if not m:
                 # Regular release
                 self.version = '%d.%d.%d' % (self.vmajor,self.vminor,self.vpoint)
@@ -860,7 +871,7 @@ class Project( Action ):
                 url_ctype = '_unstable'
                 url_ntype = 'unstable'
 
-            self.build = time.strftime('%Y%m%d') + '00'
+            self.build = time.strftime('%Y%m%d', now) + '00'
             self.title = '%s %s (%s)' % (self.name,self.version,self.build)
 
         self.url_appcast = 'https://handbrake.fr/appcast%s%s.xml' % (url_ctype,url_arch)
@@ -956,12 +967,7 @@ class VersionProbe( Action ):
         ## read data into memory buffers
         data = pipe.communicate()
         self.fail = pipe.returncode != 0
-
-        if data[0]:
-            self.session = data[0].splitlines()
-        else:
-            self.session = []
-
+        self.session = data[0].splitlines() if data[0] else []
         self.svers = '0.0.0'
         self.ivers = [0,0,0]
 
@@ -1078,22 +1084,22 @@ class ConfigDocument:
     def __init__( self ):
         self._elements = []
 
-    def _outputMake( self, file, namelen, name, value, append ):
+    def _outputMake( self, out_file, namelen, name, value, append ):
         if append:
             if value == None or len(str(value)) == 0:
-                file.write( '%-*s +=\n' % (namelen, name) )
+                out_file.write( '%-*s +=\n' % (namelen, name) )
             else:
-                file.write( '%-*s += %s\n' % (namelen, name, value) )
+                out_file.write( '%-*s += %s\n' % (namelen, name, value) )
         else:
             if value == None or len(str(value)) == 0:
-                file.write( '%-*s  =\n' % (namelen, name) )
+                out_file.write( '%-*s  =\n' % (namelen, name) )
             else:
-                file.write( '%-*s  = %s\n' % (namelen, name, value) )
+                out_file.write( '%-*s  = %s\n' % (namelen, name, value) )
 
-    def _outputM4( self, file, namelen, name, value ):
+    def _outputM4( self, out_file, namelen, name, value ):
         namelen += 7
         name = '<<__%s>>,' % name.replace( '.', '_' )
-        file.write( 'define(%-*s  <<%s>>)dnl\n' % (namelen, name, value ))
+        out_file.write( 'define(%-*s  <<%s>>)dnl\n' % (namelen, name, value ))
 
     def add( self, name, value, append=False ):
         self._elements.append( [name,value,append] )
@@ -1111,7 +1117,7 @@ class ConfigDocument:
     def addM4( self, line ):
         self._elements.append( ('?m4',line) )
 
-    def output( self, file, type ):
+    def output( self, out_file, type ):
         namelen = 0
         for item in self._elements:
             if item == None or item[0].find( '?' ) == 0:
@@ -1121,19 +1127,19 @@ class ConfigDocument:
         for item in self._elements:
             if item == None:
                 if type == 'm4':
-                    file.write( 'dnl\n' )
+                    out_file.write( 'dnl\n' )
                 else:
-                    file.write( '\n' )
+                    out_file.write( '\n' )
                 continue
             if item[0].find( '?' ) == 0:
                 if item[0].find( type, 1 ) == 1:
-                    file.write( '%s\n' % (item[1]) )
+                    out_file.write( '%s\n' % (item[1]) )
                 continue
 
             if type == 'm4':
-                self._outputM4( file, namelen, item[0], item[1] )
+                self._outputM4( out_file, namelen, item[0], item[1] )
             else:
-                self._outputMake( file, namelen, item[0], item[1], item[2] )
+                self._outputMake( out_file, namelen, item[0], item[1], item[2] )
 
     def update( self, name, value ):
         for item in self._elements:
@@ -1152,14 +1158,14 @@ class ConfigDocument:
         else:
             raise ValueError, 'unknown file type: ' + type
 
-        ftmp  = fname + '.tmp'
+        ftmp = cfg.mktmpname(fname)
         try:
             try:
-                file = cfg.open( ftmp, 'w' )
-                self.output( file, type )
+                out_file = cfg.open( ftmp, 'w' )
+                self.output( out_file, type )
             finally:
                 try:
-                    file.close()
+                    out_file.close()
                 except:
                     pass
         except Exception, x:
@@ -1174,6 +1180,41 @@ class ConfigDocument:
         except Exception, x:
             cfg.errln( 'failed writing to %s\n%s', fname, x )
 
+###############################################################################
+
+def encodeDistfileConfig():
+    fname = 'distfile.cfg'
+    ftmp = cfg.mktmpname(fname)
+    data = {
+        'disable-fetch':  options.disable_df_fetch,
+        'disable-verify': options.disable_df_verify,
+        'jobs':           options.df_jobs,
+        'verbosity':      options.df_verbosity,
+        'accept-url':     options.df_accept_url,
+        'deny-url':       options.df_deny_url,
+    }
+    try:
+        try:
+            out_file = cfg.open( ftmp, 'w' )
+            json.dump(data, out_file)
+            out_file.write('\n')
+        finally:
+            try:
+                out_file.close()
+            except:
+                pass
+    except Exception, x:
+        try:
+            os.remove( ftmp )
+        except Exception, x:
+            pass
+        cfg.errln( 'failed writing to %s\n%s', ftmp, x )
+
+    try:
+        os.rename( ftmp, fname )
+    except Exception, x:
+        cfg.errln( 'failed writing to %s\n%s', fname, x )
+
 ###############################################################################
 ##
 ## create cli parser
@@ -1212,9 +1253,21 @@ def createCLI():
 
     ## add hidden options
     cli.add_option( '--xcode-driver', default='bootstrap', action='store', help=optparse.SUPPRESS_HELP )
+
+    ## add general options
     cli.add_option( '--force', default=False, action='store_true', help='overwrite existing build config' )
     cli.add_option( '--verbose', default=False, action='store_true', help='increase verbosity' )
 
+    ## add distfile options
+    grp = OptionGroup( cli, 'Distfile Options' )
+    grp.add_option( '--disable-df-fetch', default=False, action='store_true', help='disable distfile downloads' )
+    grp.add_option( '--disable-df-verify', default=False, action='store_true', help='disable distfile data verification' )
+    grp.add_option( '--df-jobs', default=1, action='store', metavar='N', type='int', help='allow N distfile downloads at once' )
+    grp.add_option( '--df-verbose', default=1, action='count', dest='df_verbosity', help='increase distfile tools verbosity' )
+    grp.add_option( '--df-accept-url', default=[], action='append', metavar='SPEC', help='accept URLs matching regex pattern' )
+    grp.add_option( '--df-deny-url', default=[], action='append', metavar='SPEC', help='deny URLs matching regex pattern' )
+    cli.add_option_group( grp )
+
     ## add install options
     grp = OptionGroup( cli, 'Directory Locations' )
     h = IfHost( 'specify sysroot of SDK for Xcode builds', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
@@ -1234,29 +1287,32 @@ def createCLI():
     h = IfHost( 'enable assembly code in non-contrib modules', 'NOMATCH*-*-darwin*', 'NOMATCH*-*-linux*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-asm', default=False, action='store_true', help=h )
 
-    h = IfHost( 'disable GTK GUI', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'disable GTK GUI', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--disable-gtk', default=False, action='store_true', help=h )
-    h = IfHost( 'disable GTK GUI update checks', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+
+    h = IfHost( 'disable GTK GUI update checks', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
+
     grp.add_option( '--disable-gtk-update-checks', default=False, action='store_true', help=h )
+
     h = IfHost( 'enable GTK GUI (mingw)', '*-*-mingw*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-gtk-mingw', default=False, action='store_true', help=h )
-    h = IfHost( 'disable gstreamer (live preview)', '*-*-linux*', none=optparse.SUPPRESS_HELP ).value
+
+    h = IfHost( 'disable GStreamer (live preview)', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
+
     grp.add_option( '--disable-gst', default=False, action='store_true', help=h )
-    h = IfHost( 'enable use of Intel Quick Sync Video hardware acceleration', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+
+    h = IfHost( 'enable Intel Quick Sync Video (QSV) hardware acceleration. (Windows and Linux only)', '*-*-linux*', '*-*-mingw*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-qsv', default=False, action='store_true', help=h )
 
-    h = IfHost( 'enable HWD features', '*-*-*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--enable-hwd', default=False, action='store_true', help=h )
-
-    h = IfHost( 'enable use of x265 encoding', '*-*-*', none=optparse.SUPPRESS_HELP ).value
-    grp.add_option( '--enable-x265', default=True, action='store_true', help=h )
+    h = IfHost( 'enable x265 video encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--enable-x265', dest="enable_x265", default=True, action='store_true', help=h )
     grp.add_option( '--disable-x265', dest="enable_x265", action='store_false' )
 
-    h = IfHost( 'enable use of fdk-aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'enable FDK AAC audio encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-fdk-aac', dest="enable_fdk_aac", default=False, action='store_true', help=h )
     grp.add_option( '--disable-fdk-aac', dest="enable_fdk_aac", action='store_false' )
 
-    h = IfHost( 'enable use of libav aac encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
+    h = IfHost( 'enable Libav AAC audio encoder', '*-*-*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-libav-aac', dest="enable_libav_aac", default=not host.match( '*-*-darwin*' ), action='store_true', help=h )
     grp.add_option( '--disable-libav-aac', dest="enable_libav_aac", action='store_false' )
 
@@ -1294,13 +1350,18 @@ def createCLI():
     h = IfHost( 'Build and use local pkg-config', '*-*-darwin*', none=optparse.SUPPRESS_HELP ).value
     grp.add_option( '--enable-local-pkgconfig', default=False, action='store_true', help=h )
 
+    h = IfHost( 'Build extra contribs for flatpak packaging', '*-*-linux*', '*-*-freebsd*', none=optparse.SUPPRESS_HELP ).value
+    grp.add_option( '--flatpak', default=False, action='store_true', help=h )
     cli.add_option_group( grp )
 
+
     ## add Xcode options
     if host.match( '*-*-darwin*' ):
         grp = OptionGroup( cli, 'Xcode Options' )
         grp.add_option( '--disable-xcode', default=False, action='store_true',
             help='disable Xcode' )
+        grp.add_option( '--xcode-prefix', default=cfg.xcode_prefix_dir, action='store', metavar='DIR',
+            help='specify install dir for Xcode products [%s]' % (cfg.xcode_prefix_dir) )
         grp.add_option( '--xcode-symroot', default='xroot', action='store', metavar='DIR',
             help='specify root of the directory hierarchy that contains product files and intermediate build files' )
         xcconfigMode.cli_add_option( grp, '--xcode-config' )
@@ -1429,6 +1490,8 @@ try:
         if arg == '--verbose':
             verbose = Configure.OUT_VERBOSE
 
+    now = time.gmtime(int(os.environ.get('SOURCE_DATE_EPOCH', time.time())))
+
     ## create main objects; actions/probes run() is delayed.
     ## if any actions must be run earlier (eg: for configure --help purposes)
     ## then run() must be invoked earlier. subequent run() invocations
@@ -1436,7 +1499,9 @@ try:
     cfg   = Configure( verbose )
     host  = HostTupleProbe(); host.run()
 
-    cfg.prefix_dir = ForHost( '/usr/local', ['/Applications','*-*-darwin*'] ).value
+    cfg.prefix_dir = '/usr/local'
+    if host.match( '*-*-darwin*' ):
+        cfg.xcode_prefix_dir = '/Applications'
 
     build = BuildAction()
     arch  = ArchAction(); arch.run()
@@ -1450,7 +1515,6 @@ try:
     class Tools:
         ar    = ToolProbe( 'AR.exe',    'ar' )
         cp    = ToolProbe( 'CP.exe',    'cp' )
-        curl  = ToolProbe( 'CURL.exe',  'curl', abort=False )
         gcc   = ToolProbe( 'GCC.gcc',   'gcc', IfHost( 'gcc-4', '*-*-cygwin*' ))
 
         if host.match( '*-*-darwin*' ):
@@ -1458,14 +1522,14 @@ try:
         else:
             gmake = ToolProbe( 'GMAKE.exe', 'gmake', 'make' )
 
-        m4       = ToolProbe( 'M4.exe',       'm4' )
+        m4       = ToolProbe( 'M4.exe',       'gm4', 'm4' )
         mkdir    = ToolProbe( 'MKDIR.exe',    'mkdir' )
         patch    = ToolProbe( 'PATCH.exe',    'gpatch', 'patch' )
         rm       = ToolProbe( 'RM.exe',       'rm' )
         ranlib   = ToolProbe( 'RANLIB.exe',   'ranlib' )
         strip    = ToolProbe( 'STRIP.exe',    'strip' )
         tar      = ToolProbe( 'TAR.exe',      'gtar', 'tar' )
-        wget     = ToolProbe( 'WGET.exe',     'wget', abort=False )
+        nasm     = ToolProbe( 'NASM.exe',     'nasm', abort=False, minversion=[2,13,0] )
         yasm     = ToolProbe( 'YASM.exe',     'yasm', abort=False, minversion=[1,2,0] )
         autoconf = ToolProbe( 'AUTOCONF.exe', 'autoconf', abort=False )
         automake = ToolProbe( 'AUTOMAKE.exe', 'automake', abort=False )
@@ -1476,8 +1540,6 @@ try:
         xcodebuild = ToolProbe( 'XCODEBUILD.exe', 'xcodebuild', abort=False )
         lipo       = ToolProbe( 'LIPO.exe',       'lipo', abort=False )
 
-        fetch = SelectTool( 'FETCH.select', 'fetch', ['wget',wget], ['curl',curl] )
-
     ## run tool probes
     for tool in ToolProbe.tools:
         tool.run()
@@ -1528,6 +1590,13 @@ try:
     for action in Action.actions:
         action.run()
 
+    ## fail on missing or old nasm where needed
+    if host.match( '*-*-darwin*' ) or options.cross:
+        if Tools.nasm.fail:
+            raise AbortError( 'error: nasm missing\n' )
+        elif Tools.nasm.version.inadequate():
+            raise AbortError( 'error: minimum required nasm version is %s and %s is %s\n' % ('.'.join([str(i) for i in Tools.nasm.version.minversion]),Tools.nasm.pathname,Tools.nasm.version.svers) )
+
     ## enable local yasm when yasm probe fails or version is too old
     ## x264 requires 1.2.0+
     if not options.enable_local_yasm:
@@ -1589,7 +1658,8 @@ int main ()
   return 0;
 }
 """
-        pthread = LDProbe( 'static pthread', '%s -static' % Tools.gcc.pathname, '-lpthreadGC2', pthread_test )
+        pthreadGC2 = LDProbe( 'static pthread', '%s -static' % Tools.gcc.pathname, '-lpthreadGC2', pthread_test )
+        pthread = LDProbe( 'static pthread', '%s -static' % Tools.gcc.pathname, '-lpthread', pthread_test )
         pthread.run()
 
         bz2_test = """
@@ -1643,7 +1713,7 @@ int main()
     int     rv;
     regex_t exp;
 
-    rv = regcomp(&exp, "^[0-9]+$";", REG_EXTENDED);
+    rv = regcomp(&exp, "^[0-9]+$", REG_EXTENDED);
     if (rv != 0) {
         return 1;
     }
@@ -1664,6 +1734,36 @@ int main()
         regex = LDProbe( 'static regex', '%s -static' % Tools.gcc.pathname, '-lregex', regex_test )
         regex.run()
 
+        strtok_r_test = """
+#include <string.h>
+
+int main ()
+{
+  char *saveptr;
+  strtok_r("String tok string", "tok", &saveptr);
+  return 0;
+}
+"""
+        strtok_r = LDProbe( 'static strtok_r', '%s -static' % Tools.gcc.pathname, '', strtok_r_test )
+        strtok_r.run()
+
+    strerror_r_test = """
+#include <string.h>
+
+int main()
+{
+    /* some implementations fail if buf is less than 80 characters
+       so size it appropriately */
+    char errstr[128];
+    /* some implementations fail if err == 0 */
+    strerror_r(1, errstr, 127);
+    return 0;
+}
+"""
+
+    strerror_r = LDProbe( 'strerror_r', '%s' % Tools.gcc.pathname, '', strerror_r_test )
+    strerror_r.run()
+
     ## cfg hook before doc prep
     cfg.doc_ready()
 
@@ -1676,7 +1776,7 @@ int main()
     args = []
     for arg in Option.conf_args:
         args.append( arg[1] )
-    doc.add( 'CONF.args', ' '.join( args ))
+    doc.add( 'CONF.args', ' '.join(args).replace('$','$$') )
 
     doc.addBlank()
     doc.add( 'HB.title',       project.title )
@@ -1742,7 +1842,7 @@ int main()
     else:
         doc.add( 'BUILD.cross.prefix', '' )
 
-    doc.add( 'BUILD.date',   time.strftime('%c') )
+    doc.add( 'BUILD.date',   time.strftime('%c', now) ),
     doc.add( 'BUILD.arch',   arch.mode.mode )
 
     doc.addBlank()
@@ -1759,19 +1859,21 @@ int main()
     doc.add( 'FEATURE.local_cmake', int( options.enable_local_cmake ))
     doc.add( 'FEATURE.local_pkgconfig', int( options.enable_local_pkgconfig ))
     doc.add( 'FEATURE.asm',        'disabled' )
+    doc.add( 'FEATURE.flatpak',    int( options.flatpak ))
     doc.add( 'FEATURE.gtk',        int( not options.disable_gtk ))
     doc.add( 'FEATURE.gtk.update.checks', int( not options.disable_gtk_update_checks ))
     doc.add( 'FEATURE.gtk.mingw',  int( options.enable_gtk_mingw ))
     doc.add( 'FEATURE.gst',        int( not options.disable_gst ))
     doc.add( 'FEATURE.fdk_aac',    int( options.enable_fdk_aac ))
-    doc.add( 'FEATURE.libav_aac',  int( options.enable_libav_aac ))
+    doc.add( 'FEATURE.libav_aac',  int( options.enable_libav_aac or build.system == 'mingw' ))
     doc.add( 'FEATURE.qsv',        int( options.enable_qsv ))
-    doc.add( 'FEATURE.hwd',        int( options.enable_hwd ))
     doc.add( 'FEATURE.xcode',      int( not (Tools.xcodebuild.fail or options.disable_xcode or options.cross) ))
     doc.add( 'FEATURE.x265',       int( options.enable_x265 ))
 
     if not Tools.xcodebuild.fail and not options.disable_xcode:
         doc.addBlank()
+        doc.add( 'XCODE.prefix',  cfg.xcode_prefix_final )
+        doc.add( 'XCODE.prefix/', cfg.xcode_prefix_final + os.sep )
         doc.add( 'XCODE.driver', options.xcode_driver )
         if os.path.isabs(options.xcode_symroot):
             doc.add( 'XCODE.symroot', options.xcode_symroot )
@@ -1783,7 +1885,9 @@ int main()
         doc.addBlank()
         if not dlfcn.fail:
             doc.add( 'HAS.dlfcn', 1 )
-        if not pthread.fail:
+        if not pthreadGC2.fail:
+            doc.add( 'HAS.pthreadGC2', 1 )
+        elif not pthread.fail:
             doc.add( 'HAS.pthread', 1 )
         if not bz2.fail:
             doc.add( 'HAS.bz2', 1 )
@@ -1793,6 +1897,15 @@ int main()
             doc.add( 'HAS.iconv', 1 )
         if not regex.fail:
             doc.add( 'HAS.regex', 1 )
+        if strtok_r.fail:
+            doc.add( 'COMPAT.strtok_r', 1 )
+
+    else:
+        doc.addBlank()
+        if build.system == 'freebsd':
+            doc.add( 'HAS.pthread', 1 )
+        if not strerror_r.fail:
+            doc.add( 'HAS.strerror_r', 1 )
 
     doc.addMake( '' )
     doc.addMake( '## define debug mode and optimize before other includes' )
@@ -1812,12 +1925,11 @@ int main()
         select.doc_add( doc )
 
     doc.addBlank()
+    doc.add( 'GCC.archs', arch.mode.mode )
     if build.match( '*-*-darwin*' ):
-        doc.add( 'GCC.archs', arch.mode.mode )
         doc.add( 'GCC.sysroot', cfg.sysroot_dir )
         doc.add( 'GCC.minver', cfg.minver )
     else:
-        doc.add( 'GCC.archs', '' )
         doc.add( 'GCC.sysroot', '' )
         doc.add( 'GCC.minver', '' )
 
@@ -1825,6 +1937,8 @@ int main()
         doc.add( 'LIBHB.GCC.D', 'ARCH_X86_32', append=True )
     elif build.match( 'x86_64-*' ):
         doc.add( 'LIBHB.GCC.D', 'ARCH_X86_64', append=True )
+    elif build.match( 'amd64-*' ):
+        doc.add( 'LIBHB.GCC.D', 'ARCH_X86_64', append=True )
 
     if options.enable_asm and ( not Tools.yasm.fail or options.enable_local_yasm ):
         asm = ''
@@ -1837,7 +1951,7 @@ int main()
             else:
                 doc.add( 'LIBHB.YASM.f', 'elf32' )
             doc.add( 'LIBHB.YASM.m', 'x86' )
-        elif build.match( 'x86_64-*' ):
+        elif build.match( 'x86_64-*' ) or build.match( 'amd64-*' ):
             asm = 'x86'
             doc.add( 'LIBHB.GCC.D', 'HAVE_MMX ARCH_X86_64', append=True )
             if build.match( '*-*-darwin*' ):
@@ -1873,6 +1987,8 @@ int main()
     ## perform
     doc.write( 'make' )
     doc.write( 'm4' )
+    encodeDistfileConfig()
+
     if options.launch:
         Launcher( targets )
 
